@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -38,7 +39,7 @@
 
 #include <directfb.h>
 
-#include <core/fusion/shmalloc.h>
+#include <fusion/shmalloc.h>
 
 #include <core/coredefs.h>
 #include <core/coretypes.h>
@@ -47,8 +48,11 @@
 #include <core/gfxcard.h>
 #include <core/windows.h>
 #include <core/layers.h>
+#include <core/screens.h>
 #include <core/surfaces.h>
 
+#include <core/accelerators.h>
+
 #include <gfx/convert.h>
 
 #include <core/graphics_driver.h>
@@ -92,7 +96,7 @@ static void cyber5kEngineSync( void *drv, void *dev )
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     
+
      cyber_waitidle( cdrv, cdev );
 }
 
@@ -116,7 +120,7 @@ static void cyber5kCheckState( void *drv, void *dev,
                return;
 
           state->accel |= CYBER5K_DRAWING_FUNCTIONS;
-     
+
           /* no line drawing in 24bit mode */
           if (state->destination->format == DSPF_RGB24)
                state->accel &= ~DFXL_DRAWLINE;
@@ -172,10 +176,10 @@ cyber5k_validate_dst( CyberDriverData *cdrv, CyberDeviceData *cdev,
                cyber_out8( cdrv->mmio_base, COPFMT, 3 );
                break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
-     
+
      cdev->v_dst = 1;
 }
 
@@ -185,7 +189,7 @@ cyber5k_validate_src( CyberDriverData *cdrv,
 {
      CoreSurface   *source = state->source;
      SurfaceBuffer *buffer = source->front_buffer;
-     
+
      if (cdev->v_src)
           return;
 
@@ -196,7 +200,7 @@ cyber5k_validate_src( CyberDriverData *cdrv,
 
      cyber_out16( cdrv->mmio_base, SRC1WIDTH,
                   buffer->video.pitch /DFB_BYTES_PER_PIXEL(source->format) - 1);
-     
+
      cdev->v_src = 1;
 }
 
@@ -205,7 +209,7 @@ cyber5k_validate_color( CyberDriverData *cdrv,
                         CyberDeviceData *cdev, CardState *state )
 {
      __u32 fill_color = 0;
-     
+
      if (cdev->v_color)
           return;
 
@@ -228,14 +232,14 @@ cyber5k_validate_color( CyberDriverData *cdrv,
                                         state->color.b );
                break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
 
      cyber_out32( cdrv->mmio_base, FCOLOR, fill_color );
-     
+
      cdev->v_src_colorkey = 0;
-     
+
      cdev->v_color = 1;
 }
 
@@ -248,7 +252,7 @@ cyber5k_validate_src_colorkey( CyberDriverData *cdrv,
 
      cyber_out32( cdrv->mmio_base, FCOLOR, state->src_colorkey );
      cyber_out32( cdrv->mmio_base, BCOLOR, state->src_colorkey );
-     
+
      cdev->v_color = 0;
 
      cdev->v_src_colorkey = 1;
@@ -265,7 +269,7 @@ cyber5k_validate_blitting_cmd( CyberDriverData *cdrv,
 
      if (state->blittingflags & DSBLIT_SRC_COLORKEY)
           cdev->blitting_cmd |= TRANS_ENABLE | TRANS_IS_SRC1 | TRANS_INVERT;
-     
+
      cdev->v_blitting_cmd = 1;
 }
 
@@ -274,7 +278,7 @@ static void cyber5kSetState( void *drv, void *dev, GraphicsDeviceFuncs *funcs,
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     
+
      if (state->modified) {
           if (state->modified & SMF_DESTINATION)
                cdev->v_dst = cdev->v_color = 0;
@@ -291,7 +295,7 @@ static void cyber5kSetState( void *drv, void *dev, GraphicsDeviceFuncs *funcs,
      }
 
      cyber5k_validate_dst( cdrv, cdev, state, funcs );
-     
+
      switch (accel) {
           case DFXL_DRAWLINE:
           case DFXL_DRAWRECTANGLE:
@@ -312,7 +316,7 @@ static void cyber5kSetState( void *drv, void *dev, GraphicsDeviceFuncs *funcs,
                break;
 
           default:
-               BUG( "unexpected drawing/blitting function!" );
+               D_BUG( "unexpected drawing/blitting function!" );
                break;
      }
 
@@ -324,7 +328,7 @@ static bool cyber5kFillRectangle( void *drv, void *dev, DFBRectangle *rect )
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
      volatile __u8   *mmio = cdrv->mmio_base;
-     
+
      cyber_waitidle( cdrv, cdev );
 
      cyber_out32( mmio, DSTPTR, cdev->dst_pixeloffset +
@@ -342,7 +346,7 @@ static bool cyber5kFillRectangle24( void *drv, void *dev, DFBRectangle *rect )
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
      volatile __u8   *mmio = cdrv->mmio_base;
-     
+
      cyber_waitidle( cdrv, cdev );
 
      cyber_out32( mmio, DSTPTR, (cdev->dst_pixeloffset +
@@ -361,7 +365,7 @@ static bool cyber5kDrawRectangle( void *drv, void *dev, DFBRectangle *rect )
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
      volatile __u8   *mmio = cdrv->mmio_base;
-     
+
      __u32 dst = cdev->dst_pixeloffset +
                  rect->y * cdev->dst_pixelpitch + rect->x;
 
@@ -393,7 +397,7 @@ static bool cyber5kDrawRectangle24( void *drv, void *dev, DFBRectangle *rect )
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
      volatile __u8   *mmio = cdrv->mmio_base;
-     
+
      __u32 dst = cdev->dst_pixeloffset +
                 (rect->y * cdev->dst_pixelpitch + rect->x) * 3;
 
@@ -427,7 +431,7 @@ static bool cyber5kDrawLine( void *drv, void *dev, DFBRegion *line )
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
      volatile __u8   *mmio = cdrv->mmio_base;
-     
+
      __u32 cmd = COP_LINE_DRAW | PAT_FIXFGD;
 
      int dx;
@@ -471,7 +475,7 @@ static bool cyber5kBlit( void *drv, void *dev, DFBRectangle *rect, int dx, int d
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
      volatile __u8   *mmio = cdrv->mmio_base;
-     
+
      __u32 cmd = cdev->blitting_cmd;
 
      cyber_waitidle( cdrv, cdev );
@@ -506,7 +510,7 @@ static bool cyber5kBlit24( void *drv, void *dev, DFBRectangle *rect, int dx, int
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
      volatile __u8   *mmio = cdrv->mmio_base;
-     
+
      __u32 cmd = cdev->blitting_cmd;
      __u32 src = 0;
      __u32 dst = 0;
@@ -549,73 +553,72 @@ DisplayLayerFuncs  oldPrimaryFuncs;
 void              *oldPrimaryDriverData;
 
 static DFBResult
-osdInitLayer( GraphicsDevice             *device,
-              DisplayLayer               *layer,
-              DisplayLayerInfo           *layer_info,
-              DFBDisplayLayerConfig      *default_config,
-              DFBColorAdjustment         *default_adj,
+osdInitLayer( CoreLayer                  *layer,
               void                       *driver_data,
-              void                       *layer_data )
+              void                       *layer_data,
+              DFBDisplayLayerDescription *description,
+              DFBDisplayLayerConfig      *config,
+              DFBColorAdjustment         *adjustment )
 {
      DFBResult ret;
 
      /* call the original initialization function first */
-     ret = oldPrimaryFuncs.InitLayer( device, layer, layer_info,
-                                      default_config, default_adj,
-                                      oldPrimaryDriverData, layer_data );
+     ret = oldPrimaryFuncs.InitLayer( layer,
+                                      oldPrimaryDriverData,
+                                      layer_data, description,
+                                      config, adjustment );
      if (ret)
           return ret;
 
      /* set name */
-     snprintf(layer_info->desc.name,
+     snprintf(description->name,
               DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "CyberPro OSD");
 
      /* add support for options */
-     default_config->flags   |= DLCONF_OPTIONS;
-     default_config->options  = DLOP_SRC_COLORKEY | DLOP_OPACITY;
+     config->flags |= DLCONF_OPTIONS;
 
      /* add some capabilities */
-     layer_info->desc.caps |= DLCAPS_ALPHACHANNEL |
-                              DLCAPS_OPACITY | DLCAPS_SRC_COLORKEY;
+     description->caps |= DLCAPS_ALPHACHANNEL |
+                          DLCAPS_OPACITY | DLCAPS_SRC_COLORKEY;
 
      return DFB_OK;
 }
 
 static DFBResult
-osdTestConfiguration ( DisplayLayer               *layer,
-                       void                       *driver_data,
-                       void                       *layer_data,
-                       DFBDisplayLayerConfig      *config,
-                       DFBDisplayLayerConfigFlags *failed )
+osdTestRegion( CoreLayer                  *layer,
+               void                       *driver_data,
+               void                       *layer_data,
+               CoreLayerRegionConfig      *config,
+               CoreLayerRegionConfigFlags *failed )
 {
      DFBResult                  ret;
-     DFBDisplayLayerConfigFlags fail = 0;
-     DFBDisplayLayerConfigFlags options = config->flags & DLCONF_OPTIONS;
-       
-     /* remove flag before calling the original function */
-     config->flags &= ~DLCONF_OPTIONS;
+     CoreLayerRegionConfigFlags fail = 0;
+     DFBDisplayLayerOptions     options = config->options;
+
+     /* remove options before calling the original function */
+     config->options = DLOP_NONE;
 
      /* call the original function */
-     ret = oldPrimaryFuncs.TestConfiguration( layer, oldPrimaryDriverData,
-                                              layer_data, config, &fail );
+     ret = oldPrimaryFuncs.TestRegion( layer, oldPrimaryDriverData,
+                                       layer_data, config, &fail );
 
      /* check options if specified */
      if (options) {
           /* any unsupported option wanted? */
-          if (config->options & ~OSD_OPTIONS)
-               fail |= DLCONF_OPTIONS;
+          if (options & ~OSD_OPTIONS)
+               fail |= CLRCF_OPTIONS;
 
           /* opacity and alpha channel cannot be used at once */
-          if ((config->options & (DLOP_OPACITY | DLOP_ALPHACHANNEL)) ==
+          if ((options & (DLOP_OPACITY | DLOP_ALPHACHANNEL)) ==
               (DLOP_OPACITY | DLOP_ALPHACHANNEL))
           {
-               fail |= DLCONF_OPTIONS;
+               fail |= CLRCF_OPTIONS;
           }
      }
-     
-     /* readd flag */
-     config->flags |= options;
-     
+
+     /* restore options */
+     config->options = options;
+
      if (failed)
           *failed = fail;
 
@@ -626,65 +629,52 @@ osdTestConfiguration ( DisplayLayer               *layer,
 }
 
 static DFBResult
-osdSetConfiguration  ( DisplayLayer               *layer,
-                       void                       *driver_data,
-                       void                       *layer_data,
-                       DFBDisplayLayerConfig      *config )
+osdSetRegion( CoreLayer                  *layer,
+              void                       *driver_data,
+              void                       *layer_data,
+              void                       *region_data,
+              CoreLayerRegionConfig      *config,
+              CoreLayerRegionConfigFlags  updated,
+              CoreSurface                *surface,
+              CorePalette                *palette )
 {
      DFBResult ret;
 
      /* call the original function */
-     ret = oldPrimaryFuncs.SetConfiguration( layer, oldPrimaryDriverData,
-                                             layer_data, config );
+     ret = oldPrimaryFuncs.SetRegion( layer, oldPrimaryDriverData,
+                                      layer_data, region_data,
+                                      config, updated, surface,
+                                      palette );
      if (ret)
           return ret;
-     
+
      /* select pixel based or global alpha */
      if (config->options & DLOP_ALPHACHANNEL)
           cyber_select_alpha_src( ALPHA_GRAPHICS );
      else
           cyber_select_alpha_src( ALPHA_REGISTER );
 
+     cyber_set_alpha_reg( config->opacity,
+                          config->opacity,
+                          config->opacity );
+
      /* source color keying */
      cyber_select_RAM_addr( RAM_CPU );
      cyber_set_alpha_RAM_reg( 0, 0x00, 0x00, 0x00 );
      cyber_select_magic_alpha_src( ALPHA_LOOKUP );
      cyber_enable_magic_alpha_blend( config->options & DLOP_SRC_COLORKEY );
-          
+
      /* FIXME: hardcoded black color key */
      cyber_set_magic_match_reg( 0, 0, 0 );
 
      return DFB_OK;
 }
 
-static DFBResult
-osdSetOpacity        ( DisplayLayer               *layer,
-                       void                       *driver_data,
-                       void                       *layer_data,
-                       __u8                        opacity )
-{
-     cyber_set_alpha_reg( opacity, opacity, opacity );
-     
-     return DFB_OK;
-}
-     
-static DFBResult
-osdSetSrcColorKey    ( DisplayLayer               *layer,
-                       void                       *driver_data,
-                       void                       *layer_data,
-                       __u8                        r,
-                       __u8                        g,
-                       __u8                        b )
-{
-     return DFB_UNIMPLEMENTED;
-}
-
 DisplayLayerFuncs newPrimaryFuncs = {
      InitLayer:          osdInitLayer,
-     TestConfiguration:  osdTestConfiguration,
-     SetConfiguration:   osdSetConfiguration,
-     SetOpacity:         osdSetOpacity,
-     SetSrcColorKey:     osdSetSrcColorKey
+
+     TestRegion:         osdTestRegion,
+     SetRegion:          osdSetRegion
 };
 
 /* exported symbols */
@@ -716,10 +706,10 @@ driver_get_info( GraphicsDevice     *device,
 
      snprintf( info->vendor,
                DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               "directfb.org" );
 
      info->version.major = 0;
-     info->version.minor = 3;
+     info->version.minor = 4;
 
      info->driver_data_size = sizeof (CyberDriverData);
      info->device_data_size = sizeof (CyberDeviceData);
@@ -754,14 +744,15 @@ driver_init_driver( GraphicsDevice      *device,
      /* install primary layer hooks */
      dfb_layers_hook_primary( device, driver_data, &newPrimaryFuncs,
                               &oldPrimaryFuncs, &oldPrimaryDriverData );
-     
+
      /* add the video underlay */
      switch (dfb_gfxcard_get_accelerator( device )) {
           case FB_ACCEL_IGS_CYBER5000:
 #ifdef FB_ACCEL_IGS_CYBER5K
           case FB_ACCEL_IGS_CYBER5K:          /* CyberPro 5xxx */
 #endif
-               dfb_layers_register( device, driver_data, &cyberUnderlayFuncs );
+               dfb_layers_register( dfb_screens_at(DSCID_PRIMARY),
+                                    driver_data, &cyberUnderlayFuncs );
      }
 
      return DFB_OK;
