@@ -44,6 +44,7 @@
 
 typedef struct {
      CoreLayerRegionConfig config;
+     bool                  visible;
 
      /* overlay registers */
      struct {
@@ -57,22 +58,34 @@ typedef struct {
           __u32 overlay_SCALE_INC;
           __u32 overlay_SCALE_CNTL;
           __u32 scaler_HEIGHT_WIDTH;
-          __u32 scaler_BUF0_OFFSET;
           __u32 scaler_BUF_PITCH;
+          __u32 scaler_BUF0_OFFSET;
+          __u32 scaler_BUF1_OFFSET;
           __u32 scaler_BUF0_OFFSET_U;
           __u32 scaler_BUF0_OFFSET_V;
+          __u32 scaler_BUF1_OFFSET_U;
+          __u32 scaler_BUF1_OFFSET_V;
           __u32 video_FORMAT;
+          __u32 capture_CONFIG;
      } regs;
 } Mach64OverlayLayerData;
 
+static void ov_reset( Mach64DriverData *mdrv );
 static void ov_set_regs( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
 static void ov_calc_regs( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov,
-                           CoreLayerRegionConfig *config, CoreSurface *surface );
+                          CoreLayerRegionConfig *config, CoreSurface *surface );
 static void ov_set_buffer( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
 static void ov_calc_buffer( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov,
                             CoreLayerRegionConfig *config, CoreSurface *surface );
+static void ov_set_colorkey( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
+static void ov_calc_colorkey( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov,
+                              CoreLayerRegionConfig *config );
+static void ov_set_opacity( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
+static void ov_calc_opacity( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov,
+                             CoreLayerRegionConfig *config );
+static void ov_set_field( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
 
-#define OV_SUPPORTED_OPTIONS   (DLOP_SRC_COLORKEY | DLOP_DST_COLORKEY)
+#define OV_SUPPORTED_OPTIONS   (DLOP_SRC_COLORKEY | DLOP_DST_COLORKEY | DLOP_DEINTERLACING)
 
 /**********************/
 
@@ -92,12 +105,11 @@ ovInitLayer( CoreLayer                  *layer,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
 
      /* set capabilities and type */
      description->caps = DLCAPS_SCREEN_LOCATION | DLCAPS_SURFACE |
-                         DLCAPS_BRIGHTNESS | DLCAPS_SATURATION |
-                         DLCAPS_SRC_COLORKEY | DLCAPS_DST_COLORKEY;
+                         DLCAPS_SRC_COLORKEY | DLCAPS_DST_COLORKEY |
+                         DLCAPS_DEINTERLACING;
      description->type = DLTF_VIDEO | DLTF_STILL_PICTURE;
 
      /* set name */
@@ -108,35 +120,24 @@ ovInitLayer( CoreLayer                  *layer,
      config->flags       = DLCONF_WIDTH | DLCONF_HEIGHT |
                            DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE |
                            DLCONF_OPTIONS;
-     config->width       = 640;
-     config->height      = 480;
+     config->width       = (mdev->chip >= CHIP_264VT3) ? 640 : 320;
+     config->height      = (mdev->chip >= CHIP_264VT3) ? 480 : 240;
      config->pixelformat = DSPF_YUY2;
      config->buffermode  = DLBM_FRONTONLY;
      config->options     = DLOP_NONE;
 
-     /* fill out default color adjustment,
-        only fields set in flags will be accepted from applications */
-     if (mdev->rage_pro) {
-          adjustment->flags      = DCAF_BRIGHTNESS | DCAF_SATURATION;
-          adjustment->brightness = 0x8000;
-          adjustment->saturation = 0x8000;
-     } else
-          adjustment->flags      = DCAF_NONE;
+     adjustment->flags   = DCAF_NONE;
 
-     /* reset overlay */
-     if (mdev->rage_pro) {
-          mach64_waitfifo( mdrv, mdev, 6 );
-          mach64_out32( mmio, SCALER_H_COEFF0, 0x00002000 );
-          mach64_out32( mmio, SCALER_H_COEFF1, 0x0D06200D );
-          mach64_out32( mmio, SCALER_H_COEFF2, 0x0D0A1C0D );
-          mach64_out32( mmio, SCALER_H_COEFF3, 0x0C0E1A0C );
-          mach64_out32( mmio, SCALER_H_COEFF4, 0x0C14140C );
-          mach64_out32( mmio, SCALER_COLOR_CNTL, 0x00101000 );
+     if (mdev->chip >= CHIP_3D_RAGE_PRO) {
+          description->caps      |= DLCAPS_BRIGHTNESS | DLCAPS_SATURATION;
+
+          /* fill out default color adjustment,
+             only fields set in flags will be accepted from applications */
+          adjustment->flags      |= DCAF_BRIGHTNESS | DCAF_SATURATION;
+          adjustment->brightness  = 0x8000;
+          adjustment->saturation  = 0x8000;
      }
 
-     mach64_waitfifo( mdrv, mdev, 1 );
-     mach64_out32( mmio, OVERLAY_SCALE_CNTL, 0 );
-
      return DFB_OK;
 }
 
@@ -150,7 +151,45 @@ ovTestRegion( CoreLayer                  *layer,
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
      Mach64DeviceData *mdev = mdrv->device_data;
      CoreLayerRegionConfigFlags fail = 0;
-     int max_width = mdev->rage_pro ? 768 : 720;
+     int max_width, max_height;
+
+     switch (mdev->chip) {
+          case CHIP_264VT: /* not verified */
+          case CHIP_3D_RAGE: /* not verified */
+               max_width = 384;
+               max_height = 1024;
+               break;
+          case CHIP_264VT3: /* not verified */
+          case CHIP_3D_RAGE_II: /* not verified */
+          case CHIP_3D_RAGE_IIPLUS:
+               max_width = 720;
+               max_height = 1024;
+               break;
+          case CHIP_264VT4: /* not verified */
+          case CHIP_3D_RAGE_IIC:
+               max_width = 720;
+               max_height = 2048;
+               break;
+          case CHIP_3D_RAGE_PRO: /* not verified */
+               max_width = 768;
+               max_height = 2048;
+               break;
+          case CHIP_3D_RAGE_LT_PRO:
+               max_width = 768;
+               max_height = 1024;
+               break;
+          case CHIP_3D_RAGE_XLXC:
+               max_width = 720;
+               max_height = 2048;
+               break;
+          case CHIP_3D_RAGE_MOBILITY:
+               max_width = 720;
+               max_height = 1024;
+               break;
+          default:
+               D_BUG( "unknown chip" );
+               return DFB_UNSUPPORTED;
+     }
 
      /* check for unsupported options */
      if (config->options & ~OV_SUPPORTED_OPTIONS)
@@ -163,20 +202,34 @@ ovTestRegion( CoreLayer                  *layer,
           case DSPF_RGB32:
           case DSPF_YUY2:
           case DSPF_UYVY:
+               break;
           case DSPF_I420:
           case DSPF_YV12:
-               break;
-
+               if (mdev->chip >= CHIP_3D_RAGE_PRO)
+                    break;
           default:
                fail |= CLRCF_FORMAT;
      }
 
+     switch (config->format) {
+          case DSPF_I420:
+          case DSPF_YV12:
+               if (config->height & 1)
+                    fail |= CLRCF_HEIGHT;
+          case DSPF_YUY2:
+          case DSPF_UYVY:
+               if (config->width & 1)
+                    fail |= CLRCF_WIDTH;
+          default:
+               break;
+     }
+
      /* check width */
      if (config->width > max_width || config->width < 1)
           fail |= CLRCF_WIDTH;
 
      /* check height */
-     if (config->height > 1024 || config->height < 1)
+     if (config->height > max_height || config->height < 1)
           fail |= CLRCF_HEIGHT;
 
      /* write back failing fields */
@@ -206,10 +259,28 @@ ovSetRegion( CoreLayer                  *layer,
      /* remember configuration */
      mov->config = *config;
 
-     ov_calc_buffer( mdrv, mov, config, surface );
-     ov_calc_regs( mdrv, mov, config, surface );
-     ov_set_buffer( mdrv, mov );
-     ov_set_regs( mdrv, mov );
+     if (updated == CLRCF_ALL)
+          ov_reset( mdrv );
+
+     if (updated & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT | CLRCF_SOURCE | CLRCF_DEST | CLRCF_OPTIONS)) {
+          ov_calc_buffer( mdrv, mov, config, surface );
+          ov_calc_regs( mdrv, mov, config, surface );
+          ov_set_buffer( mdrv, mov );
+          ov_set_regs( mdrv, mov );
+     }
+
+     if (updated & (CLRCF_OPTIONS | CLRCF_SRCKEY | CLRCF_DSTKEY)) {
+          ov_calc_colorkey( mdrv, mov, config );
+          ov_set_colorkey( mdrv, mov );
+     }
+
+     if (updated & CLRCF_OPTIONS)
+          ov_set_field( mdrv, mov );
+
+     if (updated & (CLRCF_DEST | CLRCF_OPACITY)) {
+          ov_calc_opacity( mdrv, mov, config );
+          ov_set_opacity( mdrv, mov );
+     }
 
      return DFB_OK;
 }
@@ -220,12 +291,13 @@ ovRemoveRegion( CoreLayer *layer,
                 void      *layer_data,
                 void      *region_data )
 {
-     Mach64DriverData       *mdrv = (Mach64DriverData*) driver_data;
-     Mach64DeviceData       *mdev = mdrv->device_data;
+     Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
+     Mach64DeviceData *mdev = mdrv->device_data;
+     volatile __u8    *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, 1 );
 
-     mach64_out32( mdrv->mmio_base, OVERLAY_SCALE_CNTL, 0 );
+     mach64_out32( mmio, OVERLAY_SCALE_CNTL, 0 );
 
      return DFB_OK;
 }
@@ -261,7 +333,7 @@ ovSetColorAdjustment( CoreLayer          *layer,
 
      mach64_waitfifo( mdrv, mdev, 1 );
 
-     mach64_out32( mmio, SCALER_COLOR_CNTL,
+     mach64_out32( mmio, SCALER_COLOUR_CNTL,
                    (((adj->brightness >> 9) - 64) & 0x0000007F) |
                    ((adj->saturation >> 3)  & 0x00001F00) |
                    ((adj->saturation << 5)  & 0x001F0000) );
@@ -269,6 +341,23 @@ ovSetColorAdjustment( CoreLayer          *layer,
      return DFB_OK;
 }
 
+static DFBResult
+ovSetInputField( CoreLayer          *layer,
+                 void               *driver_data,
+                 void               *layer_data,
+                 void               *region_data,
+                 int                 field )
+{
+     Mach64DriverData       *mdrv = (Mach64DriverData*) driver_data;
+     Mach64OverlayLayerData *mov  = (Mach64OverlayLayerData*) layer_data;
+
+     mov->regs.capture_CONFIG = OVL_BUF_MODE_SINGLE | (field ? OVL_BUF_NEXT_BUF1 : OVL_BUF_NEXT_BUF0);
+
+     ov_set_field( mdrv, mov );
+
+     return DFB_OK;
+}
+
 DisplayLayerFuncs mach64OverlayFuncs = {
      LayerDataSize:      ovLayerDataSize,
      InitLayer:          ovInitLayer,
@@ -277,126 +366,291 @@ DisplayLayerFuncs mach64OverlayFuncs = {
      SetRegion:          ovSetRegion,
      RemoveRegion:       ovRemoveRegion,
      FlipRegion:         ovFlipRegion,
-     SetColorAdjustment: ovSetColorAdjustment
+     SetColorAdjustment: ovSetColorAdjustment,
+     SetInputField:      ovSetInputField,
 };
 
 /* internal */
 
-static void ov_set_buffer( Mach64DriverData       *mdrv,
-                           Mach64OverlayLayerData *mov )
+static void ov_reset( Mach64DriverData *mdrv )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile __u8    *mmio = mdrv->mmio_base;
 
-     mach64_waitfifo( mdrv, mdev, 3 );
+     if (mdev->chip >= CHIP_3D_RAGE_PRO) {
+          mach64_waitfifo( mdrv, mdev, 6 );
 
-     mach64_out32( mmio, SCALER_BUF0_OFFSET,       mov->regs.scaler_BUF0_OFFSET );
-     mach64_out32( mmio, SCALER_BUF0_OFFSET_U,     mov->regs.scaler_BUF0_OFFSET_U );
-     mach64_out32( mmio, SCALER_BUF0_OFFSET_V,     mov->regs.scaler_BUF0_OFFSET_V );
+          mach64_out32( mmio, SCALER_H_COEFF0, 0x00002000 );
+          mach64_out32( mmio, SCALER_H_COEFF1, 0x0D06200D );
+          mach64_out32( mmio, SCALER_H_COEFF2, 0x0D0A1C0D );
+          mach64_out32( mmio, SCALER_H_COEFF3, 0x0C0E1A0C );
+          mach64_out32( mmio, SCALER_H_COEFF4, 0x0C14140C );
+          mach64_out32( mmio, SCALER_COLOUR_CNTL, 0x00101000 );
+     }
+
+     mach64_waitfifo( mdrv, mdev, 4 );
+
+     mach64_out32( mmio, OVERLAY_SCALE_CNTL, 0 );
+     mach64_out32( mmio, OVERLAY_EXCLUSIVE_HORZ, 0 );
+     mach64_out32( mmio, OVERLAY_EXCLUSIVE_VERT, 0 );
+     mach64_out32( mmio, SCALER_TEST, 0 );
 }
 
 static void ov_set_regs( Mach64DriverData       *mdrv,
                          Mach64OverlayLayerData *mov )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile __u8    *mmio = mdrv->mmio_base;
 
-     mach64_waitfifo( mdrv, mdev, 12 );
+     mach64_waitfifo( mdrv, mdev, (mdev->chip >= CHIP_264VT3) ? 6 : 7 );
+
+     mach64_out32( mmio, VIDEO_FORMAT,          mov->regs.video_FORMAT );
+     mach64_out32( mmio, OVERLAY_Y_X_START,     mov->regs.overlay_Y_X_START );
+     mach64_out32( mmio, OVERLAY_Y_X_END,       mov->regs.overlay_Y_X_END );
+     mach64_out32( mmio, OVERLAY_SCALE_INC,     mov->regs.overlay_SCALE_INC );
+     mach64_out32( mmio, SCALER_HEIGHT_WIDTH,   mov->regs.scaler_HEIGHT_WIDTH );
+
+     if (mdev->chip >= CHIP_264VT3) {
+          mach64_out32( mmio, SCALER_BUF_PITCH, mov->regs.scaler_BUF_PITCH );
+     } else {
+          mach64_out32( mmio, BUF0_PITCH,       mov->regs.scaler_BUF_PITCH );
+          mach64_out32( mmio, BUF1_PITCH,       mov->regs.scaler_BUF_PITCH );
+     }
+}
+
+static void ov_set_buffer( Mach64DriverData       *mdrv,
+                           Mach64OverlayLayerData *mov )
+{
+     Mach64DeviceData *mdev = mdrv->device_data;
+     volatile __u8    *mmio = mdrv->mmio_base;
+
+     mach64_waitfifo( mdrv, mdev, (mdev->chip >= CHIP_3D_RAGE_PRO) ? 6 : 2 );
+
+     if (mdev->chip >= CHIP_264VT3) {
+          mach64_out32( mmio, SCALER_BUF0_OFFSET,   mov->regs.scaler_BUF0_OFFSET );
+          mach64_out32( mmio, SCALER_BUF1_OFFSET,   mov->regs.scaler_BUF1_OFFSET );
+     } else {
+          mach64_out32( mmio, BUF0_OFFSET,          mov->regs.scaler_BUF0_OFFSET );
+          mach64_out32( mmio, BUF1_OFFSET,          mov->regs.scaler_BUF1_OFFSET );
+     }
+
+     if (mdev->chip >= CHIP_3D_RAGE_PRO) {
+          mach64_out32( mmio, SCALER_BUF0_OFFSET_U, mov->regs.scaler_BUF0_OFFSET_U );
+          mach64_out32( mmio, SCALER_BUF0_OFFSET_V, mov->regs.scaler_BUF0_OFFSET_V );
+          mach64_out32( mmio, SCALER_BUF1_OFFSET_U, mov->regs.scaler_BUF1_OFFSET_U );
+          mach64_out32( mmio, SCALER_BUF1_OFFSET_V, mov->regs.scaler_BUF1_OFFSET_V );
+     }
+}
+
+static void ov_set_colorkey( Mach64DriverData       *mdrv,
+                             Mach64OverlayLayerData *mov )
+{
+     Mach64DeviceData *mdev = mdrv->device_data;
+     volatile __u8    *mmio = mdrv->mmio_base;
+
+     mach64_waitfifo( mdrv, mdev, 5 );
 
-     mach64_out32( mmio, SCALER_HEIGHT_WIDTH,      mov->regs.scaler_HEIGHT_WIDTH );
-     mach64_out32( mmio, SCALER_BUF_PITCH,         mov->regs.scaler_BUF_PITCH );
-     mach64_out32( mmio, VIDEO_FORMAT,             mov->regs.video_FORMAT );
-     mach64_out32( mmio, OVERLAY_Y_X_START,        mov->regs.overlay_Y_X_START );
-     mach64_out32( mmio, OVERLAY_Y_X_END,          mov->regs.overlay_Y_X_END );
      mach64_out32( mmio, OVERLAY_GRAPHICS_KEY_CLR, mov->regs.overlay_GRAPHICS_KEY_CLR );
      mach64_out32( mmio, OVERLAY_GRAPHICS_KEY_MSK, mov->regs.overlay_GRAPHICS_KEY_MSK );
      mach64_out32( mmio, OVERLAY_VIDEO_KEY_CLR,    mov->regs.overlay_VIDEO_KEY_CLR );
      mach64_out32( mmio, OVERLAY_VIDEO_KEY_MSK,    mov->regs.overlay_VIDEO_KEY_MSK );
-     mach64_out32( mmio, OVERLAY_KEY_CNTL,         mov->regs.overlay_KEY_CNTL);
-     mach64_out32( mmio, OVERLAY_SCALE_INC,        mov->regs.overlay_SCALE_INC );
-     mach64_out32( mmio, OVERLAY_SCALE_CNTL,       mov->regs.overlay_SCALE_CNTL );
+     mach64_out32( mmio, OVERLAY_KEY_CNTL,         mov->regs.overlay_KEY_CNTL );
 }
 
-static __u32 ovColorKey[] = {
-     VIDEO_MIX_TRUE | GRAPHICS_MIX_TRUE,                 /* 0 */
-     VIDEO_MIX_NOT_EQUAL | GRAPHICS_MIX_FALSE,           /* DLOP_SRC_COLORKEY */
-     VIDEO_MIX_FALSE | GRAPHICS_MIX_EQUAL,               /* DLOP_DST_COLORKEY */
-     VIDEO_MIX_NOT_EQUAL | GRAPHICS_MIX_EQUAL | CMP_MIX  /* DLOP_SRC_COLORKEY |
-                                                            DLOP_DST_COLORKEY */
-};
+static void ov_set_opacity( Mach64DriverData       *mdrv,
+                            Mach64OverlayLayerData *mov )
+{
+     Mach64DeviceData *mdev = mdrv->device_data;
+     volatile __u8    *mmio = mdrv->mmio_base;
+
+     mach64_waitfifo( mdrv, mdev, 1 );
+
+     mach64_out32( mmio, OVERLAY_SCALE_CNTL, mov->regs.overlay_SCALE_CNTL );
+}
+
+static void ov_set_field( Mach64DriverData       *mdrv,
+                          Mach64OverlayLayerData *mov )
+{
+     Mach64DeviceData *mdev = mdrv->device_data;
+     volatile __u8    *mmio = mdrv->mmio_base;
+
+     mach64_waitfifo( mdrv, mdev, 1 );
+
+     mach64_out32( mmio, CAPTURE_CONFIG, mov->regs.capture_CONFIG );
+}
 
 static void ov_calc_regs( Mach64DriverData       *mdrv,
                           Mach64OverlayLayerData *mov,
                           CoreLayerRegionConfig  *config,
                           CoreSurface            *surface )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
-     DFBSurfacePixelFormat primary_format = dfb_primary_layer_pixelformat();
-     SurfaceBuffer *front_buffer = surface->front_buffer;
-     VideoMode *mode = dfb_system_current_mode();
-     __u32 lcd_gen_ctrl, lcd_vert_stretching;
+     Mach64DeviceData *mdev   = mdrv->device_data;
+     volatile __u8    *mmio   = mdrv->mmio_base;
+     SurfaceBuffer    *buffer = surface->front_buffer;
+     VideoMode        *mode   = dfb_system_current_mode();
+     int               yres   = mode->yres;
+     int               pitch  = buffer->video.pitch / DFB_BYTES_PER_PIXEL( surface->format );
+     DFBRectangle      source = config->source;
+     DFBRectangle      dest   = config->dest;
+
      DFBRegion dst;
      int h_inc, v_inc;
+     __u32 lcd_gen_ctrl, vert_stretching;
+     __u8 ecp_div;
 
-     /* FIXME: I don't think this should be NULL ever. */
-     if (!mode)
-          return;
+     if (mode->doubled) {
+          dest.y *= 2;
+          dest.h *= 2;
+          yres   *= 2;
+     }
 
-     dst.x1 = config->dest.x;
-     dst.y1 = config->dest.y;
-     dst.x2 = config->dest.x + config->dest.w - 1;
-     dst.y2 = config->dest.y + config->dest.h - 1;
+     if (config->options & DLOP_DEINTERLACING) {
+          source.y /= 2;
+          source.h /= 2;
+          pitch    *= 2;
+     } else
+          mov->regs.capture_CONFIG = OVL_BUF_MODE_SINGLE | OVL_BUF_NEXT_BUF0;
 
-     dfb_region_intersect( &dst, 0, 0, mode->xres - 1, mode->yres - 1 );
+     dst.x1 = dest.x;
+     dst.y1 = dest.y;
+     dst.x2 = dest.x + dest.w - 1;
+     dst.y2 = dest.y + dest.h - 1;
 
-     h_inc = (config->source.w << 12) / config->dest.w;
+     mov->visible = dfb_region_intersect( &dst, 0, 0, mode->xres - 1, yres - 1 );
 
-     lcd_gen_ctrl = mach64_in_lcd( mmio, LCD_GEN_CTRL );
-     lcd_vert_stretching = mach64_in_lcd( mmio, LCD_VERT_STRETCHING );
+     if (mode->laced) {
+          dest.y /= 2;
+          dest.h /= 2;
+     }
 
-     if ((lcd_gen_ctrl & LCD_ON) && (lcd_vert_stretching & VERT_STRETCH_EN))
-          v_inc = (config->source.h << 2) * (lcd_vert_stretching & VERT_STRETCH_RATIO0) / config->dest.h;
+     ecp_div = (mach64_in_pll( mmio, PLL_VCLK_CNTL ) & ECP_DIV) >> 4;
+     h_inc = (source.w << (12 + ecp_div)) / dest.w;
+
+     lcd_gen_ctrl = mach64_in_lcd( mdev, mmio, LCD_GEN_CTRL );
+     vert_stretching = mach64_in_lcd( mdev, mmio, VERT_STRETCHING );
+     if ((lcd_gen_ctrl & LCD_ON) && (vert_stretching & VERT_STRETCH_EN))
+          v_inc = (source.h << 2) * (vert_stretching & VERT_STRETCH_RATIO0) / dest.h;
      else
-          v_inc = (config->source.h << 12) / config->dest.h;
+          v_inc = (source.h << 12) / dest.h;
 
      switch (surface->format) {
           case DSPF_ARGB1555:
                mov->regs.video_FORMAT = SCALER_IN_RGB15;
                break;
-
           case DSPF_RGB16:
                mov->regs.video_FORMAT = SCALER_IN_RGB16;
                break;
-
           case DSPF_RGB32:
                mov->regs.video_FORMAT = SCALER_IN_RGB32;
                break;
-
           case DSPF_UYVY:
                mov->regs.video_FORMAT = SCALER_IN_YVYU422;
                break;
-
           case DSPF_YUY2:
                mov->regs.video_FORMAT = SCALER_IN_VYUY422;
                break;
-
           case DSPF_I420:
-               mov->regs.video_FORMAT = SCALER_IN_YUV12;
-               break;
-
           case DSPF_YV12:
                mov->regs.video_FORMAT = SCALER_IN_YUV12;
                break;
+          default:
+               D_BUG( "unexpected pixelformat" );
+     }
+
+     mov->regs.scaler_HEIGHT_WIDTH = (source.w << 16) | source.h;
+     mov->regs.scaler_BUF_PITCH    = pitch;
+
+     mov->regs.overlay_Y_X_START   = (dst.x1 << 16) | dst.y1 | OVERLAY_LOCK_START;
+     mov->regs.overlay_Y_X_END     = (dst.x2 << 16) | dst.y2;
+     mov->regs.overlay_SCALE_INC   = (h_inc << 16) | v_inc;
+}
+
+static void ov_calc_buffer( Mach64DriverData       *mdrv,
+                            Mach64OverlayLayerData *mov,
+                            CoreLayerRegionConfig  *config,
+                            CoreSurface            *surface )
+{
+     SurfaceBuffer *buffer = surface->front_buffer;
+     int            pitch  = buffer->video.pitch;
+     DFBRectangle   source = config->source;
+
+     __u32 offset, offset_u, offset_v;
+     int cropleft, croptop;
+
+     if (config->options & DLOP_DEINTERLACING) {
+          source.y /= 2;
+          source.h /= 2;
+          pitch    *= 2;
+     }
+
+     /* Source cropping */
+     cropleft = source.x;
+     croptop  = source.y;
+
+     /* Add destination cropping */
+     if (config->dest.x < 0)
+          cropleft += -config->dest.x * source.w / config->dest.w;
+     if (config->dest.y < 0)
+          croptop  += -config->dest.y * source.h / config->dest.h;
+
+     switch (surface->format) {
+          case DSPF_I420:
+               cropleft &= ~15;
+               croptop  &= ~1;
+
+               offset_u  = buffer->video.offset + surface->height * buffer->video.pitch;
+               offset_v  = offset_u + surface->height/2 * buffer->video.pitch/2;
+               offset_u += croptop/2 * pitch/2 + cropleft/2;
+               offset_v += croptop/2 * pitch/2 + cropleft/2;
+               break;
+
+          case DSPF_YV12:
+               cropleft &= ~15;
+               croptop  &= ~1;
+
+               offset_v  = buffer->video.offset + surface->height * buffer->video.pitch;
+               offset_u  = offset_v + surface->height/2 * buffer->video.pitch/2;
+               offset_v += croptop/2 * pitch/2 + cropleft/2;
+               offset_u += croptop/2 * pitch/2 + cropleft/2;
+               break;
 
           default:
-               D_BUG("unexpected pixelformat");
-               return;
+               offset_u = 0;
+               offset_v = 0;
+               break;
      }
 
+     offset  = buffer->video.offset;
+     offset += croptop * pitch + cropleft * DFB_BYTES_PER_PIXEL( surface->format );
+
+     mov->regs.scaler_BUF0_OFFSET   = offset;
+     mov->regs.scaler_BUF0_OFFSET_U = offset_u;
+     mov->regs.scaler_BUF0_OFFSET_V = offset_v;
+
+     mov->regs.scaler_BUF1_OFFSET   = offset   + buffer->video.pitch;
+     mov->regs.scaler_BUF1_OFFSET_U = offset_u + buffer->video.pitch/2;
+     mov->regs.scaler_BUF1_OFFSET_V = offset_v + buffer->video.pitch/2;
+}
+
+static __u32 ovColorKey[] = {
+     VIDEO_KEY_FN_TRUE      | GRAPHICS_KEY_FN_TRUE  | OVERLAY_CMP_MIX_OR, /* 0 */
+     VIDEO_KEY_FN_NOT_EQUAL | GRAPHICS_KEY_FN_FALSE | OVERLAY_CMP_MIX_OR, /* DLOP_SRC_COLORKEY */
+     VIDEO_KEY_FN_FALSE     | GRAPHICS_KEY_FN_EQUAL | OVERLAY_CMP_MIX_OR, /* DLOP_DST_COLORKEY */
+     VIDEO_KEY_FN_NOT_EQUAL | GRAPHICS_KEY_FN_EQUAL | OVERLAY_CMP_MIX_AND /* DLOP_SRC_COLORKEY |
+                                                                             DLOP_DST_COLORKEY */
+};
+
+static void ov_calc_colorkey( Mach64DriverData       *mdrv,
+                              Mach64OverlayLayerData *mov,
+                              CoreLayerRegionConfig  *config )
+{
+     DFBSurfacePixelFormat primary_format = dfb_primary_layer_pixelformat();
+
      /* Video key is always RGB24 */
      mov->regs.overlay_VIDEO_KEY_CLR = PIXEL_RGB32( config->src_key.r,
                                                     config->src_key.g,
                                                     config->src_key.b );
+
      /* The same mask is used for all three components */
      mov->regs.overlay_VIDEO_KEY_MSK = 0xFF;
 
@@ -406,105 +660,43 @@ static void ov_calc_regs( Mach64DriverData       *mdrv,
                                                                   config->dst_key.g,
                                                                   config->dst_key.b );
                break;
-
           case DSPF_ARGB1555:
                mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_ARGB1555( config->dst_key.a,
                                                                     config->dst_key.r,
                                                                     config->dst_key.g,
                                                                     config->dst_key.b );
                break;
-
           case DSPF_RGB16:
                mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_RGB16( config->dst_key.r,
                                                                  config->dst_key.g,
                                                                  config->dst_key.b );
                break;
-
           case DSPF_RGB32:
                mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_RGB32( config->dst_key.r,
                                                                  config->dst_key.g,
                                                                  config->dst_key.b );
                break;
-
           case DSPF_ARGB:
                mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_ARGB( config->dst_key.a,
                                                                 config->dst_key.r,
                                                                 config->dst_key.g,
                                                                 config->dst_key.b );
                break;
-
-
           default:
-               D_BUG("unexpected pixelformat");
-               return;
+               D_BUG( "unexpected pixelformat" );
      }
+
      mov->regs.overlay_GRAPHICS_KEY_MSK = (1 << DFB_COLOR_BITS_PER_PIXEL( primary_format )) - 1;
 
      mov->regs.overlay_KEY_CNTL = ovColorKey[(config->options >> 3) & 3];
-
-     mov->regs.scaler_HEIGHT_WIDTH = (config->source.w << 16) | config->source.h;
-     mov->regs.scaler_BUF_PITCH    = front_buffer->video.pitch / DFB_BYTES_PER_PIXEL( surface->format );
-
-     mov->regs.overlay_Y_X_START = (dst.x1 << 16) | dst.y1 | OVERLAY_LOCK_START;
-     mov->regs.overlay_Y_X_END   = (dst.x2 << 16) | dst.y2;
-
-     mov->regs.overlay_SCALE_INC  = (h_inc << 16) | v_inc;
-     mov->regs.overlay_SCALE_CNTL = SCALE_PIX_EXPAND | SCALE_EN;
-
-     if (config->opacity)
-          mov->regs.overlay_SCALE_CNTL |= OVERLAY_EN;
 }
 
-
-static void ov_calc_buffer( Mach64DriverData       *mdrv,
-                            Mach64OverlayLayerData *mov,
-                            CoreLayerRegionConfig  *config,
-                            CoreSurface            *surface )
+static void ov_calc_opacity( Mach64DriverData       *mdrv,
+                             Mach64OverlayLayerData *mov,
+                             CoreLayerRegionConfig  *config )
 {
-     __u32 offset, offset_u = 0, offset_v = 0;
-     SurfaceBuffer *front_buffer = surface->front_buffer;
-     int cropleft, croptop;
+     mov->regs.overlay_SCALE_CNTL = SCALE_PIX_EXPAND;
 
-     /* Source cropping */
-     cropleft = config->source.x;
-     croptop  = config->source.y;
-
-     /* Add destination cropping */
-     if (config->dest.x < 0)
-          cropleft += -config->dest.x * config->source.w / config->dest.w;
-     if (config->dest.y < 0)
-          croptop  += -config->dest.y * config->source.h / config->dest.h;
-
-     switch (surface->format) {
-          case DSPF_I420:
-               cropleft &= ~15;
-               croptop &= ~1;
-
-               offset_u = front_buffer->video.offset + surface->height * front_buffer->video.pitch;
-               offset_v = offset_u + (surface->height/2) * (front_buffer->video.pitch/2);
-               offset_u += (croptop/2) * (front_buffer->video.pitch/2) + (cropleft/2);
-               offset_v += (croptop/2) * (front_buffer->video.pitch/2) + (cropleft/2);
-               break;
-
-          case DSPF_YV12:
-               cropleft &= ~15;
-               croptop &= ~1;
-
-               offset_v = front_buffer->video.offset + surface->height * front_buffer->video.pitch;
-               offset_u = offset_v + (surface->height/2) * (front_buffer->video.pitch/2);
-               offset_u += (croptop/2) * (front_buffer->video.pitch/2) + (cropleft/2);
-               offset_v += (croptop/2) * (front_buffer->video.pitch/2) + (cropleft/2);
-               break;
-
-          default:
-               break;
-     }
-
-     offset = front_buffer->video.offset;
-     offset += croptop * front_buffer->video.pitch + cropleft * DFB_BYTES_PER_PIXEL( surface->format );
-
-     mov->regs.scaler_BUF0_OFFSET     = offset;
-     mov->regs.scaler_BUF0_OFFSET_U   = offset_u;
-     mov->regs.scaler_BUF0_OFFSET_V   = offset_v;
+     if (config->opacity && mov->visible)
+          mov->regs.overlay_SCALE_CNTL |= OVERLAY_EN | SCALE_EN;
 }
-
