@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -34,19 +35,23 @@
 
 #include <directfb.h>
 
-#include <core/fusion/shmalloc.h>
+#include <fusion/shmalloc.h>
 
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 #include <core/gfxcard.h>
-#include <core/fbdev/fbdev.h>
 #include <core/layers.h>
+#include <core/screen.h>
 #include <core/surfaces.h>
 #include <core/windows.h>
 
-#include <misc/mem.h>
+#include <direct/mem.h>
+#include <direct/messages.h>
+
+#include <misc/conf.h>
 #include <misc/util.h>
 
+#include <fbdev/fbdev.h>
 
 #include "regs.h"
 #include "mmio.h"
@@ -54,9 +59,8 @@
 #include "matrox_maven.h"
 
 typedef struct {
-     DFBDisplayLayerConfig config;
+     CoreLayerRegionConfig config;
      DFBColorAdjustment    adj;
-     int                   enabled;
      int                   field;
 
      /* Stored registers */
@@ -81,17 +85,26 @@ typedef struct {
      MatroxMavenData mav;
 } MatroxCrtc2LayerData;
 
-static void crtc2_wait_vsync( MatroxDriverData *mdrv );
-static void crtc2_set_regs( MatroxDriverData *mdrv, MatroxCrtc2LayerData *mcrtc2 );
-static void crtc2_calc_regs( MatroxDriverData *mdrv, MatroxCrtc2LayerData *mcrtc2,
-                             DisplayLayer *layer );
-static void crtc2_calc_buffer( MatroxDriverData *mdrv, MatroxCrtc2LayerData *mcrtc2,
-                               DisplayLayer *layer );
-static void crtc2_set_buffer( MatroxDriverData *mdrv, MatroxCrtc2LayerData *mcrtc2,
-                              DisplayLayer *layer );
-static DFBResult crtc2_disable_output( MatroxDriverData *mdrv, MatroxCrtc2LayerData *mcrtc2 );
-static DFBResult crtc2_enable_output( MatroxDriverData *mdrv, MatroxCrtc2LayerData *mcrtc2,
-                                      DisplayLayer *layer );
+static void crtc2_set_regs           ( MatroxDriverData      *mdrv,
+                                       MatroxCrtc2LayerData  *mcrtc2 );
+
+static void crtc2_calc_regs          ( MatroxDriverData      *mdrv,
+                                       MatroxCrtc2LayerData  *mcrtc2,
+                                       CoreLayerRegionConfig *config,
+                                       CoreSurface           *surface );
+
+static void crtc2_calc_buffer        ( MatroxDriverData      *mdrv,
+                                       MatroxCrtc2LayerData  *mcrtc2,
+                                       CoreSurface           *surface );
+
+static void crtc2_set_buffer         ( MatroxDriverData      *mdrv,
+                                       MatroxCrtc2LayerData  *mcrtc2 );
+
+static DFBResult crtc2_disable_output( MatroxDriverData      *mdrv,
+                                       MatroxCrtc2LayerData  *mcrtc2 );
+
+static DFBResult crtc2_enable_output ( MatroxDriverData      *mdrv,
+                                       MatroxCrtc2LayerData  *mcrtc2 );
 
 #define CRTC2_SUPPORTED_OPTIONS   (DLOP_FIELD_PARITY | DLOP_FLICKER_FILTERING)
 
@@ -104,16 +117,16 @@ crtc2LayerDataSize()
 }
 
 static DFBResult
-crtc2InitLayer( GraphicsDevice             *device,
-                DisplayLayer               *layer,
-                DisplayLayerInfo           *layer_info,
-                DFBDisplayLayerConfig      *default_config,
-                DFBColorAdjustment         *default_adj,
+crtc2InitLayer( CoreLayer                  *layer,
                 void                       *driver_data,
-                void                       *layer_data )
+                void                       *layer_data,
+                DFBDisplayLayerDescription *description,
+                DFBDisplayLayerConfig      *config,
+                DFBColorAdjustment         *adjustment )
 {
      MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
      MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
+     MatroxDeviceData     *mdev   = mdrv->device_data;
      MatroxMavenData      *mav    = &mcrtc2->mav;
      bool                  ntsc   = dfb_config->matrox_ntsc;
      DFBResult             res;
@@ -123,91 +136,63 @@ crtc2InitLayer( GraphicsDevice             *device,
 
 
      /* set capabilities and type */
-     layer_info->desc.caps = DLCAPS_SURFACE |
-                             DLCAPS_FIELD_PARITY | DLCAPS_FLICKER_FILTERING |
-                             DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST |
-                             DLCAPS_HUE | DLCAPS_SATURATION;
-     layer_info->desc.type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
+     description->caps = DLCAPS_SURFACE |
+                         DLCAPS_FIELD_PARITY | DLCAPS_FLICKER_FILTERING |
+                         DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST |
+                         DLCAPS_HUE | DLCAPS_SATURATION | DLCAPS_ALPHA_RAMP;
+     description->type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
 
      /* set name */
-     snprintf( layer_info->desc.name,
-               DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "Matrox CRTC2" );
+     snprintf( description->name,
+               DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "Matrox CRTC2 Layer" );
 
      /* fill out the default configuration */
-     default_config->flags       = DLCONF_WIDTH | DLCONF_HEIGHT |
-                                   DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE |
-                                   DLCONF_OPTIONS;
-     default_config->width       = 720;
-     default_config->height      = ntsc ? 480 : 576;
-     default_config->pixelformat = DSPF_YUY2;
-     default_config->buffermode  = DLBM_FRONTONLY;
-     default_config->options     = DLOP_NONE;
+     config->flags        = DLCONF_WIDTH | DLCONF_HEIGHT |
+                            DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE |
+                            DLCONF_OPTIONS | DLCONF_SURFACE_CAPS;
+     config->width        = 720;
+     config->height       = ntsc ? 480 : 576;
+     config->pixelformat  = DSPF_YUY2;
+     config->buffermode   = DLBM_FRONTONLY;
+     config->options      = DLOP_NONE;
+     config->surface_caps = DSCAPS_INTERLACED;
 
      /* fill out default color adjustment,
         only fields set in flags will be accepted from applications */
-     default_adj->flags = DCAF_BRIGHTNESS | DCAF_CONTRAST |
-                          DCAF_HUE | DCAF_SATURATION;
-     if (mav->g450) {
-          default_adj->brightness = ntsc ? 0xAA00 : 0x9E00;
-          default_adj->saturation = ntsc ? 0xAE00 : 0xBB00;
+     adjustment->flags = DCAF_BRIGHTNESS | DCAF_CONTRAST |
+                         DCAF_HUE | DCAF_SATURATION;
+     if (mdev->g450_matrox) {
+          adjustment->brightness = ntsc ? 0xAA00 : 0x9E00;
+          adjustment->saturation = ntsc ? 0xAE00 : 0xBB00;
      } else {
-          default_adj->brightness = ntsc ? 0xB500 : 0xA800;
-          default_adj->saturation = ntsc ? 0x8E00 : 0x9500;
+          adjustment->brightness = ntsc ? 0xB500 : 0xA800;
+          adjustment->saturation = ntsc ? 0x8E00 : 0x9500;
      }
-     default_adj->contrast = ntsc ? 0xEA00 : 0xFF00;
-     default_adj->hue      = 0x0000;
+     adjustment->contrast = ntsc ? 0xEA00 : 0xFF00;
+     adjustment->hue      = 0x0000;
 
      /* remember color adjustment */
-     mcrtc2->adj = *default_adj;
+     mcrtc2->adj = *adjustment;
 
      return DFB_OK;
 }
 
 static DFBResult
-crtc2Enable( DisplayLayer *layer,
-             void         *driver_data,
-             void         *layer_data )
+crtc2TestRegion( CoreLayer                  *layer,
+                 void                       *driver_data,
+                 void                       *layer_data,
+                 CoreLayerRegionConfig      *config,
+                 CoreLayerRegionConfigFlags *failed )
 {
-     MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
-     MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
-     DFBResult             res;
-
-     res = crtc2_enable_output( mdrv, mcrtc2, layer );
-     if (res == DFB_OK)
-          mcrtc2->enabled = 1;
-
-     return res;
-}
-
-static DFBResult
-crtc2Disable( DisplayLayer *layer,
-              void         *driver_data,
-              void         *layer_data )
-{
-     MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
-     MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
-     DFBResult             res;
-
-     res = crtc2_disable_output( mdrv, mcrtc2 );
-     if (res == DFB_OK)
-          mcrtc2->enabled = 0;
-
-     return res;
-}
-
-static DFBResult
-crtc2TestConfiguration( DisplayLayer               *layer,
-                        void                       *driver_data,
-                        void                       *layer_data,
-                        DFBDisplayLayerConfig      *config,
-                        DFBDisplayLayerConfigFlags *failed )
-{
-     DFBDisplayLayerConfigFlags fail = 0;
+     CoreLayerRegionConfigFlags fail = 0;
 
      if (config->options & ~CRTC2_SUPPORTED_OPTIONS)
-          fail |= DLCONF_OPTIONS;
+          fail |= CLRCF_OPTIONS;
 
-     switch (config->pixelformat) {
+     if (config->surface_caps & ~DSCAPS_INTERLACED)
+          fail |= CLRCF_SURFACE_CAPS;
+
+     switch (config->format) {
           case DSPF_ARGB:
           case DSPF_RGB32:
           case DSPF_ARGB1555:
@@ -218,14 +203,14 @@ crtc2TestConfiguration( DisplayLayer               *layer,
           case DSPF_YV12:
                break;
           default:
-               fail |= DLCONF_PIXELFORMAT;
+               fail |= CLRCF_FORMAT;
      }
 
      if (config->width != 720)
-          fail |= DLCONF_WIDTH;
+          fail |= CLRCF_WIDTH;
 
      if (config->height != (dfb_config->matrox_ntsc ? 480 : 576))
-          fail |= DLCONF_HEIGHT;
+          fail |= CLRCF_HEIGHT;
 
      if (failed)
           *failed = fail;
@@ -237,71 +222,92 @@ crtc2TestConfiguration( DisplayLayer               *layer,
 }
 
 static DFBResult
-crtc2SetConfiguration( DisplayLayer          *layer,
-                       void                  *driver_data,
-                       void                  *layer_data,
-                       DFBDisplayLayerConfig *config )
+crtc2AddRegion( CoreLayer             *layer,
+                void                  *driver_data,
+                void                  *layer_data,
+                void                  *region_data,
+                CoreLayerRegionConfig *config )
 {
+     return DFB_OK;
+}
+
+static DFBResult
+crtc2SetRegion( CoreLayer                  *layer,
+                void                       *driver_data,
+                void                       *layer_data,
+                void                       *region_data,
+                CoreLayerRegionConfig      *config,
+                CoreLayerRegionConfigFlags  updated,
+                CoreSurface                *surface,
+                CorePalette                *palette )
+{
+     DFBResult             ret;
      MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
      MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
 
      /* remember configuration */
      mcrtc2->config = *config;
 
-     crtc2_calc_regs( mdrv, mcrtc2, layer );
-     crtc2_calc_buffer( mdrv, mcrtc2, layer );
+     mcrtc2->field = !config->parity;
 
-     if (mcrtc2->enabled)
-          return crtc2_enable_output( mdrv, mcrtc2, layer );
+     crtc2_calc_regs( mdrv, mcrtc2, config, surface );
+     crtc2_calc_buffer( mdrv, mcrtc2, surface );
+
+     ret = crtc2_enable_output( mdrv, mcrtc2 );
+     if (ret)
+          return ret;
 
      return DFB_OK;
 }
 
 static DFBResult
-crtc2FlipBuffers( DisplayLayer        *layer,
-                  void                *driver_data,
-                  void                *layer_data,
-                  DFBSurfaceFlipFlags  flags )
+crtc2RemoveRegion( CoreLayer *layer,
+                   void      *driver_data,
+                   void      *layer_data,
+                   void      *region_data )
+{
+     MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
+     MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
+
+     crtc2_disable_output( mdrv, mcrtc2 );
+
+     return DFB_OK;
+}
+
+static DFBResult
+crtc2FlipRegion( CoreLayer           *layer,
+                 void                *driver_data,
+                 void                *layer_data,
+                 void                *region_data,
+                 CoreSurface         *surface,
+                 DFBSurfaceFlipFlags  flags )
 {
      MatroxDriverData     *mdrv    = (MatroxDriverData*) driver_data;
      MatroxCrtc2LayerData *mcrtc2  = (MatroxCrtc2LayerData*) layer_data;
-     CoreSurface          *surface = dfb_layer_surface( layer );
      volatile __u8        *mmio    = mdrv->mmio_base;
 
-     int                   vdisplay = (dfb_config->matrox_ntsc ? 480/2 : 576/2) + 2;
-     int                   line;
-
-     dfb_surface_flip_buffers( surface );
-     crtc2_calc_buffer( mdrv, mcrtc2, layer );
-
-     if (!mcrtc2->enabled)
-          return DFB_OK;
-
-     dfb_gfxcard_sync();
-     
-     line = mga_in32( mmio, C2VCOUNT ) & 0x00000FFF;
-     if (line + 6 > vdisplay && line < vdisplay)
-          while ((int)(mga_in32( mmio, C2VCOUNT ) & 0x00000FFF) != vdisplay)
-               ;
+     dfb_surface_flip_buffers( surface, false );
+     crtc2_calc_buffer( mdrv, mcrtc2, surface );
 
      if (mcrtc2->config.options & DLOP_FIELD_PARITY) {
-          int field = (mga_in32( mmio, C2VCOUNT ) >> 24) & 0x1;
+          int field = (mga_in32( mmio, C2VCOUNT ) & C2FIELD) ? 1 : 0;
 
           while (field == mcrtc2->field) {
-               crtc2_wait_vsync( mdrv );
-               field = (mga_in32( mmio, C2VCOUNT ) >> 24) & 0x1;
+               dfb_screen_wait_vsync( mdrv->secondary );
+
+               field = (mga_in32( mmio, C2VCOUNT ) & C2FIELD) ? 1 : 0;
           }
      }
-     crtc2_set_buffer( mdrv, mcrtc2, layer );
+     crtc2_set_buffer( mdrv, mcrtc2 );
 
      if (flags & DSFLIP_WAIT)
-          crtc2_wait_vsync( mdrv );
+          dfb_screen_wait_vsync( mdrv->secondary );
 
      return DFB_OK;
 }
 
 static DFBResult
-crtc2SetColorAdjustment( DisplayLayer       *layer,
+crtc2SetColorAdjustment( CoreLayer          *layer,
                          void               *driver_data,
                          void               *layer_data,
                          DFBColorAdjustment *adj )
@@ -335,104 +341,37 @@ crtc2SetColorAdjustment( DisplayLayer       *layer,
 }
 
 static DFBResult
-crtc2SetOpacity( DisplayLayer *layer,
-                 void         *driver_data,
-                 void         *layer_data,
-                 __u8          opacity )
-{
-     MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
-     MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
-     DFBResult             res;
-
-     switch (opacity) {
-          case 0:
-               res = crtc2_disable_output( mdrv, mcrtc2 );
-               if (res == DFB_OK)
-                    mcrtc2->enabled = 0;
-               return res;
-          case 0xFF:
-               res = crtc2_enable_output( mdrv, mcrtc2, layer );
-               if (res == DFB_OK)
-                    mcrtc2->enabled = 1;
-               return res;
-          default:
-               return DFB_UNSUPPORTED;
-     }
-}
-
-static DFBResult
-crtc2GetCurrentOutputField( DisplayLayer *layer,
-                            void         *driver_data,
-                            void         *layer_data,
-                            int          *field )
+crtc2GetCurrentOutputField( CoreLayer *layer,
+                            void      *driver_data,
+                            void      *layer_data,
+                            int       *field )
 {
      MatroxDriverData *mdrv = (MatroxDriverData*) driver_data;
 
      if (!field)
           return DFB_INVARG;
 
-     *field = (mga_in32( mdrv->mmio_base, C2VCOUNT ) >> 24) & 0x1;
-
-     return DFB_OK;
-}
-
-static DFBResult
-crtc2SetFieldParity( DisplayLayer *layer,
-                     void         *driver_data,
-                     void         *layer_data,
-                     int           field )
-{
-     MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
-
-     mcrtc2->field = !field;
-
-     return DFB_OK;
-}
-
-static DFBResult
-crtc2WaitVSync( DisplayLayer *layer,
-                void         *driver_data,
-                void         *layer_data )
-{
-     MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
-     MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
-
-     if (mcrtc2->enabled)
-          crtc2_wait_vsync( mdrv );
+     *field = (mga_in32( mdrv->mmio_base, C2VCOUNT ) & C2FIELD) ? 1 : 0;
 
      return DFB_OK;
 }
 
 DisplayLayerFuncs matroxCrtc2Funcs = {
-     LayerDataSize:      crtc2LayerDataSize,
-     InitLayer:          crtc2InitLayer,
-     Enable:             crtc2Enable,
-     Disable:            crtc2Disable,
-     TestConfiguration:  crtc2TestConfiguration,
-     SetConfiguration:   crtc2SetConfiguration,
-     FlipBuffers:        crtc2FlipBuffers,
-     SetColorAdjustment: crtc2SetColorAdjustment,
-     SetOpacity:         crtc2SetOpacity,
-     GetCurrentOutputField: crtc2GetCurrentOutputField,
-     SetFieldParity:     crtc2SetFieldParity,
-     WaitVSync:          crtc2WaitVSync
+     LayerDataSize:         crtc2LayerDataSize,
+     InitLayer:             crtc2InitLayer,
+
+     TestRegion:            crtc2TestRegion,
+     AddRegion:             crtc2AddRegion,
+     SetRegion:             crtc2SetRegion,
+     RemoveRegion:          crtc2RemoveRegion,
+     FlipRegion:            crtc2FlipRegion,
+
+     SetColorAdjustment:    crtc2SetColorAdjustment,
+     GetCurrentOutputField: crtc2GetCurrentOutputField
 };
 
 /* internal */
 
-static void crtc2_wait_vsync( MatroxDriverData *mdrv )
-{
-     int vdisplay = (dfb_config->matrox_ntsc ? 480/2 : 576/2) + 2;
-
-#ifdef FBIO_WAITFORVSYNC
-     static const int one = 1;
-     FBDev *dfb_fbdev = dfb_system_data();
-     if (ioctl( dfb_fbdev->fd, FBIO_WAITFORVSYNC, &one ))
-#endif
-          while ((int)(mga_in32( mdrv->mmio_base, C2VCOUNT ) & 0x00000FFF) != vdisplay)
-               ;
-}
-
 static void crtc2_set_regs( MatroxDriverData     *mdrv,
                             MatroxCrtc2LayerData *mcrtc2 )
 {
@@ -449,97 +388,86 @@ static void crtc2_set_regs( MatroxDriverData     *mdrv,
      mga_out32( mmio, 0,                      C2PRELOAD );
 }
 
-static void crtc2_calc_regs( MatroxDriverData     *mdrv,
-                             MatroxCrtc2LayerData *mcrtc2,
-                             DisplayLayer         *layer )
+static void crtc2_calc_regs( MatroxDriverData      *mdrv,
+                             MatroxCrtc2LayerData  *mcrtc2,
+                             CoreLayerRegionConfig *config,
+                             CoreSurface           *surface )
 {
-     MatroxMavenData *mav     = &mcrtc2->mav;
-     CoreSurface     *surface = dfb_layer_surface( layer );
+     MatroxDeviceData *mdev = mdrv->device_data;
 
      mcrtc2->regs.c2CTL = 0;
-     mcrtc2->regs.c2DATACTL = 0;
 
-     /* c2pixcksel = 01 (vdoclk) */
-     mcrtc2->regs.c2CTL |= (1 << 1);
-     if (mav->g450)
-          mcrtc2->regs.c2CTL |= (1 << 14);
+     /* Don't touch sub-picture bits. */
+     mcrtc2->regs.c2DATACTL  = mga_in32( mdrv->mmio_base, C2DATACTL );
+     mcrtc2->regs.c2DATACTL &= C2STATICKEY | C2OFFSETDIVEN | C2STATICKEYEN | C2SUBPICEN;
+
+     if (mdev->g450_matrox)
+          mcrtc2->regs.c2CTL |= C2PIXCLKSEL_CRISTAL;
+     else
+          mcrtc2->regs.c2CTL |= C2PIXCLKSEL_VDOCLK;
 
      /*
       * High priority request level.
       * According to G400 specs these values should
       * be fixed when CRTC2 is in YUV mode.
       */
-     /* c2hiprilvl = 010 */
-     mcrtc2->regs.c2CTL |= (2 << 4);
-     /* c2maxhipri = 001 */
-     mcrtc2->regs.c2CTL |= (1 << 8);
+     /* c2hiprilvl */
+     mcrtc2->regs.c2CTL |= 2 << 4;
+     /* c2maxhipri */
+     mcrtc2->regs.c2CTL |= 1 << 8;
 
-     /* c2vidrstmod = 01 */
-     mcrtc2->regs.c2CTL |= (1 << 28);
-     /* c2hploaden = 1 */
-     mcrtc2->regs.c2CTL |= (1 << 30);
-     /* c2vploaden = 1 */
-     mcrtc2->regs.c2CTL |= (1 << 31);
+     mcrtc2->regs.c2CTL |= C2VIDRSTMOD_RISING | C2HPLOADEN | C2VPLOADEN;
 
      switch (surface->format) {
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB32:
           case DSPF_ARGB:
-               /* c2dithen = 1 */
-               mcrtc2->regs.c2DATACTL |= (1 << 0);
-               /* c2yfiltend = 1 */
-               mcrtc2->regs.c2DATACTL |= (1 << 1);
-               /* c2cbcrfilten = 1 */
-               mcrtc2->regs.c2DATACTL |= (1 << 2);
+               mcrtc2->regs.c2DATACTL |= C2DITHEN | C2YFILTEN | C2CBCRFILTEN;
                break;
           default:
                break;
      }
 
      if (dfb_config->matrox_ntsc)
-          /* c2ntscen = 1 */
-          mcrtc2->regs.c2DATACTL |= (1 << 4);
+          mcrtc2->regs.c2DATACTL |= C2NTSCEN;
 
      /* pixel format settings */
      switch (surface->format) {
           case DSPF_I420:
           case DSPF_YV12:
-               /* c2depth = 111 */
-               mcrtc2->regs.c2CTL |= (7 << 21);
+               mcrtc2->regs.c2CTL |= C2DEPTH_YCBCR420;
                break;
 
           case DSPF_UYVY:
-               /* c2uyvyfmt = 1 */
-               mcrtc2->regs.c2DATACTL |= (1 << 7);
+               mcrtc2->regs.c2DATACTL |= C2UYVYFMT;
                /* fall through */
 
           case DSPF_YUY2:
-               /* c2depth = 101 */
-               mcrtc2->regs.c2CTL |= (5 << 21);
+               mcrtc2->regs.c2CTL |= C2DEPTH_YCBCR422;
                break;
 
           case DSPF_ARGB1555:
-               /* c2depth = 001 */
-               mcrtc2->regs.c2CTL |= (1 << 21);
+               mcrtc2->regs.c2CTL |= C2DEPTH_15BPP;
                break;
 
           case DSPF_RGB16:
-               /* c2depth = 010 */
-               mcrtc2->regs.c2CTL |= (2 << 21);
+               mcrtc2->regs.c2CTL |= C2DEPTH_16BPP;
                break;
 
           case DSPF_RGB32:
           case DSPF_ARGB:
-               /* c2depth = 100 */
-               mcrtc2->regs.c2CTL |= (4 << 21);
+               mcrtc2->regs.c2CTL |= C2DEPTH_32BPP;
                break;
 
           default:
-               BUG( "unexpected pixelformat" );
+               D_BUG( "unexpected pixelformat" );
                return;
      }
 
+     if (!(surface->caps & DSCAPS_INTERLACED))
+          mcrtc2->regs.c2CTL |= C2VCBCRSINGLE;
+
      /* interleaved fields */
      mcrtc2->regs.c2OFFSET = surface->front_buffer->video.pitch * 2;
 
@@ -563,34 +491,45 @@ static void crtc2_calc_regs( MatroxDriverData     *mdrv,
 
           mcrtc2->regs.c2MISC = 0;
           /* c2vlinecomp */
-          mcrtc2->regs.c2MISC |= ((vdisplay + 2) << 16);
+          mcrtc2->regs.c2MISC |= (vdisplay + 2) << 16;
      }
+
+     /* c2bpp15halpha */
+     mcrtc2->regs.c2DATACTL |= config->alpha_ramp[3] << 8;
+
+     /* c2bpp15lalpha */
+     mcrtc2->regs.c2DATACTL |= config->alpha_ramp[0] << 16;
 }
 
 static void crtc2_calc_buffer( MatroxDriverData     *mdrv,
                                MatroxCrtc2LayerData *mcrtc2,
-                               DisplayLayer         *layer )
+                               CoreSurface          *surface )
 {
-     CoreSurface   *surface      = dfb_layer_surface( layer );
      SurfaceBuffer *front_buffer = surface->front_buffer;
+     int            field_offset = front_buffer->video.pitch;
 
      mcrtc2->regs.c2STARTADD1 = front_buffer->video.offset;
-     mcrtc2->regs.c2STARTADD0 = front_buffer->video.offset + front_buffer->video.pitch;
+     mcrtc2->regs.c2STARTADD0 = mcrtc2->regs.c2STARTADD1 + field_offset;
+
+     if (surface->caps & DSCAPS_INTERLACED)
+          field_offset /= 2;
+     else
+          field_offset = 0;
 
      switch (surface->format) {
           case DSPF_I420:
                mcrtc2->regs.c2PL2STARTADD1 = mcrtc2->regs.c2STARTADD1 + surface->height * front_buffer->video.pitch;
-               mcrtc2->regs.c2PL2STARTADD0 = mcrtc2->regs.c2PL2STARTADD1 + front_buffer->video.pitch/2;
+               mcrtc2->regs.c2PL2STARTADD0 = mcrtc2->regs.c2PL2STARTADD1 + field_offset;
 
                mcrtc2->regs.c2PL3STARTADD1 = mcrtc2->regs.c2PL2STARTADD1 + surface->height/2 * front_buffer->video.pitch/2;
-               mcrtc2->regs.c2PL3STARTADD0 = mcrtc2->regs.c2PL3STARTADD1 + front_buffer->video.pitch/2;
+               mcrtc2->regs.c2PL3STARTADD0 = mcrtc2->regs.c2PL3STARTADD1 + field_offset;
                break;
           case DSPF_YV12:
                mcrtc2->regs.c2PL3STARTADD1 = mcrtc2->regs.c2STARTADD1 + surface->height * front_buffer->video.pitch;
-               mcrtc2->regs.c2PL3STARTADD0 = mcrtc2->regs.c2PL3STARTADD1 + front_buffer->video.pitch/2;
+               mcrtc2->regs.c2PL3STARTADD0 = mcrtc2->regs.c2PL3STARTADD1 + field_offset;
 
                mcrtc2->regs.c2PL2STARTADD1 = mcrtc2->regs.c2PL3STARTADD1 + surface->height/2 *  front_buffer->video.pitch/2;
-               mcrtc2->regs.c2PL2STARTADD0 = mcrtc2->regs.c2PL2STARTADD1 + front_buffer->video.pitch/2;
+               mcrtc2->regs.c2PL2STARTADD0 = mcrtc2->regs.c2PL2STARTADD1 + field_offset;
                break;
           default:
                break;
@@ -598,17 +537,16 @@ static void crtc2_calc_buffer( MatroxDriverData     *mdrv,
 }
 
 static void crtc2_set_buffer( MatroxDriverData     *mdrv,
-                              MatroxCrtc2LayerData *mcrtc2,
-                              DisplayLayer         *layer )
+                              MatroxCrtc2LayerData *mcrtc2 )
 {
      volatile __u8 *mmio = mdrv->mmio_base;
 
-     mga_out32( mmio, mcrtc2->regs.c2STARTADD1, C2STARTADD1 );
      mga_out32( mmio, mcrtc2->regs.c2STARTADD0, C2STARTADD0 );
-     mga_out32( mmio, mcrtc2->regs.c2PL2STARTADD1, C2PL2STARTADD1 );
+     mga_out32( mmio, mcrtc2->regs.c2STARTADD1, C2STARTADD1 );
      mga_out32( mmio, mcrtc2->regs.c2PL2STARTADD0, C2PL2STARTADD0 );
-     mga_out32( mmio, mcrtc2->regs.c2PL3STARTADD1, C2PL3STARTADD1 );
+     mga_out32( mmio, mcrtc2->regs.c2PL2STARTADD1, C2PL2STARTADD1 );
      mga_out32( mmio, mcrtc2->regs.c2PL3STARTADD0, C2PL3STARTADD0 );
+     mga_out32( mmio, mcrtc2->regs.c2PL3STARTADD1, C2PL3STARTADD1 );
 }
 
 static void
@@ -619,24 +557,19 @@ crtc2OnOff( MatroxDriverData     *mdrv,
      volatile __u8 *mmio = mdrv->mmio_base;
 
      if (on)
-          /* c2en = 1 */
-          mcrtc2->regs.c2CTL |= 1;
+          mcrtc2->regs.c2CTL |= C2EN;
      else
-          /* c2en = 0 */
-          mcrtc2->regs.c2CTL &= ~1;
+          mcrtc2->regs.c2CTL &= ~C2EN;
      mga_out32( mmio, mcrtc2->regs.c2CTL, C2CTL );
 
      if (on)
-          /* c2pixclkdis = 0 */
-          mcrtc2->regs.c2CTL &= ~8;
+          mcrtc2->regs.c2CTL &= ~C2PIXCLKDIS;
      else
-          /* c2pixclkdis = 1 */
-          mcrtc2->regs.c2CTL |= 8;
+          mcrtc2->regs.c2CTL |= C2PIXCLKDIS;
      mga_out32( mmio, mcrtc2->regs.c2CTL, C2CTL );
 
      if (!on) {
-          /* c2interlace = 0 */
-          mcrtc2->regs.c2CTL &= ~(1 << 25);
+          mcrtc2->regs.c2CTL &= ~C2INTERLACE;
           mga_out32( mmio, mcrtc2->regs.c2CTL, C2CTL );
      }
 }
@@ -649,17 +582,11 @@ static void crtc2_set_mafc( MatroxDriverData     *mdrv,
 
      val = mga_in_dac( mmio, XMISCCTRL );
      if (on) {
-          /*
-           * mfcsel   = 01  (MAFC)
-           * vdoutsel = 110 (CRTC2 ITU-R 656)
-           */
-          val &= ~((3 << 1) | (7 << 5));
-          val |=   (1 << 1) | (6 << 5);
+          val &= ~(MFCSEL_MASK | VDOUTSEL_MASK);
+          val |= MFCSEL_MAFC | VDOUTSEL_CRTC2656;
      } else {
-          /*
-           * mfcsel   = 11  (Disable)
-           */
-          val |= (3 << 1);
+          val &= ~MFCSEL_MASK;
+          val |= MFCSEL_DIS;
      }
      mga_out_dac( mmio, XMISCCTRL, val );
 }
@@ -668,34 +595,39 @@ static DFBResult
 crtc2_disable_output( MatroxDriverData     *mdrv,
                       MatroxCrtc2LayerData *mcrtc2 )
 {
-     MatroxMavenData *mav = &mcrtc2->mav;
-     DFBResult        res;
+     MatroxDeviceData *mdev = mdrv->device_data;
+     MatroxMavenData  *mav  = &mcrtc2->mav;
+     DFBResult         res;
 
      if ((res = maven_open( mav, mdrv )) != DFB_OK)
           return res;
 
      maven_disable( mav, mdrv );
-     if (!mav->g450)
+     if (!mdev->g450_matrox)
           crtc2_set_mafc( mdrv, 0 );
      crtc2OnOff( mdrv, mcrtc2, 0 );
 
      maven_close( mav, mdrv );
 
-     if (mav->g450) {
+     if (mdev->g450_matrox) {
           volatile __u8 *mmio = mdrv->mmio_base;
           __u8           val;
 
-          /*
-           * dac2pdn = 0
-           * cfifopdn = 0
-           */
+          /* Set Rset to 0.7 V */
+          val = mga_in_dac( mmio, XGENIOCTRL );
+          val &= ~0x40;
+          mga_out_dac( mmio, XGENIOCTRL, val );
+          val = mga_in_dac( mmio, XGENIODATA );
+          val &= ~0x40;
+          mga_out_dac( mmio, XGENIODATA, val );
+
           val = mga_in_dac( mmio, XPWRCTRL );
-          val &= ~((1 << 4) | (1 << 0));
+          val &= ~(DAC2PDN | CFIFOPDN);
           mga_out_dac( mmio, XPWRCTRL, val );
 
-          /* dac2outsel = 00 (disable) */
           val = mga_in_dac( mmio, XDISPCTRL );
-          val &= ~(3 << 2);
+          val &= ~DAC2OUTSEL_MASK;
+          val |= DAC2OUTSEL_DIS;
           mga_out_dac( mmio, XDISPCTRL, val );
      }
 
@@ -703,21 +635,22 @@ crtc2_disable_output( MatroxDriverData     *mdrv,
 }
 
 static DFBResult
-crtc2_enable_output( MatroxDriverData     *mdrv,
-                     MatroxCrtc2LayerData *mcrtc2,
-                     DisplayLayer         *layer )
+crtc2_enable_output( MatroxDriverData      *mdrv,
+                     MatroxCrtc2LayerData  *mcrtc2 )
 {
-     MatroxMavenData *mav  = &mcrtc2->mav;
-     volatile __u8   *mmio = mdrv->mmio_base;
-     DFBResult        res;
+     MatroxDeviceData *mdev = mdrv->device_data;
+     MatroxMavenData  *mav  = &mcrtc2->mav;
+     volatile __u8    *mmio = mdrv->mmio_base;
+     DFBResult         res;
 
      if ((res = maven_open( mav, mdrv )) != DFB_OK)
           return res;
 
-     if (mav->g450) {
+     if (mdev->g450_matrox) {
           volatile __u8 *mmio = mdrv->mmio_base;
           __u8           val;
 
+          /* Set Rset to 1.0 V */
           val = mga_in_dac( mmio, XGENIOCTRL );
           val |= 0x40;
           mga_out_dac( mmio, XGENIOCTRL, val );
@@ -725,50 +658,39 @@ crtc2_enable_output( MatroxDriverData     *mdrv,
           val &= ~0x40;
           mga_out_dac( mmio, XGENIODATA, val );
 
-          /*
-           * dac2pdn = 1
-           * cfifopdn = 1
-           */
           val = mga_in_dac( mmio, XPWRCTRL );
-          val |= (1 << 4) | (1 << 0);
+          val |= DAC2PDN | CFIFOPDN;
           mga_out_dac( mmio, XPWRCTRL, val );
 
-          /* dac2outsel = 11 (TVE) */
           val = mga_in_dac( mmio, XDISPCTRL );
-          val |= (3 << 2);
+          val &= ~DAC2OUTSEL_MASK;
+          val |= DAC2OUTSEL_TVE;
           mga_out_dac( mmio, XDISPCTRL, val );
 
           if (dfb_config->matrox_cable == 1) {
-               /*
-                * dac2hsoff = 0
-                * dac2vsoff = 0
-                * dac2hspol = 0 (+)
-                * dac2vspol = 0 (+)
-                */
                val = mga_in_dac( mmio, XSYNCCTRL );
-               val &= 0x0F;
+               val &= ~(DAC2HSOFF | DAC2VSOFF | DAC2HSPOL | DAC2VSPOL);
                mga_out_dac( mmio, XSYNCCTRL, val );
           }
      }
 
      maven_disable( mav, mdrv );
-     if (!mav->g450)
+     if (!mdev->g450_matrox)
           crtc2_set_mafc( mdrv, 0 );
      crtc2OnOff( mdrv, mcrtc2, 0 );
 
      crtc2_set_regs( mdrv, mcrtc2 );
-     crtc2_set_buffer( mdrv, mcrtc2, layer );
+     crtc2_set_buffer( mdrv, mcrtc2 );
 
-     if (!mav->g450)
+     if (!mdev->g450_matrox)
           crtc2_set_mafc( mdrv, 1 );
      crtc2OnOff( mdrv, mcrtc2, 1 );
 
      maven_set_regs( mav, mdrv, &mcrtc2->config, &mcrtc2->adj );
 
-     /* c2interlace = 1 */
-     mcrtc2->regs.c2CTL |= (1 << 25);
-     if (mav->g450)
-          mcrtc2->regs.c2CTL |= (1 << 12);
+     mcrtc2->regs.c2CTL |= C2INTERLACE;
+     if (mdev->g450_matrox)
+          mcrtc2->regs.c2CTL |= 0x1000; /* Undocumented bit */
      while ((mga_in32( mmio, C2VCOUNT ) & 0x00000FFF) != 1)
           ;
      while ((mga_in32( mmio, C2VCOUNT ) & 0x00000FFF) != 0)
@@ -777,7 +699,7 @@ crtc2_enable_output( MatroxDriverData     *mdrv,
 
      maven_enable( mav, mdrv );
 
-     if (!mav->g450) {
+     if (!mdev->g450_matrox) {
           while ((mga_in32( mmio, C2VCOUNT ) & 0x00000FFF) != 1)
                ;
           while ((mga_in32( mmio, C2VCOUNT ) & 0x00000FFF) != 0)
