@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2004-2005 Claudio Ciccani <klan82@cheapnet.it>
+   Copyright (C) 2004-2005 Claudio Ciccani <klan@users.sf.net>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -32,9 +32,6 @@ bool nvDrawLine3D( void *drv, void *dev, DFBRegion *line );
 bool nvTextureTriangles( void *drv, void *dev, DFBVertex *vertices,
                          int num, DFBTriangleFormation formation );
 
-void nv_put_texture( NVidiaDriverData *nvdrv,
-                     NVidiaDeviceData *nvdev, SurfaceBuffer *source );
-
 
 #endif /* __NVIDIA_3D_H__ */
 
