@@ -37,6 +37,9 @@
 
 #include <directfb.h>
 
+#include <direct/messages.h>
+#include <direct/util.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
@@ -193,7 +196,7 @@ static inline void savage4_validate_color( Savage4DeviceData *sdev,
                                                state->color.b);
                break;
           default:
-               BUG( "unexpected destination format" );
+               D_BUG( "unexpected destination format" );
                break;
      }
 
@@ -284,7 +287,7 @@ static void savage4SetState( void *drv, void *dev,
                break;
 
           default:
-               BUG( "unexpected drawing/blitting function!" );
+               D_BUG( "unexpected drawing/blitting function!" );
                return;
      }
 
@@ -510,11 +513,11 @@ savage4_init_driver( GraphicsDevice      *device,
      funcs->StretchBlit   = savage4StretchBlit;
 
      /* setup primary layer functions */
-     dfb_layers_hook_primary(device, driver_data, &savagePrimaryFuncs,
-                             &savage_pfuncs, &savage_pdriver_data);
+//     dfb_layers_hook_primary(device, driver_data, &savagePrimaryFuncs,
+//                             &savage_pfuncs, &savage_pdriver_data);
 
      /* setup secondary layer functions */
-     dfb_layers_register(device, driver_data, &savageSecondaryFuncs);
+ //    dfb_layers_register(device, driver_data, &savageSecondaryFuncs);
 
      return DFB_OK;
 }
