@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -38,6 +39,8 @@
 
 #include <directfb.h>
 
+#include <direct/messages.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
@@ -45,6 +48,8 @@
 #include <core/gfxcard.h>
 #include <core/surfaces.h>
 
+#include <core/accelerators.h>
+
 #include <gfx/convert.h>
 #include <gfx/util.h>
 #include <misc/conf.h>
@@ -117,7 +122,7 @@ static inline void tdfx_waitfifo( TDFXDriverData *tdrv,
      tdev->fifo_space -= space;
 
      if (!timeout)
-          CAUTION( "timeout during waitfifo!" );
+          D_WARN( "timeout during waitfifo!" );
 }
 
 static inline void tdfx_waitidle( TDFXDriverData *tdrv,
@@ -139,7 +144,7 @@ static inline void tdfx_waitidle( TDFXDriverData *tdrv,
 
      }
 
-     BUG( "timeout during waitidle!\n");
+     D_BUG( "timeout during waitidle!\n");
 }
 
 
@@ -221,7 +226,7 @@ static inline void tdfx_validate_destination3D( TDFXDriverData *tdrv,
                lfbmode |= TDFX_LFBMODE_ARGB8888;
                break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
 
@@ -289,7 +294,7 @@ static inline void tdfx_validate_colorFore( TDFXDriverData *tdrv,
                                                         state->color.b );
                break;
           default:
-               BUG( "unexpected pixelformat!" );
+               D_BUG( "unexpected pixelformat!" );
                break;
      }
 
@@ -491,7 +496,7 @@ static void tdfxSetState( void *drv, void *dev,
                break;
 
           default:
-               BUG( "unexpected drawing/blitting function" );
+               D_BUG( "unexpected drawing/blitting function" );
                break;
      }
 
@@ -605,7 +610,7 @@ static bool tdfxDrawLine2D( void *drv, void *dev, DFBRegion *line )
      voodoo3D->vertexBy = S12_4_( line->y1, yt );
 
      voodoo3D->triangleCMD = 0;
-     
+
      return true;
 }*/
 
@@ -737,7 +742,7 @@ driver_get_info( GraphicsDevice     *device,
 
      snprintf( info->vendor,
                DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               "directfb.org" );
 
      info->version.major = 0;
      info->version.minor = 1;
@@ -841,30 +846,30 @@ driver_close_device( GraphicsDevice *device,
      (void) tdev;
      (void) tdrv;
 
-     DEBUGMSG( "DirectFB/TDFX: FIFO Performance Monitoring:\n" );
-     DEBUGMSG( "DirectFB/TDFX:  %9d tdfx_waitfifo calls\n",
+     D_DEBUG( "DirectFB/TDFX: FIFO Performance Monitoring:\n" );
+     D_DEBUG( "DirectFB/TDFX:  %9d tdfx_waitfifo calls\n",
                tdev->waitfifo_calls );
-     DEBUGMSG( "DirectFB/TDFX:  %9d register writes (tdfx_waitfifo sum)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d register writes (tdfx_waitfifo sum)\n",
                tdev->waitfifo_sum );
-     DEBUGMSG( "DirectFB/TDFX:  %9d FIFO wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d FIFO wait cycles (depends on CPU)\n",
                tdev->fifo_waitcycles );
-     DEBUGMSG( "DirectFB/TDFX:  %9d IDLE wait cycles (depends on CPU)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d IDLE wait cycles (depends on CPU)\n",
                tdev->idle_waitcycles );
-     DEBUGMSG( "DirectFB/TDFX:  %9d FIFO space cache hits(depends on CPU)\n",
+     D_DEBUG( "DirectFB/TDFX:  %9d FIFO space cache hits(depends on CPU)\n",
                tdev->fifo_cache_hits );
-     DEBUGMSG( "DirectFB/TDFX: Conclusion:\n" );
-     DEBUGMSG( "DirectFB/TDFX:  Average register writes/tdfx_waitfifo"
+     D_DEBUG( "DirectFB/TDFX: Conclusion:\n" );
+     D_DEBUG( "DirectFB/TDFX:  Average register writes/tdfx_waitfifo"
                "call:%.2f\n",
                tdev->waitfifo_sum/(float)(tdev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/TDFX:  Average wait cycles/tdfx_waitfifo call:"
+     D_DEBUG( "DirectFB/TDFX:  Average wait cycles/tdfx_waitfifo call:"
                " %.2f\n",
                tdev->fifo_waitcycles/(float)(tdev->waitfifo_calls) );
-     DEBUGMSG( "DirectFB/TDFX:  Average fifo space cache hits: %02d%%\n",
+     D_DEBUG( "DirectFB/TDFX:  Average fifo space cache hits: %02d%%\n",
                (int)(100 * tdev->fifo_cache_hits/
                      (float)(tdev->waitfifo_calls)) );
 
-     DEBUGMSG( "DirectFB/TDFX:  Pixels Out: %d\n", tdrv->voodoo3D->fbiPixelsOut );
-     DEBUGMSG( "DirectFB/TDFX:  Triangles Out: %d\n", tdrv->voodoo3D->fbiTrianglesOut );
+     D_DEBUG( "DirectFB/TDFX:  Pixels Out: %d\n", tdrv->voodoo3D->fbiPixelsOut );
+     D_DEBUG( "DirectFB/TDFX:  Triangles Out: %d\n", tdrv->voodoo3D->fbiTrianglesOut );
 }
 
 static void
