@@ -40,6 +40,37 @@
      { DIDTF_NONE, "NONE" } \
 };
 
+#define DirectFBSurfaceDrawingFlagsNames(Identifier) struct DFBSurfaceDrawingFlagsName { \
+     DFBSurfaceDrawingFlags flag; \
+     const char *name; \
+} Identifier[] = { \
+     { DSDRAW_BLEND, "BLEND" }, \
+     { DSDRAW_DST_COLORKEY, "DST_COLORKEY" }, \
+     { DSDRAW_SRC_PREMULTIPLY, "SRC_PREMULTIPLY" }, \
+     { DSDRAW_DST_PREMULTIPLY, "DST_PREMULTIPLY" }, \
+     { DSDRAW_DEMULTIPLY, "DEMULTIPLY" }, \
+     { DSDRAW_XOR, "XOR" }, \
+     { DSDRAW_NOFX, "NOFX" } \
+};
+
+#define DirectFBSurfaceBlittingFlagsNames(Identifier) struct DFBSurfaceBlittingFlagsName { \
+     DFBSurfaceBlittingFlags flag; \
+     const char *name; \
+} Identifier[] = { \
+     { DSBLIT_BLEND_ALPHACHANNEL, "BLEND_ALPHACHANNEL" }, \
+     { DSBLIT_BLEND_COLORALPHA, "BLEND_COLORALPHA" }, \
+     { DSBLIT_COLORIZE, "COLORIZE" }, \
+     { DSBLIT_SRC_COLORKEY, "SRC_COLORKEY" }, \
+     { DSBLIT_DST_COLORKEY, "DST_COLORKEY" }, \
+     { DSBLIT_SRC_PREMULTIPLY, "SRC_PREMULTIPLY" }, \
+     { DSBLIT_DST_PREMULTIPLY, "DST_PREMULTIPLY" }, \
+     { DSBLIT_DEMULTIPLY, "DEMULTIPLY" }, \
+     { DSBLIT_DEINTERLACE, "DEINTERLACE" }, \
+     { DSBLIT_SRC_PREMULTCOLOR, "SRC_PREMULTCOLOR" }, \
+     { DSBLIT_XOR, "XOR" }, \
+     { DSBLIT_NOFX, "NOFX" } \
+};
+
 #define DirectFBInputDeviceCapabilitiesNames(Identifier) struct DFBInputDeviceCapabilitiesName { \
      DFBInputDeviceCapabilities capability; \
      const char *name; \
@@ -228,4 +259,19 @@
      { DSESM_UNKNOWN, "UNKNOWN" } \
 };
 
+#define DirectFBAccelerationMaskNames(Identifier) struct DFBAccelerationMaskName { \
+     DFBAccelerationMask mask; \
+     const char *name; \
+} Identifier[] = { \
+     { DFXL_FILLRECTANGLE, "FILLRECTANGLE" }, \
+     { DFXL_DRAWRECTANGLE, "DRAWRECTANGLE" }, \
+     { DFXL_DRAWLINE, "DRAWLINE" }, \
+     { DFXL_FILLTRIANGLE, "FILLTRIANGLE" }, \
+     { DFXL_BLIT, "BLIT" }, \
+     { DFXL_STRETCHBLIT, "STRETCHBLIT" }, \
+     { DFXL_TEXTRIANGLES, "TEXTRIANGLES" }, \
+     { DFXL_DRAWSTRING, "DRAWSTRING" }, \
+     { DFXL_NONE, "NONE" } \
+};
+
 #endif
