@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -41,9 +42,11 @@
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 #include <core/input.h>
-#include <core/thread.h>
 
-#include <misc/mem.h>
+#include <direct/debug.h>
+#include <direct/mem.h>
+#include <direct/messages.h>
+#include <direct/thread.h>
 
 #include <core/input_driver.h>
 
@@ -139,10 +142,10 @@ static KeyCode keycodes_old_remote[] = {
  * declaration of private data
  */
 typedef struct {
-     InputDevice *device;
-     CoreThread  *thread;
-     
-     int          fd;
+     CoreInputDevice  *device;
+     DirectThread *thread;
+
+     int           fd;
 } Dbox2remoteData;
 
 
@@ -177,7 +180,7 @@ dbox2remote_parse_rccode( __u16 rccode )
  * Generates events on incoming data.
  */
 static void*
-dbox2remoteEventThread( CoreThread *thread, void *driver_data )
+dbox2remoteEventThread( DirectThread *thread, void *driver_data )
 {
      Dbox2remoteData *data = (Dbox2remoteData*) driver_data;
      int              readlen;
@@ -185,7 +188,7 @@ dbox2remoteEventThread( CoreThread *thread, void *driver_data )
      DFBInputEvent    evt;
 
      while ((readlen = read( data->fd, &rccode, 2 )) == 2) {
-          dfb_thread_testcancel( thread );
+          direct_thread_testcancel( thread );
 
           /* translate rccode to DirectFB keycode */
           evt.key_symbol = dbox2remote_parse_rccode( rccode );
@@ -203,7 +206,7 @@ dbox2remoteEventThread( CoreThread *thread, void *driver_data )
      }
 
      if (readlen <= 0 && errno != EINTR)
-          PERRORMSG ("dbox2remote thread died\n");
+          D_PERROR ("dbox2remote thread died\n");
 
      return NULL;
 }
@@ -247,7 +250,7 @@ driver_get_info( InputDriverInfo *info )
  * Called during initialization, resuming or taking over mastership.
  */
 static DFBResult
-driver_open_device( InputDevice      *device,
+driver_open_device( CoreInputDevice      *device,
                     unsigned int      number,
                     InputDeviceInfo  *info,
                     void            **driver_data )
@@ -258,7 +261,7 @@ driver_open_device( InputDevice      *device,
      /* open device */
      fd = open( DEVICE, O_RDONLY);
      if (fd < 0) {
-          PERRORMSG( "DirectFB/dbox2remote: could not open device" );
+          D_PERROR( "DirectFB/dbox2remote: could not open device" );
           return DFB_INIT;
      }
 
@@ -285,14 +288,13 @@ driver_open_device( InputDevice      *device,
 
 
      /* allocate and fill private data */
-     data = DFBCALLOC( 1, sizeof(Dbox2remoteData) );
+     data = D_CALLOC( 1, sizeof(Dbox2remoteData) );
 
      data->fd     = fd;
      data->device = device;
 
      /* start input thread */
-     data->thread = dfb_thread_create( CTT_INPUT,
-                                       dbox2remoteEventThread, data );
+     data->thread = direct_thread_create( DTT_INPUT, dbox2remoteEventThread, data, "DBOX2 Input" );
 
      /* set private data pointer */
      *driver_data = data;
@@ -304,7 +306,7 @@ driver_open_device( InputDevice      *device,
  * Fetch one entry from the device's keymap if supported.
  */
 static DFBResult
-driver_get_keymap_entry( InputDevice               *device,
+driver_get_keymap_entry( CoreInputDevice               *device,
                          void                      *driver_data,
                          DFBInputDeviceKeymapEntry *entry )
 {
@@ -320,14 +322,14 @@ driver_close_device( void *driver_data )
      Dbox2remoteData *data = (Dbox2remoteData*) driver_data;
 
      /* stop input thread */
-     dfb_thread_cancel( data->thread );
-     dfb_thread_join( data->thread );
-     dfb_thread_destroy( data->thread );
+     direct_thread_cancel( data->thread );
+     direct_thread_join( data->thread );
+     direct_thread_destroy( data->thread );
 
      /* close file */
      close( data->fd );
 
      /* free private data */
-     DFBFREE ( data );
+     D_FREE ( data );
 }
 
