@@ -140,7 +140,7 @@ int basic_keycodes [] = {
      DFB_FUNCTION_KEY(16), DFB_FUNCTION_KEY(17), DFB_FUNCTION_KEY(18),
      DFB_FUNCTION_KEY(19), DFB_FUNCTION_KEY(20),
 
-     DIKI_KP_ENTER, DIKI_CONTROL_R, DIKI_KP_DIV, DIKI_PRINT, DIKI_ALT_R,
+     DIKI_KP_ENTER, DIKI_CONTROL_R, DIKI_KP_DIV, DIKI_PRINT, DIKI_ALTGR,
 
      /*KEY_LINEFEED*/ DIKI_UNKNOWN,
 
@@ -204,18 +204,15 @@ int basic_keycodes [] = {
      /*KEY_KPLEFTPAREN,*/ DIKI_UNKNOWN,
      /*KEY_KPRIGHTPAREN,*/ DIKI_UNKNOWN,
 
-     /*KEY_INTL1,  KEY_INTL2, KEY_INTL3, KEY_INTL4, KEY_INTL5, KEY_INTL6,
-     KEY_INTL7, KEY_INTL8, KEY_INTL9,*/
      DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
      DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
 
-     /*KEY_LANG1, KEY_LANG2, KEY_LANG3, KEY_LANG4, KEY_LANG5, KEY_LANG6,
-     KEY_LANG7, KEY_LANG8, KEY_LANG9,*/
      DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
      DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
 
      DIKI_UNKNOWN,
 
+     /* KEY_PLAYCD, KEY_PAUSECD */
      DIKS_PLAY, DIKS_PAUSE,
 
      /*KEY_PROG3, KEY_PROG4,*/
@@ -223,9 +220,37 @@ int basic_keycodes [] = {
 
      DIKI_UNKNOWN,
 
-     /*KEY_SUSPEND, KEY_CLOSE*/ DIKI_UNKNOWN, DIKI_UNKNOWN,
+     /*KEY_SUSPEND, KEY_CLOSE*/
+     DIKI_UNKNOWN, DIKI_UNKNOWN,
 
-     DIKS_PLAY
+     /* KEY_PLAY */
+     DIKS_PLAY,
+     
+     /* KEY_FASTFORWARD */
+     DIKS_FASTFORWARD,
+     
+     /* KEY_BASSBOOST */
+     DIKI_UNKNOWN,
+     
+     /* KEY_PRINT */
+     DIKS_PRINT,
+     
+     /* KEY_HP             */  DIKI_UNKNOWN,
+     /* KEY_CAMERA         */  DIKI_UNKNOWN,
+     /* KEY_SOUND          */  DIKS_AUDIO,
+     /* KEY_QUESTION       */  DIKS_HELP,
+     /* KEY_EMAIL          */  DIKS_MAIL,
+     /* KEY_CHAT           */  DIKI_UNKNOWN,
+     /* KEY_SEARCH         */  DIKI_UNKNOWN,
+     /* KEY_CONNECT        */  DIKI_UNKNOWN,
+     /* KEY_FINANCE        */  DIKI_UNKNOWN,
+     /* KEY_SPORT          */  DIKI_UNKNOWN,
+     /* KEY_SHOP           */  DIKI_UNKNOWN,
+     /* KEY_ALTERASE       */  DIKI_UNKNOWN,
+     /* KEY_CANCEL         */  DIKS_CANCEL,
+     /* KEY_BRIGHTNESSDOWN */  DIKI_UNKNOWN,
+     /* KEY_BRIGHTNESSUP   */  DIKI_UNKNOWN,
+     /* KEY_MEDIA          */  DIKI_UNKNOWN,
 };
 
 static
@@ -351,6 +376,9 @@ keyboard_get_symbol( int                             code,
      }
 
      /* special keys not in the map, hack? */
+     if (code == 99)
+          return DIKS_PRINT;
+
      if (code == 125)         /* left windows key */
           return DIKS_META;
 
@@ -442,7 +470,11 @@ static int
 key_event( struct input_event *levt,
            DFBInputEvent      *devt )
 {
-     if (levt->code >= BTN_MOUSE && levt->code <= BTN_STYLUS2) {
+     /* map touchscreen and smartpad events to button mouse */
+     if (levt->code == BTN_TOUCH || levt->code == BTN_TOOL_FINGER)
+          levt->code = BTN_MOUSE;
+
+     if (levt->code >= BTN_MOUSE && levt->code < BTN_JOYSTICK) {
           devt->type   = levt->value ? DIET_BUTTONPRESS : DIET_BUTTONRELEASE;
           /* don't set DIEF_BUTTONS, it will be set by the input core */
           devt->button = DIBI_FIRST + levt->code - BTN_MOUSE;
@@ -468,6 +500,9 @@ key_event( struct input_event *levt,
           devt->key_code = levt->code;
      }
 
+     if (levt->value == 2)
+          devt->flags |= DIEF_REPEAT;
+
      return 1;
 }
 
@@ -530,7 +565,7 @@ abs_event( struct input_event *levt,
                break;
 
           default:
-               if (levt->code > ABS_MAX || levt->code > DIAI_LAST)
+               if (levt->code >= ABS_PRESSURE || levt->code > DIAI_LAST)
                     return 0;
                devt->axis = levt->code;
      }
@@ -663,7 +698,7 @@ get_device_info( int              fd,
                if (test_bit( i, keybit ))
                     num_ext_keys++;
 
-          for (i=BTN_MISC; i<KEY_OK; i++)
+          for (i=BTN_MOUSE; i<BTN_JOYSTICK; i++)
                if (test_bit( i, keybit ))
                     num_buttons++;
      }
@@ -685,13 +720,15 @@ get_device_info( int              fd,
           /* get bits for absolute axes */
           ioctl( fd, EVIOCGBIT(EV_ABS, ABS_MAX), absbit );
 
-          for (i=0; i<ABS_MAX; i++)
+          for (i=0; i<ABS_PRESSURE; i++)
                if (test_bit( i, absbit ))
                     num_abs++;
      }
 
-     /* Mouse or Touchscreen? */
-     if ((num_rels && num_buttons)  ||  (num_abs && (num_buttons == 1)))
+     /* Mouse, Touchscreen or Smartpad ? */
+     if ((test_bit( EV_KEY, evbit ) &&
+          (test_bit( BTN_TOUCH, keybit ) || test_bit( BTN_TOOL_FINGER, keybit ))) ||
+          ((num_rels >= 2 && num_buttons)  ||  (num_abs == 2 && (num_buttons == 1))))
           info->desc.type |= DIDTF_MOUSE;
      else if (num_abs && num_buttons) /* Or a Joystick? */
           info->desc.type |= DIDTF_JOYSTICK;
@@ -765,10 +802,18 @@ driver_get_available()
           else {
                InputDeviceInfo info;
 
+               /* try to grab the device */
+               /* 2.4 kernels don't have EVIOCGRAB so ignore EINVAL */
+               if (ioctl( fd, EVIOCGRAB, 1 ) && errno != EINVAL) {
+                    close( fd );
+                    continue;
+               }
+
                memset( &info, 0, sizeof(InputDeviceInfo) );
 
                get_device_info( fd, &info );
 
+               ioctl( fd, EVIOCGRAB, 0 );
                close( fd );
 
                if (!dfb_config->linux_input_ir_only ||
@@ -845,17 +890,12 @@ driver_open_device( CoreInputDevice  *device,
 
      /* check if the device has LEDs */
      ret = ioctl( fd, EVIOCGBIT(EV_LED, LED_MAX), ledbit );
-     if (ret < 0) {
+     if (ret < 0)
           D_PERROR( "DirectFB/linux_input: could not get LED bits" );
-          ioctl( fd, EVIOCGRAB, 0 );
-          close( fd );
-          D_FREE( data );
-          return DFB_INIT;
-     }
-
-     data->has_leds = test_bit( LED_SCROLLL, ledbit ) ||
-                      test_bit( LED_NUML, ledbit ) ||
-                      test_bit( LED_CAPSL, ledbit );
+     else
+          data->has_leds = test_bit( LED_SCROLLL, ledbit ) ||
+                           test_bit( LED_NUML, ledbit ) ||
+                           test_bit( LED_CAPSL, ledbit );
 
      if (data->has_leds) {
           /* get LED state */
