@@ -91,8 +91,8 @@ typedef struct {
 } FT2ImplData;
 
 typedef struct {
-     char x;
-     char y;
+     signed char x;
+     signed char y;
 } KerningCacheEntry;
 
 typedef struct {
@@ -170,8 +170,15 @@ render_glyph( CoreFont      *thiz,
                case ft_pixel_mode_grays:
                     switch (surface->format) {
                          case DSPF_ARGB:
-                              for (i=0; i<info->width; i++)
-                                   dst32[i] = (src[i] << 24) | 0xFFFFFF;
+                              if (thiz->surface_caps & DSCAPS_PREMULTIPLIED) {
+                                   for (i=0; i<info->width; i++)
+                                        dst32[i] = ((src[i] << 24) |
+                                                    (src[i] << 16) |
+                                                    (src[i] <<  8) | src[i]);
+                              }
+                              else
+                                   for (i=0; i<info->width; i++)
+                                        dst32[i] = (src[i] << 24) | 0xFFFFFF;
                               break;
                          case DSPF_AiRGB:
                               for (i=0; i<info->width; i++)
@@ -394,8 +401,8 @@ init_kerning_cache( FT2ImplKerningData *data )
 
                pthread_mutex_unlock ( &library_mutex );
 
-               cache->x = (char) (vector.x >> 6);
-               cache->y = (char) (vector.y >> 6);
+               cache->x = (signed char) (vector.x >> 6);
+               cache->y = (signed char) (vector.y >> 6);
           }
      }
 }
@@ -495,7 +502,13 @@ Probe( IDirectFBFont_ProbeContext *ctx )
      }
 
      pthread_mutex_lock ( &library_mutex );
-     err = FT_New_Face( library, ctx->filename, -1, &face );
+     /*
+      * This should be
+      * err = FT_New_Face( library, ctx->filename, -1, NULL );
+      * but due to freetype bugs it doesn't work.
+      */
+     err = FT_New_Face( library, ctx->filename, 0, &face );
+     FT_Done_Face( face );
      pthread_mutex_unlock ( &library_mutex );
 
      release_freetype();
@@ -606,11 +619,23 @@ Construct( IDirectFBFont      *thiz,
      }
 #endif
 
-     if (desc->flags & (DFDESC_HEIGHT | DFDESC_WIDTH)) {
+     if (desc->flags & (DFDESC_HEIGHT       | DFDESC_WIDTH |
+                        DFDESC_FRACT_HEIGHT | DFDESC_FRACT_WIDTH))
+     {
+          int fw = 0, fh = 0;
+
+          if (desc->flags & DFDESC_FRACT_HEIGHT)
+               fh = desc->fract_height;
+          else if (desc->flags & DFDESC_HEIGHT)
+               fh = desc->height << 6;
+
+          if (desc->flags & DFDESC_FRACT_WIDTH)
+               fw = desc->fract_width;
+          else if (desc->flags & DFDESC_WIDTH)
+               fw = desc->width << 6;
+
           pthread_mutex_lock ( &library_mutex );
-          err = FT_Set_Pixel_Sizes( face,
-                                    (desc->flags & DFDESC_WIDTH)  ? desc->width  : 0,
-                                    (desc->flags & DFDESC_HEIGHT) ? desc->height : 0 );
+          err = FT_Set_Char_Size( face, fw, fh, 0, 0 );
           pthread_mutex_unlock ( &library_mutex );
           if (err) {
                D_ERROR( "DirectB/FontFT2: "
