@@ -57,9 +57,12 @@
 #define D_FLAGS_CLEAR(flags,f)     do { (flags) &= ~(f); } while (0)
 #define D_FLAGS_IS_SET(flags,f)    (((flags) & (f)) != 0)
 #define D_FLAGS_ARE_SET(flags,f)   (((flags) & (f)) == (f))
-#define D_FLAGS_ARE_IN(flags,f)    (((flags) & ~(f)) != 0)
+#define D_FLAGS_ARE_IN(flags,f)    (((flags) & ~(f)) == 0)
+#define D_FLAGS_INVALID(flags,f)   (((flags) & ~(f)) != 0)
 
-#define D_ARRAY_SIZE(array)        (sizeof(array) / sizeof((array)[0]))
+#define D_FLAGS_ASSERT(flags,f)    D_ASSERT( D_FLAGS_ARE_IN(flags,f) )
+
+#define D_ARRAY_SIZE(array)        ((int)(sizeof(array) / sizeof((array)[0])))
 
 
 #if __GNUC__ >= 3
@@ -76,7 +79,7 @@ DirectResult errno2result( int erno );
 
 const char *DirectResultString( DirectResult result );
 
-int direct_try_open( const char *name1, const char *name2, int flags );
+int direct_try_open( const char *name1, const char *name2, int flags, bool error_msg );
 
 void direct_trim( char **s );
 
@@ -96,6 +99,23 @@ direct_util_count_bits( unsigned int mask )
      return ret;
 }
 
+/*
+ * Generic alignment routine.
+ */
+static inline int
+direct_util_align( int value,
+                   int alignment )
+{
+     if (alignment > 1) {
+          int tail = value % alignment;
+
+          if (tail)
+               value += alignment - tail;
+     }
+
+     return value;
+}
+
 /*
  * Utility function to initialize recursive mutexes.
  */
@@ -106,8 +126,9 @@ direct_util_recursive_pthread_mutex_init( pthread_mutex_t *mutex )
      pthread_mutexattr_t attr;
 
      pthread_mutexattr_init( &attr );
+#ifdef __USE_UNIX98
      pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE );
-
+#endif
      ret = pthread_mutex_init( mutex, &attr );
      if (ret)
           D_PERROR( "Fusion/Lock: Could not initialize recursive mutex!\n" );
