@@ -150,7 +150,9 @@ dfb_gfxcard_initialize( CoreDFB *core, void *data_local, void *data_shared )
      direct_modules_explore_directory( &dfb_graphics_drivers );
 
      /* Load driver */
-     dfb_gfxcard_find_driver();
+     if (dfb_system_caps() & CSCAPS_ACCELERATION)
+          dfb_gfxcard_find_driver();
+
      if (card->driver_funcs) {
           const GraphicsDriverFuncs *funcs = card->driver_funcs;
 
@@ -228,7 +230,9 @@ dfb_gfxcard_join( CoreDFB *core, void *data_local, void *data_shared )
      direct_modules_explore_directory( &dfb_graphics_drivers );
 
      /* Load driver. */
-     dfb_gfxcard_load_driver();
+     if (dfb_system_caps() & CSCAPS_ACCELERATION)
+          dfb_gfxcard_load_driver();
+
      if (card->driver_funcs) {
           const GraphicsDriverFuncs *funcs = card->driver_funcs;
 
@@ -414,6 +418,12 @@ dfb_gfxcard_state_check( CardState *state, DFBAccelerationMask accel )
      if (!card->funcs.CheckState)
           return false;
 
+     /*
+      * Check if this function has been disabled temporarily.
+      */
+     if (state->disabled & accel)
+          return false;
+
      /* Destination may have been destroyed. */
      if (!state->destination) {
           D_BUG( "no destination" );
@@ -450,7 +460,7 @@ dfb_gfxcard_state_check( CardState *state, DFBAccelerationMask accel )
       * If the front buffer policy of the source is 'system only'
       * no accelerated blitting is available.
       */
-     if (state->source &&
+     if (!(card->caps.flags & CCF_READSYSMEM) && state->source &&
          state->source->front_buffer->policy == CSP_SYSTEMONLY)
      {
           /* Clear 'accelerated blitting functions'. */
@@ -737,31 +747,37 @@ build_clipped_rectangle_outlines( DFBRectangle    *rect,
           out->h = 1;
      }
 
-     if (edges & DFEF_LEFT) {
-          DFBRectangle *out = &ret_outlines[num++];
+     if (rect->h > t) {
+          if (edges & DFEF_BOTTOM) {
+               DFBRectangle *out = &ret_outlines[num++];
 
-          out->x = rect->x;
-          out->y = rect->y + t;
-          out->w = 1;
-          out->h = rect->h - tb;
-     }
+               out->x = rect->x;
+               out->y = rect->y + rect->h - 1;
+               out->w = rect->w;
+               out->h = 1;
+          }
 
-     if (edges & DFEF_RIGHT) {
-          DFBRectangle *out = &ret_outlines[num++];
+          if (rect->h > tb) {
+               if (edges & DFEF_LEFT) {
+                    DFBRectangle *out = &ret_outlines[num++];
 
-          out->x = rect->x + rect->w - 1;
-          out->y = rect->y + t;
-          out->w = 1;
-          out->h = rect->h - tb;
-     }
+                    out->x = rect->x;
+                    out->y = rect->y + t;
+                    out->w = 1;
+                    out->h = rect->h - tb;
+               }
 
-     if (edges & DFEF_BOTTOM) {
-          DFBRectangle *out = &ret_outlines[num++];
+               if (rect->w > 1 || !(edges & DFEF_LEFT)) {
+                    if (edges & DFEF_RIGHT) {
+                         DFBRectangle *out = &ret_outlines[num++];
 
-          out->x = rect->x;
-          out->y = rect->y + rect->h - 1;
-          out->w = rect->w;
-          out->h = 1;
+                         out->x = rect->x + rect->w - 1;
+                         out->y = rect->y + t;
+                         out->w = 1;
+                         out->h = rect->h - tb;
+                    }
+               }
+          }
      }
 
      *ret_num = num;
@@ -960,16 +976,13 @@ fill_tri( DFBTriangle *tri, CardState *state, bool accelerated )
      y = tri->y1;
      yend = tri->y3;
 
-     if (y < state->clip.y1)
-          y = state->clip.y1;
-
      if (yend > state->clip.y2)
           yend = state->clip.y2;
 
      SETUP_DDA(tri->x1, tri->y1, tri->x3, tri->y3, dda1);
      SETUP_DDA(tri->x1, tri->y1, tri->x2, tri->y2, dda2);
 
-     while (y < yend) {
+     while (y <= yend) {
           DFBRectangle rect;
 
           if (y == tri->y2) {
@@ -982,19 +995,23 @@ fill_tri( DFBTriangle *tri, CardState *state, bool accelerated )
           rect.x = MIN(dda1.xi, dda2.xi);
 
           if (clip_x2 < rect.x + rect.w)
-               rect.w = clip_x2 - rect.x;
+               rect.w = clip_x2 - rect.x + 1;
 
           if (rect.w > 0) {
-               if (clip_x1 > rect.x)
+               if (clip_x1 > rect.x) {
+                    rect.w -= (clip_x1 - rect.x);
                     rect.x = clip_x1;
+               }
                rect.y = y;
                rect.h = 1;
 
-               if (accelerated)
-                    card->funcs.FillRectangle( card->driver_data,
-                                               card->device_data, &rect );
-               else
-                    gFillRectangle( state, &rect );
+               if (rect.y >=  state->clip.y1) {
+                    if (accelerated)
+                         card->funcs.FillRectangle( card->driver_data,
+                                                    card->device_data, &rect );
+                    else
+                         gFillRectangle( state, &rect );
+               }
           }
 
           INC_DDA(dda1);
@@ -1332,9 +1349,66 @@ void dfb_gfxcard_texture_triangles( DFBVertex *vertices, int num,
      dfb_state_unlock( state );
 }
 
-void dfb_gfxcard_drawstring( const __u8 *text, int bytes,
-                             int x, int y,
-                             CoreFont *font, CardState *state )
+static void
+setup_font_state( CoreFont *font, CardState *state )
+{
+     DFBSurfaceBlittingFlags flags = font->state.blittingflags;
+
+     /* set destination */
+     dfb_state_set_destination( &font->state, state->destination );
+
+     /* set clip */
+     dfb_state_set_clip( &font->state, &state->clip );
+
+     /* set color */
+     if (DFB_PIXELFORMAT_IS_INDEXED( state->destination->format ))
+          dfb_state_set_color_index( &font->state, state->color_index );
+     else
+          dfb_state_set_color( &font->state, &state->color );
+
+     /* additional blending? */
+     if ((state->drawingflags & DSDRAW_BLEND) && (state->color.a != 0xff))
+          flags |= DSBLIT_BLEND_COLORALPHA;
+
+     if (state->drawingflags & DSDRAW_DST_COLORKEY) {
+          flags |= DSBLIT_DST_COLORKEY;
+          dfb_state_set_dst_colorkey( &font->state, state->dst_colorkey );
+     }
+
+     if (state->drawingflags & DSDRAW_XOR)
+          flags |= DSBLIT_XOR;
+
+     /* Porter/Duff SRC_OVER composition */
+     if ((DFB_PIXELFORMAT_HAS_ALPHA( state->destination->format )
+          && (state->destination->caps & DSCAPS_PREMULTIPLIED))
+         ||
+         (font->surface_caps & DSCAPS_PREMULTIPLIED))
+     {
+          if (font->surface_caps & DSCAPS_PREMULTIPLIED) {
+               if (flags & DSBLIT_BLEND_COLORALPHA)
+                    flags |= DSBLIT_SRC_PREMULTCOLOR;
+          }
+          else
+               flags |= DSBLIT_SRC_PREMULTIPLY;
+
+          dfb_state_set_src_blend( &font->state, DSBF_ONE );
+     }
+     else
+          dfb_state_set_src_blend( &font->state, DSBF_SRCALPHA );
+
+
+     /* set blitting flags */
+     dfb_state_set_blitting_flags( &font->state, flags );
+
+
+     /* set disabled functions */
+     font->state.disabled = state->disabled;
+}
+
+void
+dfb_gfxcard_drawstring( const __u8 *text, int bytes,
+                        int x, int y,
+                        CoreFont *font, CardState *state )
 {
      int            steps[bytes];
      unichar        chars[bytes];
@@ -1390,25 +1464,7 @@ void dfb_gfxcard_drawstring( const __u8 *text, int bytes,
           return;
      }
 
-     /* set destination */
-     dfb_state_set_destination( &font->state, state->destination );
-
-     /* set clip */
-     dfb_state_set_clip( &font->state, &state->clip );
-
-     /* set color */
-     if (DFB_PIXELFORMAT_IS_INDEXED( state->destination->format ))
-          dfb_state_set_color_index( &font->state, state->color_index );
-     else
-          dfb_state_set_color( &font->state, &state->color );
-
-     /* set blitting flags */
-     if (state->drawingflags & DSDRAW_BLEND)
-          dfb_state_set_blitting_flags( &font->state,
-                                        font->state.blittingflags | DSBLIT_BLEND_COLORALPHA );
-     else
-          dfb_state_set_blitting_flags( &font->state,
-                                        font->state.blittingflags & ~DSBLIT_BLEND_COLORALPHA );
+     setup_font_state( font, state );
 
      /* blit glyphs */
      for (offset = 0; offset < bytes; offset += steps[offset]) {
@@ -1525,25 +1581,8 @@ void dfb_gfxcard_drawglyph( unichar index, int x, int y,
           return;
      }
 
-     /* set destination */
-     dfb_state_set_destination( &font->state, state->destination );
+     setup_font_state( font, state );
 
-     /* set clip */
-     dfb_state_set_clip( &font->state, &state->clip );
-
-     /* set color */
-     if (DFB_PIXELFORMAT_IS_INDEXED( state->destination->format ))
-          dfb_state_set_color_index( &font->state, state->color_index );
-     else
-          dfb_state_set_color( &font->state, &state->color );
-
-     /* set blitting flags */
-     if (state->drawingflags & DSDRAW_BLEND)
-          dfb_state_set_blitting_flags( &font->state,
-                                        font->state.blittingflags | DSBLIT_BLEND_COLORALPHA );
-     else
-          dfb_state_set_blitting_flags( &font->state,
-                                        font->state.blittingflags & ~DSBLIT_BLEND_COLORALPHA );
 
      /* set blitting source */
      dfb_state_set_source( &font->state, data->surface );
@@ -1591,17 +1630,7 @@ void dfb_gfxcard_drawstring_check_state( CoreFont *font, CardState *state )
           return;
      }
 
-     /* set destination */
-     font->state.destination  = state->destination;
-     font->state.modified    |= SMF_DESTINATION;
-
-     /* set blitting flags */
-     if (state->drawingflags & DSDRAW_BLEND)
-          dfb_state_set_blitting_flags( &font->state,
-                                        font->state.blittingflags | DSBLIT_BLEND_COLORALPHA );
-     else
-          dfb_state_set_blitting_flags( &font->state,
-                                        font->state.blittingflags & ~DSBLIT_BLEND_COLORALPHA );
+     setup_font_state( font, state );
 
      /* set the source */
      dfb_state_set_source( &font->state, data->surface );
@@ -1612,8 +1641,6 @@ void dfb_gfxcard_drawstring_check_state( CoreFont *font, CardState *state )
      else
           state->accel &= ~DFXL_DRAWSTRING;
 
-     font->state.destination = NULL;
-
      dfb_font_unlock( font );
      dfb_state_unlock( state );
 }
@@ -1626,6 +1653,14 @@ void dfb_gfxcard_sync()
           card->funcs.EngineSync( card->driver_data, card->device_data );
 }
 
+void dfb_gfxcard_invalidate_state()
+{
+     D_ASSERT( card != NULL );
+     D_ASSERT( card->shared != NULL );
+
+     card->shared->state = NULL;
+}
+
 void dfb_gfxcard_wait_serial( const CoreGraphicsSerial *serial )
 {
      D_ASSERT( serial != NULL );
@@ -1675,13 +1710,38 @@ dfb_gfxcard_surface_manager()
 }
 
 void
-dfb_gfxcard_get_capabilities( CardCapabilities *caps )
+dfb_gfxcard_get_capabilities( CardCapabilities *ret_caps )
 {
      D_ASSERT( card != NULL );
 
-     *caps = card->caps;
+     D_ASSERT( ret_caps != NULL );
+
+     *ret_caps = card->caps;
 }
 
+void
+dfb_gfxcard_get_device_info( GraphicsDeviceInfo *ret_info )
+{
+     D_ASSERT( card != NULL );
+     D_ASSERT( card->shared != NULL );
+
+     D_ASSERT( ret_info != NULL );
+
+     *ret_info = card->shared->device_info;
+}
+
+void
+dfb_gfxcard_get_driver_info( GraphicsDriverInfo *ret_info )
+{
+     D_ASSERT( card != NULL );
+     D_ASSERT( card->shared != NULL );
+
+     D_ASSERT( ret_info != NULL );
+
+     *ret_info = card->shared->driver_info;
+}
+
+
 int
 dfb_gfxcard_reserve_memory( GraphicsDevice *device, unsigned int size )
 {
@@ -1758,9 +1818,6 @@ static void dfb_gfxcard_find_driver()
 {
      DirectLink *link;
 
-     if (dfb_system_type() != CORE_FBDEV)
-          return;
-
      direct_list_foreach (link, dfb_graphics_drivers.entries) {
           DirectModuleEntry *module = (DirectModuleEntry*) link;
 
@@ -1789,9 +1846,6 @@ static void dfb_gfxcard_load_driver()
 {
      DirectLink *link;
 
-     if (dfb_system_type() != CORE_FBDEV)
-          return;
-
      if (!card->shared->module_name)
           return;
 
