@@ -38,8 +38,9 @@
 #include <core/coretypes.h>
 
 typedef enum {
-     CCF_CLIPPING = 0x00000001,
-     CCF_NOTRIEMU = 0x00000002
+     CCF_CLIPPING   = 0x00000001,
+     CCF_NOTRIEMU   = 0x00000002,
+     CCF_READSYSMEM = 0x00000004
 } CardCapabilitiesFlags;
 
 struct __DFB_CoreGraphicsSerial {
@@ -47,14 +48,6 @@ struct __DFB_CoreGraphicsSerial {
      unsigned int generation;
 };
 
-/*
- * return value for hardware accelerated card functions
- */
-typedef enum {
-     CR_OK = 0,
-     CR_FALLBACK
-} CardResult;
-
 typedef struct {
      CardCapabilitiesFlags   flags;
 
@@ -306,19 +299,22 @@ void dfb_gfxcard_drawglyph( unichar index, int x, int y,
                             CoreFont *font, CardState *state );
 
 void dfb_gfxcard_sync();
+void dfb_gfxcard_invalidate_state();
 void dfb_gfxcard_wait_serial( const CoreGraphicsSerial *serial );
 void dfb_gfxcard_flush_texture_cache();
 void dfb_gfxcard_after_set_var();
 
 DFBResult dfb_gfxcard_adjust_heap_offset( int offset );
 
-SurfaceManager *dfb_gfxcard_surface_manager();
-void            dfb_gfxcard_get_capabilities( CardCapabilities *caps );
+SurfaceManager *dfb_gfxcard_surface_manager ();
+void            dfb_gfxcard_get_capabilities( CardCapabilities   *ret_caps );
+void            dfb_gfxcard_get_device_info ( GraphicsDeviceInfo *ret_info );
+void            dfb_gfxcard_get_driver_info ( GraphicsDriverInfo *ret_info );
 
-int             dfb_gfxcard_reserve_memory( GraphicsDevice *device,
-                                            unsigned int    size );
+int             dfb_gfxcard_reserve_memory  ( GraphicsDevice      *device,
+                                              unsigned int         size );
 
-unsigned int    dfb_gfxcard_memory_length();
+unsigned int    dfb_gfxcard_memory_length   ();
 
 /*
  * Graphics drivers call this function to get access to MMIO regions.
