@@ -846,12 +846,12 @@ dfb_layer_context_set_screenlocation( CoreLayerContext  *context,
           return DFB_FUSION;
 
      /* Do nothing if the location didn't change. */
-     if (context->screen.mode == CLLM_LOCATION &&
+/*     if (context->screen.mode == CLLM_LOCATION &&
          DFB_LOCATION_EQUAL( context->screen.location, *location ))
      {
           dfb_layer_context_unlock( context );
           return DFB_OK;
-     }
+     }*/
 
      /* Take the current configuration. */
      config = context->primary.config;
@@ -889,12 +889,12 @@ dfb_layer_context_set_screenrectangle( CoreLayerContext   *context,
           return DFB_FUSION;
 
      /* Do nothing if the location didn't change. */
-     if (context->screen.mode == CLLM_RECTANGLE &&
+/*     if (context->screen.mode == CLLM_RECTANGLE &&
          DFB_RECTANGLE_EQUAL( context->screen.rectangle, *rectangle ))
      {
           dfb_layer_context_unlock( context );
           return DFB_OK;
-     }
+     }*/
 
      /* Take the current configuration. */
      config = context->primary.config;
@@ -969,7 +969,7 @@ dfb_layer_context_set_opacity( CoreLayerContext *context,
           return DFB_FUSION;
 
      /* Do nothing if the opacity didn't change. */
-     if (context->opacity == opacity) {
+     if (context->primary.config.opacity == opacity) {
           dfb_layer_context_unlock( context );
           return DFB_OK;
      }
@@ -983,7 +983,7 @@ dfb_layer_context_set_opacity( CoreLayerContext *context,
      /* Try to set the new configuration. */
      ret = update_primary_region_config( context, &config, CLRCF_OPACITY );
      if (ret == DFB_OK)
-          context->opacity = opacity;
+          context->primary.config.opacity = opacity;
 
      /* Unlock the context. */
      dfb_layer_context_unlock( context );
@@ -1493,18 +1493,15 @@ reallocate_surface( CoreLayer             *layer,
                return ret;
      }
 
-     /* FIXME: remove this? */
-     if (config->options & DLOP_DEINTERLACING)
-          surface->caps |= DSCAPS_INTERLACED;
-     else
-          surface->caps &= ~DSCAPS_INTERLACED;
-
      /* Add available surface capabilities. */
      surface->caps &= ~(DSCAPS_INTERLACED | DSCAPS_SEPARATED | DSCAPS_PREMULTIPLIED);
      surface->caps |= config->surface_caps & (DSCAPS_INTERLACED |
                                               DSCAPS_SEPARATED  |
-                                              DSCAPS_PREMULTIPLIED);
-
+                                              DSCAPS_PREMULTIPLIED); 
+     /* FIXME: remove this? */
+     if (config->options & DLOP_DEINTERLACING)
+          surface->caps |= DSCAPS_INTERLACED;
+     
      return DFB_OK;
 }
 
