@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,7 +30,7 @@
 
 #include <directfb.h>
 #include <core/coretypes.h>
-#include <core/fusion/object.h>
+#include <fusion/object.h>
 
 struct _CorePalette {
      FusionObject  object;
@@ -41,7 +42,7 @@ struct _CorePalette {
           int      index;
           DFBColor color;
      } search_cache;
-  
+
      bool          hash_attached;
 };
 
@@ -58,7 +59,8 @@ typedef struct {
 } CorePaletteNotification;
 
 
-DFBResult    dfb_palette_create             ( unsigned int   size,
+DFBResult    dfb_palette_create             ( CoreDFB       *core,
+                                              unsigned int   size,
                                               CorePalette  **ret_palette );
 
 void         dfb_palette_generate_rgb332_map( CorePalette   *palette );
