@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -34,8 +35,8 @@
 
 #include <pthread.h>
 
-#include <core/fusion/list.h>
-#include <core/fusion/shmalloc.h>
+#include <direct/list.h>
+#include <fusion/shmalloc.h>
 
 #include <directfb.h>
 
@@ -48,9 +49,14 @@
 #include <core/surfacemanager.h>
 #include <core/system.h>
 
-#include <misc/util.h>
-#include <misc/mem.h>
-#include <misc/memcpy.h>
+#include <direct/debug.h>
+#include <direct/mem.h>
+#include <direct/memcpy.h>
+#include <direct/messages.h>
+#include <direct/util.h>
+
+
+D_DEBUG_DOMAIN( Core_SM, "Core/SurfaceMgr", "DirectFB Surface Manager" );
 
 /*
  * initially there is one big free chunk,
@@ -58,6 +64,8 @@
  * two chunks are merged to one free chunk if memory is deallocated
  */
 struct _Chunk {
+     int             magic;
+
      int             offset;      /* offset in video memory,
                                      is greater or equal to the heap offset */
      int             length;      /* length of this chunk in bytes */
@@ -73,6 +81,8 @@ struct _Chunk {
 };
 
 struct _SurfaceManager {
+     int             magic;
+
      FusionSkirmish  lock;
 
      Chunk          *chunks;
@@ -89,6 +99,11 @@ struct _SurfaceManager {
      /* card limitations for surface offsets and their pitch */
      unsigned int    byteoffset_align;
      unsigned int    pixelpitch_align;
+     unsigned int    bytepitch_align;
+
+     unsigned int    max_power_of_two_pixelpitch;
+     unsigned int    max_power_of_two_bytepitch;
+     unsigned int    max_power_of_two_height;
 };
 
 
@@ -98,9 +113,8 @@ static void occupy_chunk( SurfaceManager *manager, Chunk *chunk, SurfaceBuffer *
 
 
 SurfaceManager *
-dfb_surfacemanager_create( unsigned int length,
-                           unsigned int byteoffset_align,
-                           unsigned int pixelpitch_align )
+dfb_surfacemanager_create( unsigned int     length,
+                           CardLimitations *limits )
 {
      Chunk          *chunk;
      SurfaceManager *manager;
@@ -121,10 +135,18 @@ dfb_surfacemanager_create( unsigned int length,
      manager->chunks           = chunk;
      manager->length           = length;
      manager->available        = length;
-     manager->byteoffset_align = byteoffset_align;
-     manager->pixelpitch_align = pixelpitch_align;
+     manager->byteoffset_align = limits->surface_byteoffset_alignment;
+     manager->pixelpitch_align = limits->surface_pixelpitch_alignment;
+     manager->bytepitch_align  = limits->surface_bytepitch_alignment;
+     manager->max_power_of_two_pixelpitch = limits->surface_max_power_of_two_pixelpitch;
+     manager->max_power_of_two_bytepitch  = limits->surface_max_power_of_two_bytepitch;
+     manager->max_power_of_two_height     = limits->surface_max_power_of_two_height;
 
-     fusion_skirmish_init( &manager->lock );
+     fusion_skirmish_init( &manager->lock, "Surface Manager" );
+
+     D_MAGIC_SET( chunk, _Chunk_ );
+
+     D_MAGIC_SET( manager, SurfaceManager );
 
      return manager;
 }
@@ -134,14 +156,20 @@ dfb_surfacemanager_destroy( SurfaceManager *manager )
 {
      Chunk *chunk;
 
-     DFB_ASSERT( manager != NULL );
-     DFB_ASSERT( manager->chunks != NULL );
+     D_ASSERT( manager != NULL );
+     D_ASSERT( manager->chunks != NULL );
+
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
+     D_MAGIC_CLEAR( manager );
 
      /* Deallocate all chunks. */
      chunk = manager->chunks;
      while (chunk) {
           Chunk *next = chunk->next;
 
+          D_MAGIC_CLEAR( chunk );
+
           SHFREE( chunk );
 
           chunk = next;
@@ -149,7 +177,7 @@ dfb_surfacemanager_destroy( SurfaceManager *manager )
 
      /* Destroy manager lock. */
      fusion_skirmish_destroy( &manager->lock );
-     
+
      /* Deallocate manager struct. */
      SHFREE( manager );
 }
@@ -158,6 +186,8 @@ DFBResult dfb_surfacemanager_suspend( SurfaceManager *manager )
 {
      Chunk *c;
 
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
      dfb_surfacemanager_lock( manager );
 
      manager->suspended = true;
@@ -176,35 +206,43 @@ DFBResult dfb_surfacemanager_suspend( SurfaceManager *manager )
      }
 
      dfb_surfacemanager_unlock( manager );
-     
+
      return DFB_OK;
 }
 
 DFBResult dfb_surfacemanager_resume( SurfaceManager *manager )
 {
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
      dfb_surfacemanager_lock( manager );
-     
+
      manager->suspended = false;
-     
+
      dfb_surfacemanager_unlock( manager );
-     
+
      return DFB_OK;
 }
 
 void dfb_surfacemanager_lock( SurfaceManager *manager )
 {
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
      fusion_skirmish_prevail( &manager->lock );
 }
 
 void dfb_surfacemanager_unlock( SurfaceManager *manager )
 {
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
      fusion_skirmish_dismiss( &manager->lock );
 }
 
 DFBResult dfb_surfacemanager_adjust_heap_offset( SurfaceManager *manager,
                                                  int             offset )
 {
-     DFB_ASSERT( offset >= 0 );
+     D_ASSERT( offset >= 0 );
+
+     D_MAGIC_ASSERT( manager, SurfaceManager );
 
      dfb_surfacemanager_lock( manager );
 
@@ -221,13 +259,13 @@ DFBResult dfb_surfacemanager_adjust_heap_offset( SurfaceManager *manager,
                manager->chunks->offset = offset;
           }
           else {
-               CAUTION("unable to adjust heap offset");
+               D_WARN("unable to adjust heap offset");
                /* more space needed than free at the beginning */
                /* TODO: move/destroy instances */
           }
      }
      else {
-          CAUTION("unable to adjust heap offset");
+          D_WARN("unable to adjust heap offset");
           /* very rare case that the first chunk is occupied */
           /* TODO: move/destroy instances */
      }
@@ -246,8 +284,10 @@ dfb_surfacemanager_enumerate_chunks( SurfaceManager  *manager,
 {
      Chunk *c;
 
-     DFB_ASSERT( manager != NULL );
-     DFB_ASSERT( callback != NULL );
+     D_ASSERT( manager != NULL );
+     D_ASSERT( callback != NULL );
+
+     D_MAGIC_ASSERT( manager, SurfaceManager );
 
      dfb_surfacemanager_lock( manager );
 
@@ -278,26 +318,43 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
 
      CoreSurface *surface = buffer->surface;
 
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
      if (!manager->length || manager->suspended)
           return DFB_NOVIDEOMEMORY;
 
      /* calculate the required length depending on limitations */
      pitch = MAX( surface->width, surface->min_width );
+
+     if (pitch < manager->max_power_of_two_pixelpitch &&
+         surface->height < manager->max_power_of_two_height)
+          pitch = 1 << direct_log2( pitch );
+
      if (manager->pixelpitch_align > 1) {
           pitch += manager->pixelpitch_align - 1;
           pitch -= pitch % manager->pixelpitch_align;
      }
 
-     pitch = DFB_BYTES_PER_LINE( surface->format, pitch );
-     length = DFB_PLANE_MULTIPLY( surface->format,
-                                  MAX( surface->height,
-                                       surface->min_height ) * pitch );
+     pitch = DFB_BYTES_PER_LINE( buffer->format, pitch );
+
+     if (pitch < manager->max_power_of_two_bytepitch &&
+         surface->height < manager->max_power_of_two_height)
+          pitch = 1 << direct_log2( pitch );
+
+     if (manager->bytepitch_align > 1) {
+          pitch += manager->bytepitch_align - 1;
+          pitch -= pitch % manager->bytepitch_align;
+     }
+
+     length = DFB_PLANE_MULTIPLY( buffer->format,
+                                  MAX( surface->height, surface->min_height ) * pitch );
 
      if (manager->byteoffset_align > 1) {
           length += manager->byteoffset_align - 1;
           length -= length % manager->byteoffset_align;
      }
 
+     /* Do a pre check before iterating through all chunks. */
      if (length > manager->available - manager->heap_offset)
           return DFB_NOVIDEOMEMORY;
 
@@ -311,7 +368,7 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
                     c->tolerations++;
                     if (c->tolerations > 0xff)
                          c->tolerations = 0xff;
-                    
+
                     if (!c->buffer->video.locked              &&
                         c->buffer->policy <= buffer->policy   &&
                         c->buffer->policy != CSP_VIDEOONLY    &&
@@ -348,9 +405,9 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
      if (best_occupied) {
           CoreSurface *kicked = best_occupied->buffer->surface;
 
-          DEBUGMSG( "DirectFB/core/surfacemanager: Kicking out buffer at %d (%d) with tolerations %d...\n",
-                    best_occupied->offset,
-                    best_occupied->length, best_occupied->tolerations );
+          D_DEBUG_AT( Core_SM, "Kicking out buffer at %d (%d) with tolerations %d...\n",
+                      best_occupied->offset,
+                      best_occupied->length, best_occupied->tolerations );
 
           dfb_surfacemanager_assure_system( manager, best_occupied->buffer );
 
@@ -363,14 +420,12 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
 
 
           occupy_chunk( manager, best_occupied, buffer, length );
-          
+
           return DFB_OK;
      }
-     
-     
-     DEBUGMSG( "DirectFB/core/surfacemanager: "
-               "Couldn't allocate enough heap space "
-               "for video memory surface!\n" );
+
+
+     D_DEBUG_AT( Core_SM, "Couldn't allocate enough heap space for video memory surface!\n" );
 
      /* no luck */
      return DFB_NOVIDEOMEMORY;
@@ -382,13 +437,13 @@ DFBResult dfb_surfacemanager_deallocate( SurfaceManager *manager,
      int    loops = 0;
      Chunk *chunk = buffer->video.chunk;
 
-     DFB_ASSERT( buffer->surface );
+     D_ASSERT( buffer->surface );
+
+     D_MAGIC_ASSERT( manager, SurfaceManager );
 
      if (buffer->video.health == CSH_INVALID)
           return DFB_OK;
 
-     //DEBUGMSG( "deallocating...\n" );
-
      buffer->video.health = CSH_INVALID;
      buffer->video.chunk = NULL;
 
@@ -397,17 +452,15 @@ DFBResult dfb_surfacemanager_deallocate( SurfaceManager *manager,
      while (buffer->video.locked) {
           if (++loops > 1000)
                break;
-          
+
           sched_yield();
      }
 
      if (buffer->video.locked)
-          CAUTION( "Freeing chunk with a non-zero lock counter" );
+          D_WARN( "Freeing chunk with a non-zero lock counter" );
 
      if (chunk)
           free_chunk( manager, chunk );
-     
-     //DEBUGMSG( "deallocated.\n" );
 
      return DFB_OK;
 }
@@ -418,6 +471,8 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
      DFBResult    ret;
      CoreSurface *surface = buffer->surface;
 
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
      if (manager->suspended)
           return DFB_NOVIDEOMEMORY;
 
@@ -425,7 +480,7 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
           case CSH_STORED:
                if (buffer->video.chunk)
                     buffer->video.chunk->tolerations = 0;
-               
+
                return DFB_OK;
 
           case CSH_INVALID:
@@ -435,33 +490,47 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
 
                /* FALL THROUGH, because after successful allocation
                   the surface health is CSH_RESTORE */
-          
+
           case CSH_RESTORE:
                if (buffer->system.health != CSH_STORED)
-                    BUG( "system/video instances both not stored!" );
+                    D_BUG( "system/video instances both not stored!" );
 
                if (buffer->flags & SBF_WRITTEN) {
                     int   i;
                     char *src = buffer->system.addr;
                     char *dst = dfb_system_video_memory_virtual( buffer->video.offset );
-                    
+
                     for (i=0; i<surface->height; i++) {
-                         dfb_memcpy( dst, src,
-                                     DFB_BYTES_PER_LINE(surface->format,
-                                                        surface->width) );
+                         direct_memcpy( dst, src,
+                                        DFB_BYTES_PER_LINE(buffer->format, surface->width) );
                          src += buffer->system.pitch;
                          dst += buffer->video.pitch;
                     }
-                    
-                    if (DFB_PLANAR_PIXELFORMAT( surface->format )) {
+
+                    if (buffer->format == DSPF_YV12 || buffer->format == DSPF_I420) {
                          for (i=0; i<surface->height; i++) {
-                              dfb_memcpy( dst, src,
-                                          DFB_BYTES_PER_LINE(surface->format,
-                                                             surface->width / 2) );
+                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
+                                                                          surface->width / 2) );
                               src += buffer->system.pitch / 2;
                               dst += buffer->video.pitch  / 2;
                          }
                     }
+                    else if (buffer->format == DSPF_NV12 || buffer->format == DSPF_NV21) {
+                         for (i=0; i<surface->height/2; i++) {
+                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
+                                                                          surface->width) );
+                              src += buffer->system.pitch;
+                              dst += buffer->video.pitch;
+                         }
+                    }
+                    else if (buffer->format == DSPF_NV16) {
+                         for (i=0; i<surface->height; i++) {
+                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
+                                                                          surface->width) );
+                              src += buffer->system.pitch;
+                              dst += buffer->video.pitch;
+                         }
+                    }
                }
 
                buffer->video.health             = CSH_STORED;
@@ -475,7 +544,7 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
                break;
      }
 
-     BUG( "unknown video instance health" );
+     D_BUG( "unknown video instance health" );
      return DFB_BUG;
 }
 
@@ -484,15 +553,17 @@ DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
 {
      CoreSurface *surface = buffer->surface;
 
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+
      if (buffer->policy == CSP_VIDEOONLY) {
-          BUG( "surface_manager_assure_system() called on video only surface" );
+          D_BUG( "surface_manager_assure_system() called on video only surface" );
           return DFB_BUG;
      }
 
      if (buffer->system.health == CSH_STORED)
           return DFB_OK;
      else if (buffer->video.health == CSH_STORED) {
-          int   h   = surface->height;
+          int   i;
           char *src = dfb_system_video_memory_virtual( buffer->video.offset );
           char *dst = buffer->system.addr;
 
@@ -503,21 +574,37 @@ DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
           }
           buffer->video.access |= VAF_SOFTWARE_READ;
 
-          while (h--) {
-               dfb_memcpy( dst, src, DFB_BYTES_PER_LINE(surface->format,
-                                                        surface->width) );
+          for (i=0; i<surface->height; i++) {
+               direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format, surface->width) );
                src += buffer->video.pitch;
                dst += buffer->system.pitch;
           }
-          if (DFB_PLANAR_PIXELFORMAT( surface->format )) {
-               h = surface->height;
-               while (h--) {
-                    dfb_memcpy( dst, src, DFB_BYTES_PER_LINE(surface->format,
-                                                             surface->width / 2) );
+
+          if (buffer->format == DSPF_YV12 || buffer->format == DSPF_I420) {
+               for (i=0; i<surface->height; i++) {
+                    direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
+                                                                surface->width / 2) );
                     src += buffer->video.pitch  / 2;
                     dst += buffer->system.pitch / 2;
                }
           }
+          else if (buffer->format == DSPF_NV12 || buffer->format == DSPF_NV21) {
+               for (i=0; i<surface->height/2; i++) {
+                    direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
+                                                                surface->width) );
+                    src += buffer->video.pitch;
+                    dst += buffer->system.pitch;
+               }
+          }
+          else if (buffer->format == DSPF_NV16) {
+               for (i=0; i<surface->height; i++) {
+                    direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
+                                                                surface->width) );
+                    src += buffer->video.pitch;
+                    dst += buffer->system.pitch;
+               }
+          }
+
           buffer->system.health = CSH_STORED;
 
           dfb_surface_notify_listeners( surface, CSNF_SYSTEM );
@@ -525,7 +612,7 @@ DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
           return DFB_OK;
      }
 
-     BUG( "no valid surface instance" );
+     D_BUG( "no valid surface instance" );
      return DFB_BUG;
 }
 
@@ -535,6 +622,8 @@ static Chunk* split_chunk( Chunk *c, int length )
 {
      Chunk *newchunk;
 
+     D_MAGIC_ASSERT( c, _Chunk_ );
+
      if (c->length == length)          /* does not need be splitted */
           return c;
 
@@ -552,20 +641,24 @@ static Chunk* split_chunk( Chunk *c, int length )
           c->next->prev = newchunk;
      c->next = newchunk;
 
+     D_MAGIC_SET( newchunk, _Chunk_ );
+
      return newchunk;
 }
 
 static Chunk*
 free_chunk( SurfaceManager *manager, Chunk *chunk )
 {
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+     D_MAGIC_ASSERT( chunk, _Chunk_ );
+
      if (!chunk->buffer) {
-          BUG( "freeing free chunk" );
+          D_BUG( "freeing free chunk" );
           return chunk;
      }
      else {
-          DEBUGMSG( "DirectFB/core/surfacemanager: "
-                    "Deallocating %d bytes at offset %d.\n",
-                    chunk->length, chunk->offset );
+          D_DEBUG_AT( Core_SM, "Deallocating %d bytes at offset %d.\n",
+                      chunk->length, chunk->offset );
      }
 
      if (chunk->buffer->policy == CSP_VIDEOONLY)
@@ -578,7 +671,7 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
      if (chunk->prev  &&  !chunk->prev->buffer) {
           Chunk *prev = chunk->prev;
 
-          //DEBUGMSG( "  merging with previous chunk at %d\n", prev->offset );
+          //D_DEBUG_AT( Core_SM, "  -> merging with previous chunk at %d\n", prev->offset );
 
           prev->length += chunk->length;
 
@@ -586,7 +679,9 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
           if (prev->next)
                prev->next->prev = prev;
 
-          //DEBUGMSG("freeing %p (prev %p, next %p)\n", chunk, chunk->prev, chunk->next);
+          //D_DEBUG_AT( Core_SM, "  -> freeing %p (prev %p, next %p)\n", chunk, chunk->prev, chunk->next);
+
+          D_MAGIC_CLEAR( chunk );
 
           SHFREE( chunk );
           chunk = prev;
@@ -595,7 +690,7 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
      if (chunk->next  &&  !chunk->next->buffer) {
           Chunk *next = chunk->next;
 
-          //DEBUGMSG( "  merging with next chunk at %d\n", next->offset );
+          //D_DEBUG_AT( Core_SM, "  -> merging with next chunk at %d\n", next->offset );
 
           chunk->length += next->length;
 
@@ -603,6 +698,8 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
           if (chunk->next)
                chunk->next->prev = chunk;
 
+          D_MAGIC_CLEAR( next );
+
           SHFREE( next );
      }
 
@@ -612,15 +709,16 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
 static void
 occupy_chunk( SurfaceManager *manager, Chunk *chunk, SurfaceBuffer *buffer, int length )
 {
+     D_MAGIC_ASSERT( manager, SurfaceManager );
+     D_MAGIC_ASSERT( chunk, _Chunk_ );
+
      if (buffer->policy == CSP_VIDEOONLY)
           manager->available -= length;
-     
+
      chunk = split_chunk( chunk, length );
 
-     DEBUGMSG( "DirectFB/core/surfacemanager: "
-               "Allocating %d bytes at offset %d.\n",
-               chunk->length, chunk->offset );
-     
+     D_DEBUG_AT( Core_SM, "Allocating %d bytes at offset %d.\n", chunk->length, chunk->offset );
+
      buffer->video.health = CSH_RESTORE;
      buffer->video.offset = chunk->offset;
      buffer->video.chunk  = chunk;
