@@ -39,10 +39,20 @@
 typedef enum {
      CORE_ANY,
      CORE_FBDEV,
+     CORE_X11,
      CORE_OSX,
-     CORE_SDL
+     CORE_SDL,
+     CORE_VNC
 } CoreSystemType;
 
+typedef enum {
+     CSCAPS_NONE         = 0x00000000,  /* None of these. */
+
+     CSCAPS_ACCELERATION = 0x00000001,  /* HW acceleration supported, so probe graphics drivers. */
+
+     CSCAPS_ALL          = 0x00000001   /* All of these. */
+} CoreSystemCapabilities;
+
 /*
  * hold information of a Videomode read from /etc/fb.modes
  * (to be replaced by DirectFB's own config system)
@@ -52,6 +62,8 @@ typedef struct _VideoMode {
      int yres;
      int bpp;
 
+     int priority;
+
      int pixclock;
      int left_margin;
      int right_margin;
@@ -78,7 +90,7 @@ DECLARE_MODULE_DIRECTORY( dfb_core_systems );
 /*
  * Increase this number when changes result in binary incompatibility!
  */
-#define DFB_CORE_SYSTEM_ABI_VERSION           7
+#define DFB_CORE_SYSTEM_ABI_VERSION           9
 
 #define DFB_CORE_SYSTEM_INFO_NAME_LENGTH     60
 #define DFB_CORE_SYSTEM_INFO_VENDOR_LENGTH   80
@@ -92,23 +104,24 @@ typedef struct {
 } CoreSystemVersion;        /* major.minor, e.g. 0.1 */
 
 typedef struct {
-     CoreSystemVersion  version;
+     CoreSystemVersion        version;
 
-     CoreSystemType     type;
+     CoreSystemType           type;
+     CoreSystemCapabilities   caps;
 
-     char               name[DFB_CORE_SYSTEM_INFO_NAME_LENGTH];
-                                /* Name of system, e.g. 'FBDev' */
+     char                     name[DFB_CORE_SYSTEM_INFO_NAME_LENGTH];
+                                   /* Name of system, e.g. 'FBDev' */
 
-     char               vendor[DFB_CORE_SYSTEM_INFO_VENDOR_LENGTH];
-                                /* Vendor (or author) of the driver,
-                                   e.g. 'convergence' or 'Denis Oliver Kropp' */
+     char                     vendor[DFB_CORE_SYSTEM_INFO_VENDOR_LENGTH];
+                                   /* Vendor (or author) of the driver,
+                                      e.g. 'convergence' or 'Denis Oliver Kropp' */
 
-     char               url[DFB_CORE_SYSTEM_INFO_URL_LENGTH];
-                                /* URL for driver updates,
-                                   e.g. 'http://www.directfb.org/' */
+     char                     url[DFB_CORE_SYSTEM_INFO_URL_LENGTH];
+                                   /* URL for driver updates,
+                                      e.g. 'http://www.directfb.org/' */
 
-     char               license[DFB_CORE_SYSTEM_INFO_LICENSE_LENGTH];
-                                /* License, e.g. 'LGPL' or 'proprietary' */
+     char                     license[DFB_CORE_SYSTEM_INFO_LICENSE_LENGTH];
+                                   /* License, e.g. 'LGPL' or 'proprietary' */
 } CoreSystemInfo;
 
 typedef struct {
@@ -164,6 +177,10 @@ typedef struct {
      void*          (*VideoMemoryVirtual)( unsigned int offset );
 
      unsigned int   (*VideoRamLength)();
+
+     void           (*GetBusID)( int *ret_bus, int *ret_dev, int *ret_func );
+     void           (*GetDeviceID)( unsigned int *ret_vendor_id,
+                                    unsigned int *ret_device_id );
 } CoreSystemFuncs;
 
 
@@ -174,6 +191,9 @@ dfb_system_lookup();
 CoreSystemType
 dfb_system_type();
 
+CoreSystemCapabilities
+dfb_system_caps();
+
 void *
 dfb_system_data();
 
@@ -210,5 +230,12 @@ dfb_system_video_memory_virtual( unsigned int offset );
 unsigned int
 dfb_system_videoram_length();
 
+void
+dfb_system_get_busid( int *ret_bus, int *ret_dev, int *ret_func );
+
+void
+dfb_system_get_deviceid( unsigned int *ret_vendor_id,
+                         unsigned int *ret_device_id );
+
 #endif
 
