@@ -282,8 +282,13 @@ dfb_window_create( CoreWindowStack        *stack,
 
      /* Choose pixel format. */
      if (caps & DWCAPS_ALPHACHANNEL) {
-          if (pixelformat == DSPF_UNKNOWN)
-               pixelformat = DSPF_ARGB;
+          if (pixelformat == DSPF_UNKNOWN) {
+               if (context->config.flags & DLCONF_PIXELFORMAT)
+                    pixelformat = context->config.pixelformat;
+
+               if (! DFB_PIXELFORMAT_HAS_ALPHA(pixelformat))
+                    pixelformat = DSPF_ARGB;
+          }
           else if (! DFB_PIXELFORMAT_HAS_ALPHA(pixelformat)) {
                dfb_windowstack_unlock( stack );
                return DFB_INVARG;
@@ -876,7 +881,7 @@ dfb_window_set_opaque( CoreWindow      *window,
      config.opaque.x1 = 0;
      config.opaque.y1 = 0;
      config.opaque.x2 = window->config.bounds.w - 1;
-     config.opaque.y1 = window->config.bounds.h - 1;
+     config.opaque.y2 = window->config.bounds.h - 1;
 
      if (region && !dfb_region_region_intersect( &config.opaque, region ))
           ret = DFB_INVAREA;
