@@ -239,7 +239,8 @@ IDirectFBDisplayLayer_SetCooperativeLevel( IDirectFBDisplayLayer           *thiz
 
                     data->context = context;
                     data->region  = region;
-               }
+                    data->stack   = dfb_layer_context_windowstack( data->context );
+                }
 
                break;
 
@@ -265,6 +266,7 @@ IDirectFBDisplayLayer_SetCooperativeLevel( IDirectFBDisplayLayer           *thiz
 
                data->context = context;
                data->region  = region;
+               data->stack   = dfb_layer_context_windowstack( data->context );
 
                break;
 
@@ -583,7 +585,7 @@ IDirectFBDisplayLayer_CreateWindow( IDirectFBDisplayLayer       *thiz,
      int                    posx         = 0;
      int                    posy         = 0;
      DFBWindowCapabilities  caps         = 0;
-     DFBSurfaceCapabilities surface_caps = 0;
+     DFBSurfaceCapabilities surface_caps = DSCAPS_NONE;
      DFBSurfacePixelFormat  format       = DSPF_UNKNOWN;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
@@ -650,7 +652,7 @@ IDirectFBDisplayLayer_EnableCursor( IDirectFBDisplayLayer *thiz, int enable )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
 
-     if (data->level != DLSCL_ADMINISTRATIVE)
+     if (data->level == DLSCL_SHARED)
           return DFB_ACCESSDENIED;
 
      return dfb_windowstack_cursor_enable( data->stack, enable );
