@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -35,14 +36,14 @@
 
 
 #include <directfb.h>
-#include <directfb_internals.h>
 
 #include <core/surfaces.h>
 #include <core/palette.h>
 
-#include <misc/util.h>
-#include <misc/mem.h>
-#include <misc/memcpy.h>
+#include <direct/interface.h>
+#include <direct/mem.h>
+#include <direct/memcpy.h>
+#include <direct/util.h>
 
 #include <gfx/convert.h>
 #include <gfx/util.h>
@@ -59,13 +60,13 @@ IDirectFBPalette_Destruct( IDirectFBPalette *thiz )
      if (data->palette)
           dfb_palette_unref( data->palette );
 
-     DFB_DEALLOCATE_INTERFACE( thiz );
+     DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 static DFBResult
 IDirectFBPalette_AddRef( IDirectFBPalette *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      data->ref++;
 
@@ -75,7 +76,7 @@ IDirectFBPalette_AddRef( IDirectFBPalette *thiz )
 static DFBResult
 IDirectFBPalette_Release( IDirectFBPalette *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      if (--data->ref == 0)
           IDirectFBPalette_Destruct( thiz );
@@ -89,7 +90,7 @@ IDirectFBPalette_GetCapabilities( IDirectFBPalette       *thiz,
 {
      CorePalette *palette;
 
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      palette = data->palette;
      if (!palette)
@@ -100,7 +101,7 @@ IDirectFBPalette_GetCapabilities( IDirectFBPalette       *thiz,
 
      /* FIXME: no caps yet */
      *caps = DPCAPS_NONE;
-     
+
      return DFB_OK;
 }
 
@@ -110,7 +111,7 @@ IDirectFBPalette_GetSize( IDirectFBPalette *thiz,
 {
      CorePalette *palette;
 
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      palette = data->palette;
      if (!palette)
@@ -120,19 +121,19 @@ IDirectFBPalette_GetSize( IDirectFBPalette *thiz,
           return DFB_INVARG;
 
      *size = palette->num_entries;
-     
+
      return DFB_OK;
 }
 
 static DFBResult
 IDirectFBPalette_SetEntries( IDirectFBPalette *thiz,
-                             DFBColor         *entries,
+                             const DFBColor   *entries,
                              unsigned int      num_entries,
                              unsigned int      offset )
 {
      CorePalette *palette;
 
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      palette = data->palette;
      if (!palette)
@@ -142,12 +143,11 @@ IDirectFBPalette_SetEntries( IDirectFBPalette *thiz,
           return DFB_INVARG;
 
      if (num_entries) {
-          dfb_memcpy( palette->entries + offset,
-                      entries, num_entries * sizeof(DFBColor));
+          direct_memcpy( palette->entries + offset, entries, num_entries * sizeof(DFBColor));
 
           dfb_palette_update( palette, offset, offset + num_entries - 1 );
      }
-     
+
      return DFB_OK;
 }
 
@@ -159,7 +159,7 @@ IDirectFBPalette_GetEntries( IDirectFBPalette *thiz,
 {
      CorePalette *palette;
 
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      palette = data->palette;
      if (!palette)
@@ -168,8 +168,8 @@ IDirectFBPalette_GetEntries( IDirectFBPalette *thiz,
      if (!entries  ||  offset + num_entries > palette->num_entries)
           return DFB_INVARG;
 
-     dfb_memcpy( entries, palette->entries + offset, num_entries * sizeof(DFBColor));
-     
+     direct_memcpy( entries, palette->entries + offset, num_entries * sizeof(DFBColor));
+
      return DFB_OK;
 }
 
@@ -183,7 +183,7 @@ IDirectFBPalette_FindBestMatch( IDirectFBPalette *thiz,
 {
      CorePalette *palette;
 
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      if (!index)
           return DFB_INVARG;
@@ -193,7 +193,7 @@ IDirectFBPalette_FindBestMatch( IDirectFBPalette *thiz,
           return DFB_DESTROYED;
 
      *index = dfb_palette_search( palette, r, g, b, a );
-     
+
      return DFB_OK;
 }
 
@@ -205,25 +205,26 @@ IDirectFBPalette_CreateCopy( IDirectFBPalette  *thiz,
      IDirectFBPalette *iface;
      CorePalette      *palette = NULL;
 
-     INTERFACE_GET_DATA(IDirectFBPalette)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBPalette)
 
      if (!data->palette)
           return DFB_DESTROYED;
-     
+
      if (!interface)
           return DFB_INVARG;
 
-     ret = dfb_palette_create( data->palette->num_entries, &palette );
+     ret = dfb_palette_create( NULL,    /* FIXME */
+                               data->palette->num_entries, &palette );
      if (ret)
           return ret;
-     
-     dfb_memcpy( palette->entries, data->palette->entries,
-                 palette->num_entries * sizeof(DFBColor));
+
+     direct_memcpy( palette->entries, data->palette->entries,
+                    palette->num_entries * sizeof(DFBColor));
 
      dfb_palette_update( palette, 0, palette->num_entries - 1 );
-     
-     
-     DFB_ALLOCATE_INTERFACE( iface, IDirectFBPalette );
+
+
+     DIRECT_ALLOCATE_INTERFACE( iface, IDirectFBPalette );
 
      ret = IDirectFBPalette_Construct( iface, palette );
 
@@ -240,13 +241,13 @@ IDirectFBPalette_CreateCopy( IDirectFBPalette  *thiz,
 DFBResult IDirectFBPalette_Construct( IDirectFBPalette *thiz,
                                       CorePalette      *palette )
 {
-     DFB_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBPalette)
+     DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBPalette)
 
      if (dfb_palette_ref( palette )) {
-          DFB_DEALLOCATE_INTERFACE(thiz);
+          DIRECT_DEALLOCATE_INTERFACE(thiz);
           return DFB_FAILURE;
      }
-     
+
      data->ref     = 1;
      data->palette = palette;
 
@@ -256,13 +257,13 @@ DFBResult IDirectFBPalette_Construct( IDirectFBPalette *thiz,
 
      thiz->GetCapabilities = IDirectFBPalette_GetCapabilities;
      thiz->GetSize         = IDirectFBPalette_GetSize;
-     
+
      thiz->SetEntries      = IDirectFBPalette_SetEntries;
      thiz->GetEntries      = IDirectFBPalette_GetEntries;
      thiz->FindBestMatch   = IDirectFBPalette_FindBestMatch;
 
      thiz->CreateCopy      = IDirectFBPalette_CreateCopy;
-     
+
      return DFB_OK;
 }
 
