@@ -416,7 +416,7 @@ IDirectFBSurface_SetField( IDirectFBSurface    *thiz,
      if (!data->surface)
           return DFB_DESTROYED;
 
-     if (!(data->caps & DSCAPS_INTERLACED))
+     if (!(data->surface->caps & DSCAPS_INTERLACED))
           return DFB_UNSUPPORTED;
 
      if (field < 0 || field > 1)
@@ -1778,6 +1778,20 @@ IDirectFBSurface_Dump( IDirectFBSurface   *thiz,
      return dfb_surface_dump( surface, directory, prefix );
 }
 
+static DFBResult
+IDirectFBSurface_DisableAcceleration( IDirectFBSurface    *thiz,
+                                      DFBAccelerationMask  mask )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     if (D_FLAGS_INVALID( mask, DFXL_ALL ))
+          return DFB_INVARG;
+
+     data->state.disabled = mask;
+
+     return DFB_OK;
+}
+
 /******/
 
 DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
@@ -1882,6 +1896,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->GetGL = IDirectFBSurface_GetGL;
 
      thiz->Dump = IDirectFBSurface_Dump;
+     thiz->DisableAcceleration = IDirectFBSurface_DisableAcceleration;
 
 
      dfb_surface_attach( surface,
