@@ -6,8 +6,9 @@
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjl <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -36,12 +37,19 @@
 
 /* this order is required for Intel with MMX, how about bigendian? */
 
-typedef struct
-{
-     __u16 b;
-     __u16 g;
-     __u16 r;
-     __u16 a;
+typedef union {
+     struct {
+          __u16 b;
+          __u16 g;
+          __u16 r;
+          __u16 a;
+     } RGB;
+     struct {
+          __u16 u;
+          __u16 v;
+          __u16 y;
+          __u16 a;
+     } YUV;
 } GenefxAccumulator;
 
 
@@ -55,7 +63,9 @@ typedef void (*GenefxFunc)(GenefxState *gfxs);
 struct _GenefxState {
      GenefxFunc funcs[32];
 
-     int length;    /* span length (destination) */
+     int length;    /* span length */
+     int Slen;      /* span length (source) */
+     int Dlen;      /* span length (destination) */
 
      /*
       * state values
@@ -89,6 +99,7 @@ struct _GenefxState {
      void *Bop;
      __u32 Cop;
 
+     __u8 YCop;
      __u8 CbCop;
      __u8 CrCop;
 
