@@ -284,22 +284,33 @@ IDirectFB_SetCooperativeLevel( IDirectFB           *thiz,
 }
 
 static DFBResult
-IDirectFB_GetCardCapabilities( IDirectFB           *thiz,
-                               DFBCardCapabilities *caps )
+IDirectFB_GetDeviceDescription( IDirectFB                    *thiz,
+                                DFBGraphicsDeviceDescription *ret_desc )
 {
-     CardCapabilities card_caps;
+     GraphicsDeviceInfo device_info;
+     GraphicsDriverInfo driver_info;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB)
 
-     if (!caps)
+     if (!ret_desc)
           return DFB_INVARG;
 
-     dfb_gfxcard_get_capabilities( &card_caps );
+     dfb_gfxcard_get_device_info( &device_info );
+     dfb_gfxcard_get_driver_info( &driver_info );
 
-     caps->acceleration_mask = card_caps.accel;
-     caps->blitting_flags    = card_caps.blitting;
-     caps->drawing_flags     = card_caps.drawing;
-     caps->video_memory      = dfb_gfxcard_memory_length();
+     ret_desc->acceleration_mask = device_info.caps.accel;
+     ret_desc->blitting_flags    = device_info.caps.blitting;
+     ret_desc->drawing_flags     = device_info.caps.drawing;
+     ret_desc->video_memory      = dfb_gfxcard_memory_length();
+
+     snprintf( ret_desc->name,   DFB_GRAPHICS_DEVICE_DESC_NAME_LENGTH, device_info.name );
+     snprintf( ret_desc->vendor, DFB_GRAPHICS_DEVICE_DESC_NAME_LENGTH, device_info.vendor );
+
+     ret_desc->driver.major = driver_info.version.major;
+     ret_desc->driver.minor = driver_info.version.minor;
+
+     snprintf( ret_desc->driver.name,   DFB_GRAPHICS_DRIVER_INFO_NAME_LENGTH,   driver_info.name );
+     snprintf( ret_desc->driver.vendor, DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH, driver_info.vendor );
 
      return DFB_OK;
 }
@@ -408,7 +419,7 @@ IDirectFB_CreateSurface( IDirectFB                    *thiz,
      int height = 256;
      int policy = CSP_VIDEOLOW;
      DFBSurfacePixelFormat format;
-     DFBSurfaceCapabilities caps = 0;
+     DFBSurfaceCapabilities caps = DSCAPS_NONE;
      DFBDisplayLayerConfig  config;
      CoreSurface *surface = NULL;
 
@@ -594,6 +605,11 @@ IDirectFB_CreateSurface( IDirectFB                    *thiz,
                     config.flags |= DLCONF_BUFFERMODE | DLCONF_PIXELFORMAT |
                                     DLCONF_WIDTH | DLCONF_HEIGHT;
 
+                    if (caps & DSCAPS_PREMULTIPLIED) {
+                          config.flags        |= DLCONF_SURFACE_CAPS;
+                          config.surface_caps  = DSCAPS_PREMULTIPLIED;
+                    }
+
                     if (caps & DSCAPS_TRIPLE) {
                          if (caps & DSCAPS_SYSTEMONLY)
                               return DFB_UNSUPPORTED;
@@ -967,10 +983,9 @@ IDirectFB_CreateVideoProvider( IDirectFB               *thiz,
                                const char              *filename,
                                IDirectFBVideoProvider **interface )
 {
-     DFBResult                            ret;
-     DirectInterfaceFuncs                *funcs = NULL;
-     IDirectFBVideoProvider              *videoprovider;
-     IDirectFBVideoProvider_ProbeContext  ctx;
+     DFBResult                 ret; 
+     DFBDataBufferDescription  desc;
+     IDirectFBDataBuffer      *databuffer;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB)
 
@@ -978,27 +993,21 @@ IDirectFB_CreateVideoProvider( IDirectFB               *thiz,
      if (!interface || !filename)
           return DFB_INVARG;
 
-     if (access( filename, R_OK ) != 0)
-          return errno2result( errno );
+     /* Create a data buffer. */
+     desc.flags = DBDESC_FILE;
+     desc.file  = filename;
 
-     /* Fill out probe context */
-     ctx.filename = filename;
-
-     /* Find a suitable implemenation */
-     ret = DirectGetInterface( &funcs, "IDirectFBVideoProvider", NULL, DirectProbeInterface, &ctx );
+     ret = thiz->CreateDataBuffer( thiz, &desc, &databuffer );
      if (ret)
           return ret;
 
-     DIRECT_ALLOCATE_INTERFACE( videoprovider, IDirectFBVideoProvider );
+     /* Create (probing) the video provider. */
+     ret = IDirectFBVideoProvider_CreateFromBuffer( databuffer, interface );
 
-     /* Construct the interface */
-     ret = funcs->Construct( videoprovider, filename );
-     if (ret)
-          return ret;
+     /* We don't need it anymore, video provider has its own reference. */
+     databuffer->Release( databuffer );
 
-     *interface = videoprovider;
-
-     return DFB_OK;
+     return ret;
 }
 
 static DFBResult
@@ -1231,7 +1240,7 @@ IDirectFB_Construct( IDirectFB *thiz, CoreDFB *core )
      thiz->AddRef = IDirectFB_AddRef;
      thiz->Release = IDirectFB_Release;
      thiz->SetCooperativeLevel = IDirectFB_SetCooperativeLevel;
-     thiz->GetCardCapabilities = IDirectFB_GetCardCapabilities;
+     thiz->GetDeviceDescription = IDirectFB_GetDeviceDescription;
      thiz->EnumVideoModes = IDirectFB_EnumVideoModes;
      thiz->SetVideoMode = IDirectFB_SetVideoMode;
      thiz->CreateSurface = IDirectFB_CreateSurface;
