@@ -32,23 +32,24 @@
 #include <stdio.h>
 #include <string.h>
 
-
 #include <directfb.h>
 
-#include <core/coretypes.h>
-
-#include <core/surfaces.h>
-#include <core/layers.h>
 
 #include <direct/conf.h>
-#include <fusion/conf.h>
-#include <misc/conf.h>
-
+#include <direct/log.h>
 #include <direct/mem.h>
 #include <direct/memcpy.h>
 #include <direct/messages.h>
 #include <direct/util.h>
 
+#include <fusion/conf.h>
+
+#include <core/coretypes.h>
+#include <core/surfaces.h>
+#include <core/layers.h>
+
+#include <misc/conf.h>
+
 
 DFBConfig *dfb_config = NULL;
 
@@ -62,6 +63,7 @@ static const char *config_usage =
      "\n"
      "  system=<system>                Specify the system (FBDev, SDL, etc.)\n"
      "  fbdev=<device>                 Open <device> instead of /dev/fb0\n"
+     "  busid=<id>                     Specify the bus location of the graphics card (default 1:0:0)\n"
      "  mode=<width>x<height>          Set the default resolution\n"
      "  depth=<pixeldepth>             Set the default pixel depth\n"
      "  pixelformat=<pixelformat>      Set the default pixel format\n"
@@ -74,6 +76,8 @@ static const char *config_usage =
      "  [no-]banner                    Show DirectFB Banner on startup\n"
      "  [no-]debug                     Enable debug output\n"
      "  [no-]trace                     Enable stack trace support\n"
+     "  log-file=<name>                Write all messages to a file\n"
+     "  log-udp=<host>:<port>          Send all messages via UDP to host:port\n"
      "  force-windowed                 Primary surface always is a window\n"
      "  force-desktop                  Primary surface is the desktop background\n"
      "  [no-]hardware                  Hardware acceleration\n"
@@ -89,9 +93,11 @@ static const char *config_usage =
      "  [no-]vt-switch                 Allocate/switch to a new VT\n"
      "  [no-]vt-switching              Allow Ctrl+Alt+<F?> (EXPERIMENTAL)\n"
      "  [no-]graphics-vt               Put terminal into graphics mode\n"
-     "  [no-]-vt                       Use VT handling code at all?\n"
+     "  [no-]vt                        Use VT handling code at all?\n"
+     "  mouse-source=<device>          Mouse device for serial mouse\n"
+     "  [no-]mouse-gpm-source          Enable mouse input repeated by GPM\n"
      "  [no-]motion-compression        Mouse motion event compression\n"
-     "  mouse-protocol=<protocol>      Mouse protocol (serial mouse)\n"
+     "  mouse-protocol=<protocol>      Mouse protocol\n"
      "  [no-]lefty                     Swap left and right mouse buttons\n"
      "  [no-]capslock-meta             Map the CapsLock key to Meta\n"
      "  linux-input-ir-only            Ignore all non-IR Linux Input devices\n"
@@ -153,8 +159,8 @@ static const FormatString format_strings[] = {
      { "I420",     DSPF_I420     },
      { "LUT8",     DSPF_LUT8     },
      { "NV12",     DSPF_NV12     },
-     { "NV21",     DSPF_NV21     },
      { "NV16",     DSPF_NV16     },
+     { "NV21",     DSPF_NV21     },
      { "RGB16",    DSPF_RGB16    },
      { "RGB24",    DSPF_RGB24    },
      { "RGB32",    DSPF_RGB32    },
@@ -233,7 +239,7 @@ parse_args( const char *args )
                *next++ = '\0';
 
           if (strcmp (buf, "help") == 0) {
-               fprintf( stderr, config_usage );
+               direct_log_printf( NULL, config_usage );
                exit(1);
           }
 
@@ -302,6 +308,10 @@ static void config_allocate()
      dfb_config->layer_bg_color.g         = 0x7c;
      dfb_config->layer_bg_color.b         = 0xe8;
      dfb_config->layer_bg_mode            = DLBM_COLOR;
+     
+     dfb_config->pci.bus                  = 1;
+     dfb_config->pci.dev                  = 0;
+     dfb_config->pci.func                 = 0;
 
      dfb_config->banner                   = true;
      dfb_config->deinit_check             = true;
@@ -386,6 +396,20 @@ DFBResult dfb_config_set( const char *name, const char *value )
                return DFB_INVARG;
           }
      } else
+     if (strcmp (name, "busid") == 0 || strcmp (name, "pci-id") == 0) {
+          if (value) {
+               int bus, dev, func;
+               
+               if (sscanf( value, "%d:%d:%d", &bus, &dev, &func ) != 3) {
+                    D_ERROR( "DirectFB/Config 'busid': Could not parse busid!\n");
+                    return DFB_INVARG;
+               }                  
+               
+               dfb_config->pci.bus  = bus;
+               dfb_config->pci.dev  = dev;
+               dfb_config->pci.func = func;
+          }
+     } else                    
      if (strcmp (name, "tmpfs" ) == 0) {
           if (value) {
                if (fusion_config->tmpfs)
@@ -586,6 +610,30 @@ DFBResult dfb_config_set( const char *name, const char *value )
      if (strcmp (name, "no-trace" ) == 0) {
           direct_config->trace = false;
      } else
+     if (strcmp (name, "log-file" ) == 0 || strcmp (name, "log-udp" ) == 0) {
+          if (value) {
+               DirectResult  ret;
+               DirectLog    *log;
+
+               ret = direct_log_create( strcmp(name,"log-udp") ? DLT_FILE : DLT_UDP, value, &log );
+               if (ret)
+                    return ret;
+
+               if (direct_config->log)
+                    direct_log_destroy( direct_config->log );
+
+               direct_config->log = log;
+
+               direct_log_set_default( log );
+          }
+          else {
+               if (strcmp(name,"log-udp"))
+                    D_ERROR("DirectFB/Config 'log-file': No file name specified!\n");
+               else
+                    D_ERROR("DirectFB/Config 'log-udp': No host and port specified!\n");
+               return DFB_INVARG;
+          }
+     } else
      if (strcmp (name, "force-windowed" ) == 0) {
           dfb_config->force_windowed = true;
      } else
@@ -661,8 +709,13 @@ DFBResult dfb_config_set( const char *name, const char *value )
      } else
      if (strcmp (name, "mouse-gpm-source" ) == 0) {
           dfb_config->mouse_gpm_source = true;
-	  D_FREE( dfb_config->mouse_source );
-	  dfb_config->mouse_source = D_STRDUP( DEV_NAME_GPM );
+	     D_FREE( dfb_config->mouse_source );
+	     dfb_config->mouse_source = D_STRDUP( DEV_NAME_GPM );
+     } else
+     if (strcmp (name, "no-mouse-gpm-source" ) == 0) {
+          dfb_config->mouse_gpm_source = false;
+          D_FREE( dfb_config->mouse_source );
+          dfb_config->mouse_source = D_STRDUP( DEV_NAME );
      } else
      if (strcmp (name, "translucent-windows" ) == 0) {
           dfb_config->translucent_windows = true;
@@ -934,7 +987,7 @@ DFBResult dfb_config_set( const char *name, const char *value )
      return DFB_OK;
 }
 
-DFBResult dfb_config_init( int *argc, char **argv[] )
+DFBResult dfb_config_init( int *argc, char *(*argv[]) )
 {
      DFBResult ret;
      int i;
@@ -1020,7 +1073,7 @@ DFBResult dfb_config_init( int *argc, char **argv[] )
           for (i = 1; i < *argc; i++) {
 
                if (strcmp ((*argv)[i], "--dfb-help") == 0) {
-                    fprintf( stderr, config_usage );
+                    direct_log_printf( NULL, config_usage );
                     exit(1);
                }
 
