@@ -80,6 +80,12 @@ typedef struct
 
      char     *fb_device;                         /* Used framebuffer device,
                                                      e.g. "/dev/fb0" */
+                                                     
+     struct {
+          int  bus;                               /* PCI Bus */
+          int  dev;                               /* PCI Device */
+          int  func;                              /* PCI Function */
+     } pci;
 
      bool      lefty;                             /* Left handed mouse, swaps
                                                      left/right mouse buttons */
@@ -141,7 +147,7 @@ extern DFBConfig *dfb_config;
  * for overrides. Options identified as DirectFB options are stripped out
  * of the array.
  */
-DFBResult dfb_config_init( int *argc, char **argv[] );
+DFBResult dfb_config_init( int *argc, char *(*argv[]) );
 
 /*
  * Read configuration options from file. Called by config_init().
