@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2002-2004  convergence GmbH.
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -36,105 +37,15 @@
 #include <sys/time.h>
 #include <time.h>
 
-#include "util.h"
+#include <direct/messages.h>
+#include <direct/util.h>
+
+#include <misc/util.h>
 
 
-static struct timeval start_time = { 0, 0 };
-
-long long dfb_get_micros()
-{
-     struct timeval tv;
-     
-     if (start_time.tv_sec == 0) {
-          gettimeofday( &start_time, NULL );
-          return 0;
-     }
-
-     gettimeofday( &tv, NULL );
-
-     return (long long)(tv.tv_sec - start_time.tv_sec) * (long long) 1000000 +
-            (long long)(tv.tv_usec - start_time.tv_usec);
-}
-
-long long dfb_get_millis()
-{
-     return dfb_get_micros() / (long long) 1000;
-}
-
-int
-dfb_try_open( const char *name1, const char *name2, int flags )
-{
-     int fd;
-
-     fd = open (name1, flags);
-     if (fd >= 0)
-          return fd;
-
-     if (errno != ENOENT) {
-          PERRORMSG( "opening '%s' failed\n", name1 );
-          return -1;
-     }
-     
-     fd = open (name2, flags);
-     if (fd >= 0)
-          return fd;
-     
-     if (errno == ENOENT)
-          PERRORMSG( "opening '%s' and '%s' failed\n", name1, name2 );
-     else
-          PERRORMSG( "opening '%s' failed\n", name2 );
-
-     return -1;
-}
-
-void dfb_trim( char **s )
-{
-     int i;
-     int len = strlen( *s );
-
-     for (i = len-1; i >= 0; i--)
-          if ((*s)[i] <= ' ')
-               (*s)[i] = 0;
-          else
-               break;
-
-     while (**s)
-          if (**s <= ' ')
-               (*s)++;
-          else
-               return;
-}
-
-/*
- * translates errno to DirectFB DFBResult
- */
-DFBResult errno2dfb( int erno )
-{
-     switch (erno) {
-          case 0:
-               return DFB_OK;
-          case ENOENT:
-               return DFB_FILENOTFOUND;
-          case EACCES:
-          case EPERM:
-               return DFB_ACCESSDENIED;
-          case EBUSY:
-          case EAGAIN:
-               return DFB_BUSY;
-          case ENODEV:
-          case ENXIO:
-#ifdef ENOTSUP
-          /* ENOTSUP is not defined on NetBSD */
-          case ENOTSUP:
-#endif
-               return DFB_UNSUPPORTED;
-     }
-
-     return DFB_FAILURE;
-}
-
-bool dfb_region_intersect( DFBRegion *region,
-                           int x1, int y1, int x2, int y2 )
+bool
+dfb_region_intersect( DFBRegion *region,
+                      int x1, int y1, int x2, int y2 )
 {
      if (region->x2 < x1 ||
          region->y2 < y1 ||
@@ -142,16 +53,49 @@ bool dfb_region_intersect( DFBRegion *region,
          region->y1 > y2)
           return false;
 
-     region->x1 = MAX( region->x1, x1 );
-     region->y1 = MAX( region->y1, y1 );
-     region->x2 = MIN( region->x2, x2 );
-     region->y2 = MIN( region->y2, y2 );
+     if (region->x1 < x1)
+          region->x1 = x1;
+
+     if (region->y1 < y1)
+          region->y1 = y1;
+
+     if (region->x2 > x2)
+          region->x2 = x2;
+
+     if (region->y2 > y2)
+          region->y2 = y2;
 
      return true;
 }
 
-int dfb_region_rectangle_intersect( DFBRegion          *region,
-                                    const DFBRectangle *rect )
+bool
+dfb_region_region_intersect( DFBRegion       *region,
+                             const DFBRegion *clip )
+{
+     if (region->x2 < clip->x1 ||
+         region->y2 < clip->y1 ||
+         region->x1 > clip->x2 ||
+         region->y1 > clip->y2)
+          return false;
+
+     if (region->x1 < clip->x1)
+          region->x1 = clip->x1;
+
+     if (region->y1 < clip->y1)
+          region->y1 = clip->y1;
+
+     if (region->x2 > clip->x2)
+          region->x2 = clip->x2;
+
+     if (region->y2 > clip->y2)
+          region->y2 = clip->y2;
+
+     return true;
+}
+
+bool
+dfb_region_rectangle_intersect( DFBRegion          *region,
+                                const DFBRectangle *rect )
 {
      int x2 = rect->x + rect->w - 1;
      int y2 = rect->y + rect->h - 1;
@@ -160,18 +104,26 @@ int dfb_region_rectangle_intersect( DFBRegion          *region,
          region->y2 < rect->y ||
          region->x1 > x2 ||
          region->y1 > y2)
-          return 0;
+          return false;
 
-     region->x1 = MAX( region->x1, rect->x );
-     region->y1 = MAX( region->y1, rect->y );
-     region->x2 = MIN( region->x2, x2 );
-     region->y2 = MIN( region->y2, y2 );
+     if (region->x1 < rect->x)
+          region->x1 = rect->x;
 
-     return 1;
+     if (region->y1 < rect->y)
+          region->y1 = rect->y;
+
+     if (region->x2 > x2)
+          region->x2 = x2;
+
+     if (region->y2 > y2)
+          region->y2 = y2;
+
+     return true;
 }
 
-int dfb_unsafe_region_intersect( DFBRegion *region,
-                                 int x1, int y1, int x2, int y2 )
+bool
+dfb_unsafe_region_intersect( DFBRegion *region,
+                             int x1, int y1, int x2, int y2 )
 {
      if (region->x1 > region->x2) {
           int temp = region->x1;
@@ -188,8 +140,9 @@ int dfb_unsafe_region_intersect( DFBRegion *region,
      return dfb_region_intersect( region, x1, y1, x2, y2 );
 }
 
-int dfb_unsafe_region_rectangle_intersect( DFBRegion          *region,
-                                           const DFBRectangle *rect )
+bool
+dfb_unsafe_region_rectangle_intersect( DFBRegion          *region,
+                                       const DFBRectangle *rect )
 {
      if (region->x1 > region->x2) {
           int temp = region->x1;
@@ -206,8 +159,9 @@ int dfb_unsafe_region_rectangle_intersect( DFBRegion          *region,
      return dfb_region_rectangle_intersect( region, rect );
 }
 
-int dfb_rectangle_intersect_by_unsafe_region( DFBRectangle *rectangle,
-                                              DFBRegion    *region )
+bool
+dfb_rectangle_intersect_by_unsafe_region( DFBRectangle *rectangle,
+                                          DFBRegion    *region )
 {
      /* validate region */
      if (region->x1 > region->x2) {
@@ -245,14 +199,47 @@ int dfb_rectangle_intersect_by_unsafe_region( DFBRectangle *rectangle,
           rectangle->w = 0;
           rectangle->h = 0;
 
-          return 0;
+          return false;
      }
 
-     return 1;
+     return true;
 }
 
-int dfb_rectangle_intersect( DFBRectangle       *rectangle,
-                             const DFBRectangle *clip )
+bool
+dfb_rectangle_intersect_by_region( DFBRectangle    *rectangle,
+                                   const DFBRegion *region )
+{
+     /* adjust position */
+     if (region->x1 > rectangle->x) {
+          rectangle->w -= region->x1 - rectangle->x;
+          rectangle->x = region->x1;
+     }
+
+     if (region->y1 > rectangle->y) {
+          rectangle->h -= region->y1 - rectangle->y;
+          rectangle->y = region->y1;
+     }
+
+     /* adjust size */
+     if (region->x2 <= rectangle->x + rectangle->w)
+        rectangle->w = region->x2 - rectangle->x + 1;
+
+     if (region->y2 <= rectangle->y + rectangle->h)
+        rectangle->h = region->y2 - rectangle->y + 1;
+
+     /* set size to zero if there's no intersection */
+     if (rectangle->w <= 0 || rectangle->h <= 0) {
+          rectangle->w = 0;
+          rectangle->h = 0;
+
+          return false;
+     }
+
+     return true;
+}
+
+bool dfb_rectangle_intersect( DFBRectangle       *rectangle,
+                              const DFBRectangle *clip )
 {
      DFBRegion region = { clip->x, clip->y,
                           clip->x + clip->w - 1, clip->y + clip->h - 1 };
@@ -280,10 +267,10 @@ int dfb_rectangle_intersect( DFBRectangle       *rectangle,
           rectangle->w = 0;
           rectangle->h = 0;
 
-          return 0;
+          return false;
      }
 
-     return 1;
+     return true;
 }
 
 void dfb_rectangle_union ( DFBRectangle       *rect1,
@@ -292,6 +279,8 @@ void dfb_rectangle_union ( DFBRectangle       *rect1,
      if (!rect2->w || !rect2->h)
           return;
 
+     /* FIXME: OPTIMIZE */
+
      if (rect1->w) {
           int temp = MIN (rect1->x, rect2->x);
           rect1->w = MAX (rect1->x + rect1->w, rect2->x + rect2->w) - temp;
