@@ -45,6 +45,10 @@
 
 #include <pthread.h>
 
+#ifdef USE_SYSFS
+# include <sysfs/libsysfs.h>
+#endif
+
 #include <fusion/shmalloc.h>
 #include <fusion/reactor.h>
 #include <fusion/arena.h>
@@ -222,6 +226,8 @@ static DFBResult dfb_fbdev_blank( int level );
 static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                                      VideoMode             *mode,
                                      CoreLayerRegionConfig *config );
+static void      dfb_fbdev_var_to_mode( struct fb_var_screeninfo *var,
+                                        VideoMode                *mode );
 
 /******************************************************************************/
 
@@ -256,7 +262,7 @@ static DFBResult dfb_fbdev_open()
      if (dfb_config->fb_device) {
           dfb_fbdev->fd = open( dfb_config->fb_device, O_RDWR );
           if (dfb_fbdev->fd < 0) {
-               D_PERROR( "DirectFB/FBDev: Error opening `%s'!\n",
+               D_PERROR( "DirectFB/FBDev: Error opening '%s'!\n",
                          dfb_config->fb_device);
 
                return errno2result( errno );
@@ -265,47 +271,130 @@ static DFBResult dfb_fbdev_open()
      else if (getenv( "FRAMEBUFFER" ) && *getenv( "FRAMEBUFFER" ) != '\0') {
           dfb_fbdev->fd = open( getenv ("FRAMEBUFFER"), O_RDWR );
           if (dfb_fbdev->fd < 0) {
-               D_PERROR( "DirectFB/FBDev: Error opening `%s'!\n",
+               D_PERROR( "DirectFB/FBDev: Error opening '%s'!\n",
                           getenv ("FRAMEBUFFER"));
 
                return errno2result( errno );
           }
      }
      else {
-          dfb_fbdev->fd = open( "/dev/fb0", O_RDWR );
+          dfb_fbdev->fd = direct_try_open( "/dev/fb0", "/dev/fb/0", O_RDWR, true );
           if (dfb_fbdev->fd < 0) {
-               if (errno == ENOENT) {
-                    dfb_fbdev->fd = open( "/dev/fb/0", O_RDWR );
-                    if (dfb_fbdev->fd < 0) {
-                         if (errno == ENOENT) {
-                              D_PERROR( "DirectFB/FBDev: Couldn't open "
-                                         "neither `/dev/fb0' nor `/dev/fb/0'!\n" );
-                         }
-                         else {
-                              D_PERROR( "DirectFB/FBDev: "
-                                         "Error opening `/dev/fb/0'!\n" );
-                         }
-
-                         return errno2result( errno );
-                    }
-               }
-               else {
-                    D_PERROR( "DirectFB/FBDev: Error opening `/dev/fb0'!\n");
-
-                    return errno2result( errno );
-               }
+               D_ERROR( "DirectFB/FBDev: Error opening framebuffer device!\n" );
+               D_ERROR( "DirectFB/FBDev: Use 'fbdev' option or set FRAMEBUFFER environment variable.\n" );
+               return DFB_INIT;
           }
      }
 
      return DFB_OK;
 }
 
+/******************************************************************************/
+
+static void
+dfb_fbdev_get_pci_info( FBDevShared *shared )
+{
+     char buf[512];
+     int  vendor = -1;
+     int  model  = -1;
+
+#ifdef USE_SYSFS
+     if (!sysfs_get_mnt_path( buf, 512 )) {
+          struct sysfs_class_device *classdev;
+          struct sysfs_device       *device;
+          struct sysfs_attribute    *attr;
+          char                      *fbdev;
+          char                       dev[5] = { 'f', 'b', '0', 0, 0 };
+          
+          fbdev = dfb_config->fb_device;
+          if (!fbdev)
+               fbdev = getenv( "FRAMEBUFFER" );
+          
+          if (fbdev) {
+               if (!strncmp( fbdev, "/dev/fb/", 8 ))
+                    snprintf( dev, 5, "fb%s", fbdev+8 );
+               else if (!strncmp( fbdev, "/dev/fb", 7 ))
+                    snprintf( dev, 5, "fb%s", fbdev+7 );
+          }    
+          
+          classdev = sysfs_open_class_device( "graphics", dev );
+          if (classdev) {
+               device = sysfs_get_classdev_device( classdev );
+               
+               if (device) {
+                    attr = sysfs_get_device_attr( device, "vendor" );
+                    if (attr)
+                           sscanf( attr->value, "0x%04x", &vendor );
+                           
+                    attr = sysfs_get_device_attr( device, "device" );
+                    if (attr)
+                         sscanf( attr->value, "0x%04x", &model );
+                         
+                    if (vendor != -1 && model != -1) {
+                         sscanf( device->name, "0000:%02x:%02x.%1x", 
+                                 &shared->pci.bus, 
+                                 &shared->pci.dev, 
+                                 &shared->pci.func );
+                    
+                         shared->device.vendor = vendor;
+                         shared->device.model  = model;
+                    }
+               }
+               
+               sysfs_close_class_device( classdev );
+          }     
+     }
+#endif /* USE_SYSFS */
+
+     /* try /proc interface */
+     if (vendor == -1 || model == -1) {
+          FILE *fp;
+          int   id;
+          int   bus;
+          int   dev;
+          int   func;
+
+          fp = fopen( "/proc/bus/pci/devices", "r" );
+          if (!fp) {
+               D_PERROR( "DirectFB/FBDev: "
+                         "couldn't access /proc/bus/pci/devices!\n" );
+               return;
+          }
+
+          while (fgets( buf, 512, fp )) {
+               if (sscanf( buf, "%04x\t%04x%04x", &id, &vendor, &model ) == 3) {
+                    bus  = (id & 0xff00) >> 8;
+                    dev  = (id & 0x00ff) >> 3;
+                    func = (id & 0x0007);
+                    
+                    if (bus  == dfb_config->pci.bus &&
+                        dev  == dfb_config->pci.dev &&
+                        func == dfb_config->pci.func) 
+                    {
+                         shared->pci.bus  = bus;
+                         shared->pci.dev  = dev;
+                         shared->pci.func = func;
+                         
+                         shared->device.vendor = vendor;
+                         shared->device.model  = model;
+                         
+                         break;
+                    }
+               }
+          }
+
+          fclose( fp );
+     }
+}
+
+
 /** public **/
 
 static void
 system_get_info( CoreSystemInfo *info )
 {
      info->type = CORE_FBDEV;
+     info->caps = CSCAPS_ACCELERATION;
 
      snprintf( info->name, DFB_CORE_SYSTEM_INFO_NAME_LENGTH, "FBDev" );
 }
@@ -405,6 +494,9 @@ system_initialize( CoreDFB *core, void **data )
           return DFB_INIT;
      }
 
+     dfb_fbdev_var_to_mode( &dfb_fbdev->shared->current_var,
+                            &dfb_fbdev->shared->current_mode );
+
      dfb_fbdev->shared->orig_cmap.start  = 0;
      dfb_fbdev->shared->orig_cmap.len    = 256;
      dfb_fbdev->shared->orig_cmap.red    = (__u16*)SHMALLOC( 2 * 256 );
@@ -433,6 +525,8 @@ system_initialize( CoreDFB *core, void **data )
      dfb_fbdev->shared->current_cmap.green  = SHCALLOC( 256, 2 );
      dfb_fbdev->shared->current_cmap.blue   = SHCALLOC( 256, 2 );
      dfb_fbdev->shared->current_cmap.transp = SHCALLOC( 256, 2 );
+     
+     dfb_fbdev_get_pci_info( dfb_fbdev->shared );
 
      fusion_call_init( &dfb_fbdev->shared->fbdev_ioctl,
                        fbdev_ioctl_call_handler, NULL );
@@ -519,13 +613,13 @@ system_shutdown( bool emergency )
 
      if (ioctl( dfb_fbdev->fd, FBIOPUT_VSCREENINFO, &dfb_fbdev->shared->orig_var ) < 0) {
           D_PERROR( "DirectFB/FBDev: "
-                     "Could not restore variable screen information!\n" );
+                    "Could not restore variable screen information!\n" );
      }
 
      if (dfb_fbdev->shared->orig_cmap.len) {
           if (ioctl( dfb_fbdev->fd, FBIOPUTCMAP, &dfb_fbdev->shared->orig_cmap ) < 0)
                D_PERROR( "DirectFB/FBDev: "
-                          "Could not restore palette!\n" );
+                         "Could not restore palette!\n" );
 
           SHFREE( dfb_fbdev->shared->orig_cmap.red );
           SHFREE( dfb_fbdev->shared->orig_cmap.green );
@@ -641,7 +735,9 @@ system_get_accelerator()
      if (!strcmp( dfb_fbdev->shared->fix.id, "MATROX DH" ))
           return FB_ACCEL_MATROX_MGAG400;
 #endif
-     return dfb_fbdev->shared->fix.accel;
+     if (dfb_fbdev->shared->fix.mmio_len > 0)
+          return dfb_fbdev->shared->fix.accel;
+     return -1;
 }
 
 static VideoMode *
@@ -653,7 +749,7 @@ system_get_modes()
 static VideoMode *
 system_get_current_mode()
 {
-     return dfb_fbdev->shared->current_mode;
+     return &dfb_fbdev->shared->current_mode;
 }
 
 static DFBResult
@@ -714,6 +810,22 @@ system_videoram_length()
      return dfb_fbdev->shared->fix.smem_len;
 }
 
+static void
+system_get_busid( int *ret_bus, int *ret_dev, int *ret_func )
+{
+     *ret_bus  = dfb_fbdev->shared->pci.bus;
+     *ret_dev  = dfb_fbdev->shared->pci.dev;
+     *ret_func = dfb_fbdev->shared->pci.func;
+}
+
+static void
+system_get_deviceid( unsigned int *ret_vendor_id,
+                     unsigned int *ret_device_id )
+{
+     *ret_vendor_id = dfb_fbdev->shared->device.vendor;
+     *ret_device_id = dfb_fbdev->shared->device.model;
+}
+
 /******************************************************************************/
 
 static DFBResult
@@ -725,27 +837,7 @@ init_modes()
           /* try to use current mode*/
           dfb_fbdev->shared->modes = (VideoMode*) SHCALLOC( 1, sizeof(VideoMode) );
 
-          dfb_fbdev->shared->modes->xres = dfb_fbdev->shared->orig_var.xres;
-          dfb_fbdev->shared->modes->yres = dfb_fbdev->shared->orig_var.yres;
-          dfb_fbdev->shared->modes->bpp  = dfb_fbdev->shared->orig_var.bits_per_pixel;
-          dfb_fbdev->shared->modes->hsync_len = dfb_fbdev->shared->orig_var.hsync_len;
-          dfb_fbdev->shared->modes->vsync_len = dfb_fbdev->shared->orig_var.vsync_len;
-          dfb_fbdev->shared->modes->left_margin = dfb_fbdev->shared->orig_var.left_margin;
-          dfb_fbdev->shared->modes->right_margin = dfb_fbdev->shared->orig_var.right_margin;
-          dfb_fbdev->shared->modes->upper_margin = dfb_fbdev->shared->orig_var.upper_margin;
-          dfb_fbdev->shared->modes->lower_margin = dfb_fbdev->shared->orig_var.lower_margin;
-          dfb_fbdev->shared->modes->pixclock = dfb_fbdev->shared->orig_var.pixclock;
-
-
-          if (dfb_fbdev->shared->orig_var.sync & FB_SYNC_HOR_HIGH_ACT)
-               dfb_fbdev->shared->modes->hsync_high = 1;
-          if (dfb_fbdev->shared->orig_var.sync & FB_SYNC_VERT_HIGH_ACT)
-               dfb_fbdev->shared->modes->vsync_high = 1;
-
-          if (dfb_fbdev->shared->orig_var.vmode & FB_VMODE_INTERLACED)
-               dfb_fbdev->shared->modes->laced = 1;
-          if (dfb_fbdev->shared->orig_var.vmode & FB_VMODE_DOUBLE)
-               dfb_fbdev->shared->modes->doubled = 1;
+          *dfb_fbdev->shared->modes = dfb_fbdev->shared->current_mode;
 
           if (dfb_fbdev_set_mode(NULL, dfb_fbdev->shared->modes, NULL)) {
                D_ERROR("DirectFB/FBDev: "
@@ -841,25 +933,11 @@ primaryGetScreenSize( CoreScreen *screen,
                       int        *ret_width,
                       int        *ret_height )
 {
-     FBDevShared *shared;
-
      D_ASSERT( dfb_fbdev != NULL );
      D_ASSERT( dfb_fbdev->shared != NULL );
 
-     shared = dfb_fbdev->shared;
-
-     if (shared->current_mode) {
-          *ret_width  = shared->current_mode->xres;
-          *ret_height = shared->current_mode->yres;
-     }
-     else if (shared->modes) {
-          *ret_width  = shared->modes->xres;
-          *ret_height = shared->modes->yres;
-     }
-     else {
-          D_WARN( "no current and no default mode" );
-          return DFB_UNSUPPORTED;
-     }
+     *ret_width  = dfb_fbdev->shared->current_mode.xres;
+     *ret_height = dfb_fbdev->shared->current_mode.yres;
 
      return DFB_OK;
 }
@@ -1152,21 +1230,25 @@ primarySetRegion( CoreLayer                  *layer,
 {
      DFBResult  ret;
      VideoMode *videomode;
+     VideoMode *highest = NULL;
 
      videomode = dfb_fbdev->shared->modes;
      while (videomode) {
           if (videomode->xres == config->width  &&
               videomode->yres == config->height)
-               break;
+          {
+               if (!highest || highest->priority < videomode->priority)
+                    highest = videomode;
+          }
 
           videomode = videomode->next;
      }
 
-     if (!videomode)
+     if (!highest)
           return DFB_UNSUPPORTED;
 
      if (updated & (CLRCF_BUFFERMODE | CLRCF_FORMAT | CLRCF_HEIGHT | CLRCF_SURFACE | CLRCF_WIDTH)) {
-          ret = dfb_fbdev_set_mode( surface, videomode, config );
+          ret = dfb_fbdev_set_mode( surface, highest, config );
           if (ret)
                return ret;
      }
@@ -1233,6 +1315,8 @@ primaryAllocateSurface( CoreLayer              *layer,
      else if (config->buffermode != DLBM_FRONTONLY)
           caps |= DSCAPS_DOUBLE;
 
+     caps |= config->surface_caps & DSCAPS_PREMULTIPLIED;
+
      /* allocate surface object */
      surface = dfb_core_create_surface( dfb_fbdev->core );
      if (!surface)
@@ -1294,12 +1378,40 @@ primaryReallocateSurface( CoreLayer             *layer,
           dfb_palette_unref( palette );
      }
 
+     if (config->surface_caps & DSCAPS_PREMULTIPLIED)
+          surface->caps |= DSCAPS_PREMULTIPLIED;
+     else
+          surface->caps &= ~DSCAPS_PREMULTIPLIED;
+
      return DFB_OK;
 }
 
 
 /** fbdev internal **/
 
+static void dfb_fbdev_var_to_mode( struct fb_var_screeninfo *var, 
+                                   VideoMode                *mode )
+{
+     mode->xres          = var->xres;
+     mode->yres          = var->yres;
+     mode->bpp           = var->bits_per_pixel;
+     mode->hsync_len     = var->hsync_len;
+     mode->vsync_len     = var->vsync_len;
+     mode->left_margin   = var->left_margin;
+     mode->right_margin  = var->right_margin;
+     mode->upper_margin  = var->upper_margin;
+     mode->lower_margin  = var->lower_margin;
+     mode->pixclock      = var->pixclock;
+     mode->hsync_high    = (var->sync & FB_SYNC_HOR_HIGH_ACT) ? 1 : 0;
+     mode->vsync_high    = (var->sync & FB_SYNC_VERT_HIGH_ACT) ? 1 : 0;
+     mode->csync_high    = (var->sync & FB_SYNC_COMP_HIGH_ACT) ? 1 : 0;
+     mode->sync_on_green = (var->sync & FB_SYNC_ON_GREEN) ? 1 : 0;
+     mode->external_sync = (var->sync & FB_SYNC_EXT) ? 1 : 0;
+     mode->broadcast     = (var->sync & FB_SYNC_BROADCAST) ? 1 : 0;
+     mode->laced         = (var->vmode & FB_VMODE_INTERLACED) ? 1 : 0;
+     mode->doubled       = (var->vmode & FB_VMODE_DOUBLE) ? 1 : 0;
+}
+
 static int dfb_fbdev_compatible_format( struct fb_var_screeninfo *var,
                                         int al, int rl, int gl, int bl,
                                         int ao, int ro, int go, int bo )
@@ -1352,6 +1464,10 @@ static DFBSurfacePixelFormat dfb_fbdev_get_pixelformat( struct fb_var_screeninfo
                    dfb_fbdev_compatible_format( var, 1, 5, 5, 5,15, 10, 5, 0 ) )
                     return DSPF_ARGB1555;
 
+               if (dfb_fbdev_compatible_format( var, 0, 4, 4, 4,  0, 8, 4, 0 ) |
+                   dfb_fbdev_compatible_format( var, 4, 4, 4, 4, 12, 8, 4, 0 ) )
+                    return DSPF_ARGB4444;
+
                if (dfb_fbdev_compatible_format( var, 0, 5, 6, 5, 0, 11, 5, 0 ))
                     return DSPF_RGB16;
 
@@ -1448,7 +1564,7 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
               config ? config->buffermode : DLBM_FRONTONLY);
 
      if (!mode)
-          mode = shared->current_mode ? shared->current_mode : shared->modes;
+          mode = &shared->current_mode;
 
      vyres = mode->yres;
 
@@ -1491,6 +1607,17 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                     var.blue.offset   = 0;
                     break;
 
+               case DSPF_ARGB4444:
+                    var.transp.length = 4;
+                    var.red.length    = 4;
+                    var.green.length  = 4;
+                    var.blue.length   = 4;
+                    var.transp.offset = 12;
+                    var.red.offset    = 8;
+                    var.green.offset  = 4;
+                    var.blue.offset   = 0;
+                    break;
+
                case DSPF_RGB16:
                     var.red.length    = 5;
                     var.green.length  = 6;
@@ -1634,7 +1761,11 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                dfb_fbdev_set_gamma_ramp( config->format );
 
           shared->current_var = var;
-          shared->current_mode = mode;
+          dfb_fbdev_var_to_mode( &var, &shared->current_mode );
+
+          /* invalidate original pan offset */
+          shared->orig_var.xoffset = 0;
+          shared->orig_var.yoffset = 0;
 
           surface->width  = mode->xres;
           surface->height = mode->yres;
