@@ -1,12 +1,13 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2002-2004  convergence GmbH.
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org> and
+              Ville Syrjl <syrjala@sci.fi>.
 
    directfb-csource is based on gdk-pixbuf-csource, a GdkPixbuf
    based image CSource generator  Copyright (C) 1999, 2001 Tim Janik
@@ -27,21 +28,22 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
-#include <libgen.h>
 
 #include <png.h>
 
-#include "directfb.h"
+#include <directfb.h>
 
-#include "gfx/convert.h"
-#include "misc/util.h"
+#include <direct/types.h>
+#include <direct/util.h>
+
+#include <gfx/convert.h>
 
 
 static struct {
@@ -50,6 +52,8 @@ static struct {
 } pixelformats[] = {
      { DSPF_ARGB,     "ARGB"     },
      { DSPF_ARGB1555, "ARGB1555" },
+     { DSPF_ARGB2554, "ARGB2554" },
+     { DSPF_ARGB4444, "ARGB4444" },
      { DSPF_RGB32,    "RGB32"    },
      { DSPF_RGB24,    "RGB24"    },
      { DSPF_RGB16,    "RGB16"    },
@@ -60,15 +64,25 @@ static struct {
 static int n_pixelformats = sizeof (pixelformats) / sizeof (pixelformats[0]);
 
 
-static DFBResult  load_image  (const char            *filename,
-                               DFBSurfaceDescription *desc,
-                               DFBColor              *palette,
-                               int                   *palette_size);
-static DFBResult  dump_image  (const char            *name,
-                               DFBSurfaceDescription *desc,
-                               DFBColor              *palette,
-                               int                    palette_size);
-static void       print_usage (const char            *prg_name);
+static void       print_usage     (const char             *prg_name);
+static DFBResult  load_image      (const char             *filename,
+                                   DFBSurfaceDescription  *desc,
+                                   DFBColor               *palette,
+                                   int                    *palette_size);
+static DFBResult  merge_images    (DFBSurfaceDescription  *images,
+                                   int                     num_images,
+                                   DFBSurfaceDescription  *dest,
+                                   DFBRectangle           *rectangles);
+static DFBResult  dump_image      (const char             *name,
+                                   DFBSurfaceDescription  *desc,
+                                   DFBColor               *palette,
+                                   int                     palette_size);
+static DFBResult  dump_rectangles (const char             *name,
+                                   DFBRectangle           *rectangles,
+                                   const char            **names,
+                                   int                     num_rects);
+static char *     variable_name   (const char             *name);
+static char *     base_name       (const char             *name);
 
 
 int main (int         argc,
@@ -76,77 +90,107 @@ int main (int         argc,
 {
      DFBSurfaceDescription  desc   = { flags: 0 };
      DFBSurfacePixelFormat  format = DSPF_UNKNOWN;
-     DFBColor    palette[256];
-     int         palette_size = 0;
-     const char *name         = NULL;
-     const char *filename     = NULL;
-     char       *vname;
-     int         i, j, n;
+     DFBColor  palette[256];
+
+     const char *filename[argc];
+     const char *name           = NULL;
+     int         palette_size   = 0;
+     int         num_images     = 0;
+     int         i, n;
+
+     /*  parse command line  */
 
-     /* parse command line */
      for (n = 1; n < argc; n++) {
           if (strncmp (argv[n], "--", 2) == 0) {
-               if (strcmp (argv[n] + 2, "help") == 0) {
+
+               const char *arg = argv[n] + 2;
+
+               if (strcmp (arg, "help") == 0) {
                     print_usage (argv[0]);
                     return EXIT_SUCCESS;
                }
-               if (strcmp (argv[n] + 2, "version") == 0) {
+               if (strcmp (arg, "version") == 0) {
                     fprintf (stderr, "directfb-csource version %s\n",
                              DIRECTFB_VERSION);
                     return EXIT_SUCCESS;
                }
-               if (strncmp (argv[n] + 2, "format=", 7) == 0 && !format) {
-                    for (j = 0; j < n_pixelformats && !format; j++)
-                         if (!strcasecmp (pixelformats[j].name, argv[n] + 9))
-                              format = pixelformats[j].format;
+               if (strncmp (arg, "format=", 7) == 0 && !format) {
+                    for (i = 0; i < n_pixelformats && !format; i++)
+                         if (!strcasecmp (pixelformats[i].name, arg + 7))
+                              format = pixelformats[i].format;
                     if (format)
                          continue;
                }
-               if (strncmp (argv[n] + 2, "name=", 5) == 0 && !name) {
-                    name = argv[n] + 7;
+               if (strncmp (arg, "name=", 5) == 0 && !name) {
+                    name = arg + 5;
                     if (*name)
                          continue;
                }
-               filename = ""; /* bail out below */
-          }
-          if (filename) {
+
                print_usage (argv[0]);
                return EXIT_FAILURE;
           }
-          filename = argv[n];
+
+          filename[num_images++] = argv[n];
      }
 
-     if (!filename) {
+     /*  check parameters  */
+
+     if (! num_images) {
           print_usage (argv[0]);
           return EXIT_FAILURE;
      }
 
-     if (name)
-          vname = strdup (name);
-     else
-          vname = basename (strdup (filename));
-
-     for (i = 0; vname[i]; i++) {
-          switch (vname[i]) {
-               case 'a'...'z':
-               case 'A'...'Z':
-               case '0'...'9':
-               case '_':
-                    break;
-               default:
-                    vname[i] = '_';
-          }
+     if (num_images > 1 && !name) {
+          fprintf (stderr,
+                   "You must specify a variable name when using multiple images.\n");
+          return EXIT_FAILURE;
      }
 
+     /*  load the first image  */
+
      if (format) {
           desc.flags = DSDESC_PIXELFORMAT;
           desc.pixelformat = format;
      }
 
-     if (load_image (filename, &desc, palette, &palette_size) != DFB_OK)
+     if (load_image (filename[0], &desc, palette, &palette_size) != DFB_OK)
           return EXIT_FAILURE;
 
-     return dump_image (vname, &desc, palette, palette_size);
+     /*  dump it and quit if this is the only image on the command line  */
+
+     if (num_images == 1)
+          return dump_image (name ? : strrchr (filename[0], '/') ? : filename[0],
+                             &desc, palette, palette_size);
+
+     /*  merge multiple images into one surface  */
+     {
+          DFBSurfaceDescription  image[num_images];
+          DFBRectangle           rect[num_images];
+          DFBColor               foo[256];
+          int                    foo_size;
+
+          image[0] = desc;
+
+          for (i = 1; i < num_images; i++) {
+               image[i].flags = DSDESC_PIXELFORMAT;
+               image[i].pixelformat = desc.pixelformat;
+
+               if (load_image (filename[i],
+                               image + i, foo, &foo_size) != DFB_OK)
+                    return EXIT_FAILURE;
+          }
+
+          if (merge_images (image, num_images, &desc, rect) != DFB_OK)
+               return EXIT_FAILURE;
+
+          /*  dump the rectangles, then the surface  */
+
+          if (dump_rectangles (name, rect, filename, num_images) != DFB_OK)
+               return EXIT_FAILURE;
+
+          return dump_image (name, &desc, palette, palette_size);
+     }
 }
 
 static void print_usage (const char *prg_name)
@@ -155,9 +199,12 @@ static void print_usage (const char *prg_name)
      fprintf (stderr, "Usage: %s [options] <imagefile>\n", prg_name);
      fprintf (stderr, "   --name=<identifer>     specifies variable name\n");
      fprintf (stderr, "   --format=<identifer>   specifies surface format\n");
+     fprintf (stderr, "   --multi                multiple images\n");
      fprintf (stderr, "   --help                 show this help message\n");
      fprintf (stderr, "   --version              print version information\n");
      fprintf (stderr, "\n");
+     fprintf (stderr, "See the directfb-csource(1) man-page for more information.\n");
+     fprintf (stderr, "\n");
 }
 
 static DFBResult load_image (const char            *filename,
@@ -306,8 +353,7 @@ static DFBResult load_image (const char            *filename,
 
      data  = malloc (height * pitch);
      if (!data) {
-          fprintf (stderr, "Failed to allocate %ld bytes.\n",
-                   height * pitch);
+          fprintf (stderr, "Failed to allocate %ld bytes.\n", height * pitch);
           goto cleanup;
      }
 
@@ -351,6 +397,14 @@ static DFBResult load_image (const char            *filename,
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
                          dfb_argb_to_argb1555 ((__u32 *) s, (__u16 *) d, width);
                     break;
+               case DSPF_ARGB2554:
+                    for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
+                         dfb_argb_to_argb2554 ((__u32 *) s, (__u16 *) d, width);
+                    break;
+               case DSPF_ARGB4444:
+                    for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
+                         dfb_argb_to_argb4444 ((__u32 *) s, (__u16 *) d, width);
+                    break;
                case DSPF_RGB332:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
                          dfb_argb_to_rgb332 ((__u32 *) s, (__u8 *) d, width);
@@ -393,6 +447,87 @@ static DFBResult load_image (const char            *filename,
      return ((desc->flags) ? DFB_OK : DFB_FAILURE);
 }
 
+static DFBResult merge_images (DFBSurfaceDescription *images,
+                               int                    num_images,
+                               DFBSurfaceDescription *dest,
+                               DFBRectangle          *rectangles)
+{
+  DFBSurfaceDescription *image = images;
+  DFBRectangle          *rect  = rectangles;
+  unsigned char         *data;
+  int                    bpp;
+  int                    pitch, i;
+
+  rect->x = 0;
+  rect->y = 0;
+  rect->w = image->width;
+  rect->h = image->height;
+
+  dest->flags       = (DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT |
+                       DSDESC_PREALLOCATED);
+  dest->pixelformat = image->pixelformat;
+
+  bpp = DFB_BYTES_PER_PIXEL (dest->pixelformat);
+
+  if (bpp == 1)
+       dest->width  = (rect->w + 3) & ~3;
+  else
+       dest->width  = rect->w;
+
+  dest->height      = rect->h;
+
+  for (i = 1; i < num_images; i++) {
+       image++;
+       rect++;
+
+       if (image->pixelformat != dest->pixelformat)
+            return DFB_INVARG;
+
+       rect->x = dest->width;
+       rect->y = 0;
+       rect->w = image->width;
+       rect->h = image->height;
+
+       if (bpp == 1)
+            dest->width += (rect->w + 3) & ~3;
+       else
+            dest->width += rect->w;
+
+       if (dest->height < rect->h)
+            dest->height = rect->h;
+  }
+
+  pitch = (dest->width * bpp + 3) &~ 3;
+  data  = malloc (dest->height * pitch);
+  if (!data) {
+       fprintf (stderr, "Failed to allocate %ld bytes.\n",
+                (long) dest->height * pitch);
+       return DFB_FAILURE;
+  }
+
+
+  for (i = 0, image = images, rect = rectangles;
+       i < num_images;
+       i++, image++, rect++) {
+
+       unsigned char *dest   = data + rect->x * bpp;
+       unsigned char *src    = image->preallocated[0].data;
+       int            height = rect->h;
+
+       do {
+            memcpy (dest, src, rect->w * bpp);
+            src  += image->preallocated[0].pitch;
+            dest += pitch;
+       }
+       while (--height);
+  }
+
+  dest->preallocated[0].pitch = pitch;
+  dest->preallocated[0].data  = data;
+
+  return DFB_OK;
+}
+
 
 typedef struct {
      FILE  *fp;
@@ -400,10 +535,8 @@ typedef struct {
      bool    pad;
 } CSourceData;
 
-
-static inline void
-save_uchar (CSourceData   *csource,
-	    unsigned char  d)
+static inline void save_uchar (CSourceData   *csource,
+                               unsigned char  d)
 {
      if (csource->pos > 70) {
           fprintf (csource->fp, "\"\n  \"");
@@ -445,6 +578,7 @@ static DFBResult dump_image (const char            *name,
 {
      CSourceData    csource = { stdout, 0, 0 };
      const char    *format  = NULL;
+     char          *vname   = variable_name (name);
      unsigned char *data;
      unsigned long  len;
      int            i;
@@ -473,7 +607,7 @@ static DFBResult dump_image (const char            *name,
 
      /* dump data */
      fprintf (csource.fp,
-              "static unsigned char %s_data[] = \n", name);
+              "static unsigned char %s_data[] =\n", vname);
      fprintf (csource.fp, "  \"");
 
      csource.pos = 3;
@@ -486,7 +620,7 @@ static DFBResult dump_image (const char            *name,
      /* dump palette */
      if (palette_size > 0) {
           fprintf (csource.fp,
-                   "static DFBColor %s_palette[%d] = {\n", name, palette_size);
+                   "static DFBColor %s_palette[%d] = {\n", vname, palette_size);
           for (i = 0; i < palette_size; i++)
                fprintf (csource.fp,
                         "  { 0x%02x, 0x%02x, 0x%02x, 0x%02x }%c\n",
@@ -497,7 +631,7 @@ static DFBResult dump_image (const char            *name,
 
      /* dump description */
      fprintf (csource.fp,
-              "static DFBSurfaceDescription %s_desc = {\n", name);
+              "static DFBSurfaceDescription %s_desc = {\n", vname);
      fprintf (csource.fp,
               "  flags                   : DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT |\n"
               "                            DSDESC_PREALLOCATED");
@@ -511,17 +645,119 @@ static DFBResult dump_image (const char            *name,
      fprintf (csource.fp,
               "  pixelformat             : DSPF_%s,\n", format);
      fprintf (csource.fp,
-              "  preallocated : {{  data : (void *) %s_data,\n", name);
+              "  preallocated : {{  data : (void *) %s_data,\n", vname);
      fprintf (csource.fp,
               "                    pitch : %d  }}", desc->preallocated[0].pitch);
      if (palette_size > 0) {
           fprintf (csource.fp, ",\n");
           fprintf (csource.fp,
-                   "  palette :    {  entries : %s_palette,\n", name);
+                   "  palette :    {  entries : %s_palette,\n", vname);
           fprintf (csource.fp,
                    "                     size : %d  }", palette_size);
      }
      fprintf (csource.fp, "\n};\n\n");
 
+     free (vname);
+
      return DFB_OK;
 }
+
+static DFBResult dump_rectangles (const char    *name,
+                                  DFBRectangle  *rectangles,
+                                  const char   **names,
+                                  int            num_rects)
+{
+     DFBRectangle *rect;
+     const char   *blanks = "                                  ";
+     char         *vname  = variable_name (name);
+     FILE         *fp     = stdout;
+     int           len, i;
+
+     if (num_rects < 1)
+          return DFB_INVARG;
+
+     fprintf (fp,
+              "/* DirectFB multi-surface dump created by directfb-csource %s */\n\n",
+              DIRECTFB_VERSION);
+
+     fprintf (fp,
+              "static struct {\n"
+              "  const char   *name;\n"
+              "  DFBRectangle  rect;\n"
+              "} %s[] = {\n", vname);
+
+     for (i = 0, len = 0; i < num_rects; i++)
+          len = MAX (len, strlen (names[i]));
+
+     len = len + 4 - strlen (blanks);
+
+     for (i = 0, rect = rectangles; i < num_rects; i++, rect++) {
+
+          char *v = base_name (names[i]);
+
+          if (i)
+               fprintf (fp, ",\n");
+
+          if (len < 0) {
+               int l = fprintf (fp, "  { \"%s\", ", v);
+
+               fprintf (fp, blanks - len + l);
+               fprintf (fp, "{ x : %4d, y : %4d, w : %4d, h : %4d } }",
+                        rect->x, rect->y, rect->w, rect->h);
+          }
+          else {
+               fprintf (fp,
+                        "  { \"%s\",\n"
+                        "    { x : %4d, y : %4d, w : %4d, h : %4d } }",
+                        v, rect->x, rect->y, rect->w, rect->h);
+          }
+
+          free (v);
+     }
+     fprintf (fp, "\n};\n\n");
+
+     free (vname);
+
+     return DFB_OK;
+}
+
+static char *
+variable_name (const char *name)
+{
+     char *vname = strdup (name);
+     char *v     = vname;
+
+     while (DFB_TRUE) {
+          switch (*v) {
+               case 0:
+                    return vname;
+               case 'a'...'z':
+               case 'A'...'Z':
+               case '0'...'9':
+               case '_':
+                    break;
+               default:
+                    *v = '_';
+          }
+          v++;
+     }
+}
+
+static char *
+base_name (const char *name)
+{
+     char *vname = strdup (name);
+     char *v     = vname;
+
+     while (DFB_TRUE) {
+          switch (*v) {
+               case '.':
+                    *v = 0;
+               case 0:
+                    return vname;
+               default:
+                    break;
+          }
+          v++;
+     }
+}
