@@ -2,7 +2,7 @@
 #
 #   (c) Copyright 2000-2002  convergence integrated media GmbH.
 #   (c) Copyright 2002       convergence GmbH.
-#   
+#
 #   All rights reserved.
 #
 #   Written by Denis Oliver Kropp <dok@directfb.org>,
@@ -35,76 +35,212 @@
 ################################################################
 
 
-html_create( INDEX, "index.html", "DirectFB Interfaces" );
+html_create( INDEX, "index.html", "Index Page" );
 html_create( TYPES, "types.html", "DirectFB Types" );
 
 print INDEX "<P>\n",
-            "  <CENTER><TABLE width=90% border=0 cellpadding=2>\n";
+            "  <CENTER>\n",
+            "    <H3>Interfaces</H3>\n",
+            "    <TABLE width=90% border=0 cellpadding=2>\n";
 
-while (<>)
-   {
-      chomp;
+while (<>) {
+   chomp;
 
-      if ( /^\s*\/\*\s*$/ )
-         {
-           $comment = "<p>\n";
+   if ( /^\s*\/\*\s*$/ ) {
+      %options = ();
+      $comment = "";
 
-            while (<>)
-               {
-                  chomp;
-                  last if ( /^\s*\*\/\s*$/ );
+      while (<>) {
+         chomp;
 
-                  if (/^\s*\*\s*(.*)$/)
-                    {
-                      if ($1 eq "")
-                        {
-                          $comment .= "</p><p>\n";
-                        }
-                      else
-                        {
-                          $comment .= "   $1\n";
-                        }
-                    }
-               }
+         last if ( /^\s*\*\/\s*$/ );
 
-           $comment .= "</p>\n";
-         }
-      elsif ( /^\s*DECLARE_INTERFACE\s*\(\s*(\w+)\s\)\s*$/ )
-         {
-            $interface_abstracts{$1} = $comment;
-            print INDEX "    <TR><TD valign=top>\n",
-                        "      <A href=\"$1.html\">$1</A>\n",
-                        "    </TD><TD valign=top>\n",
-                        "      $comment\n",
-                        "    </TD></TR>\n";
-         }
-      elsif ( /^\s*DEFINE_INTERFACE\s*\(\s*(\w+),\s*$/ )
-         {
-            parse_interface( $1 );
-         }
-      elsif ( /^\s*typedef\s+enum\s*\{?\s*$/ )
-         {
-            parse_enum();
-         }
-      elsif ( /^\s*typedef\s+(struct|union)\s*\{?\s*$/ )
-         {
-            parse_struct();
-         }
-      else
-         {
-            $comment = "";
+         if (/^\s*\*\s*(.*)$/) {
+            $line = $1;
+
+            if ($line eq "" && $comment ne "") {
+               $comment .= "    <br><br>\n";
+            }
+            elsif ($line =~ /^@(\w+)\s*=?\s*(.*)$/) {
+               $options{$1} = $2;
+            }
+            else {
+               $comment .= "      $line\n";
+            }
          }
+      }
+
+      substitute_method_links (\$comment);
    }
+   elsif ( /^\s*DECLARE_INTERFACE\s*\(\s*(\w+)\s\)\s*$/ ) {
+      $interface_abstracts{$1} = $comment;
 
-print INDEX "  </CENTER></TABLE>\n",
+      print INDEX "    <TR><TD valign=top>\n",
+                  "      <A href=\"$1.html\">$1</A>\n",
+                  "    </TD><TD valign=top>\n",
+                  "      $comment\n",
+                  "    </TD></TR>\n";
+   }
+   elsif ( /^\s*DEFINE_INTERFACE\s*\(\s*(\w+),\s*$/ ) {
+      parse_interface( $1 );
+   }
+   elsif ( /^\s*typedef\s+enum\s*\{?\s*$/ ) {
+      parse_enum();
+   }
+   elsif ( /^\s*typedef\s+(struct|union)\s*\{?\s*$/ ) {
+      parse_struct();
+   }
+   elsif ( /^\s*typedef\s+(\w+)\s+\(\*(\w+)\)\s*\(\s*$/ ) {
+      parse_func( $1, $2 );
+   }
+   elsif ( /^\s*#define\s+([^\(\s]+)\s*(\([^\)]*\))?/ ) {
+      $macro  = $1;
+      $params = $2;
+
+      chomp $params;
+
+      if ($comment ne "" && !defined ($options{"internal"})) {
+         parse_macro( $macro, $params );
+      }
+   }
+   else {
+      $comment = "";
+   }
+}
+
+print INDEX "  </TABLE></CENTER>\n",
             "</P>\n";
 
+print_list( \%types, "Types" );
+print_list( \%definitions, "Definitions" );
+
+
 html_close( INDEX );
 html_close( TYPES );
 
+
+
+
+
+sub print_list ($$) {
+   local (*list, $title) = @_;
+
+   print INDEX "<P>\n",
+               "  <CENTER>\n",
+               "    <H3>$title</H3>\n",
+               "    <TABLE width=90% border=0 cellpadding=2>\n";
+
+   foreach $key (sort keys %list)
+      {
+         print INDEX "    <TR><TD valign=top>\n",
+                     "      <A href=\"types.html#$key\">$key</A>\n",
+                     "    </TD><TD valign=top>\n",
+                     "      $list{$key}\n",
+                     "    </TD></TR>\n";
+      }
+
+   print INDEX "  </TABLE></CENTER>\n",
+               "</P>\n";
+}
+
+
+
+
+
+sub substitute_method_links ($)
+   {
+      my $str = shift(@_);
+
+      $$str =~ s/(I\w+)\:\:(\w+)\(\)/\<a\ href=\"\1\_\2\.html\"\>\1\:\:\2\(\)\<\/a\>/g;
+   }
+
+sub parse_comment ($$) {
+   local (*head, *body) = @_;
+   my $headline_mode = 1;
+
+   $head = "";
+   $body = "";
+
+
+   while (<>)
+      {
+         chomp;
+         last if /^\s*\*\/\s*$/;
+
+         if ($headline_mode == 1)
+            {
+               if (/^\s*\*?\s*$/)
+                  {
+                     $headline_mode = 0;
+                  }
+               elsif (/^\s*\*?\s*(.+)$/)
+                  {
+                     $head .= " $1";
+                  }
+            }
+         else
+            {
+               if (/^\s*\*?\s*$/)
+                  {
+                     $body .= " </P><P>";
+                  }
+               elsif (/^\s*\*?\s*(.+)$/)
+                  {
+                     $body .= " $1";
+                  }
+            }
+      }
+
+   substitute_method_links (\$head);
+   substitute_method_links (\$body);
+}
+
+sub parse_params () {
+   my @entries;
+
+   while (<>)
+      {
+         chomp;
+         last if /^\s*\)\;\s*$/;
+
+         if ( /^\s*(const )?\s*([\w\ ]+)\s+(\**)(\w+,?)\s*$/ )
+            {
+               my $const = $1;
+               my $type  = $2;
+               my $ptr   = $3;
+               my $name  = $4;
+
+               $type =~ s/\s*$//g;
+
+               if ($types{$type})
+                  {
+                     $type = "$const<A href=\"types.html#$type\">$type</A>";
+                  }
+               elsif ($interface_abstracts{$type})
+                  {
+                     $type = "$const<A href=\"$type.html\">$type</A>";
+                  }
+               else
+                  {
+                     $type = "$const$type";
+                  }
+
+               my $rec = {
+                  TYPE   => $type,
+                  PTR    => $ptr,
+                  NAME   => $name
+               };
+
+               push (@entries, $rec);
+            }
+      }
+
+   return @entries;
+}
+
 #
 # Reads stdin until the end of the interface is reached.
-# Writes formatted HTML to stdout.
+# Writes formatted HTML to one file for the interface and one file per method.
 # Parameter is the interface name.
 #
 sub parse_interface (NAME)
@@ -124,8 +260,7 @@ sub parse_interface (NAME)
 #                      "  $interface_abstracts{$interface}\n",
 #                      "</P>";
 
-      $comment =~ s/\<p\>/\<p style=\"margin\-left:3%; margin\-right:3%;\"\>/g;
-      print INTERFACE "$comment";
+      print INTERFACE "<p style=\"margin\-left:3%; margin\-right:3%;\"\>$comment</p>";
 
       print INTERFACE "<P>\n",
                       "  <CENTER><TABLE width=93% border=1 rules=groups cellpadding=2 cellspacing=0>\n";
@@ -135,7 +270,7 @@ sub parse_interface (NAME)
       print INTERFACE "    </THEAD>\n";
 
       print INTERFACE "    <TBODY>\n";
-      
+
       while (<>)
          {
             chomp;
@@ -151,7 +286,7 @@ sub parse_interface (NAME)
                                   "      <B><SMALL>$section</SMALL></B>\n",
                                   "    </TD><TD valign=top>\n",
                                   "      <A href=\"${interface}_$1.html\">",
-                                         "<B>$1</B></A>\n",
+                                  "      <B>$1</B></A>\n",
                                   "    </TD><TD valign=top>\n",
                                   "      $headline\n",
                                   "    </TD></TR>\n";
@@ -162,37 +297,20 @@ sub parse_interface (NAME)
                                "</A>", $1 );
 
                   print FUNCTION "<P>$headline</P><P>\n",
-                                 "  <TABLE border=0 cellspacing=0 cellpadding=4 bgcolor=#C0C0C0>\n",
-                                 "    <TR><TD colspan=3><I><FONT color=black>$1 (</FONT></I></TD></TR>\n";
+                                 "  <TABLE border=0 cellspacing=0 cellpadding=2 bgcolor=#C0C0C0>\n",
+                                 "    <TR><TD colspan=5><I><FONT color=black>$1 (</FONT></I></TD></TR>\n";
 
-                  while (<>)
+                  my @params = parse_params();
+
+                  for my $param (@params)
                      {
-                        chomp;
-                        last if /^\s*\)\;\s*$/;
-
-
-                        if ( /^\s*([\w\ ]+)\s+(\**\w+)\s*,?\s*$/ )
-                           {
-                              $type = $1;
-                              $name = $2;
-
-                              $type =~ s/\ *$//;
-
-                              if ($types{$type})
-                                 {
-                                    $type = "<A href=\"types.html#$type\">$type</A>";
-                                 }
-                              elsif ($interface_abstracts{$type})
-                                 {
-                                    $type = "<A href=\"$type.html\">$type</A>";
-                                 }
-
-                              print FUNCTION "    <TR><TD width=50>&nbsp;</TD><TD valign=top>\n",
-                                             "      $type\n",
-                                             "    </TD><TD valign=top>\n",
-                                             "      <FONT color=black><B>$name</B></FONT>\n",
-                                             "    </TD></TR>\n";
-                           }
+                        print FUNCTION "    <TR><TD width=50>&nbsp;</TD><TD valign=top>\n",
+                                       "      $param->{TYPE}\n",
+                                       "    </TD><TD width=20>&nbsp;</TD><TD align=right>\n",
+                                       "      <FONT color=black><B>$param->{PTR}</B></FONT>\n",
+                                       "    </TD><TD valign=top>\n",
+                                       "      <FONT color=black><B>$param->{NAME}</B></FONT>\n",
+                                       "    </TD></TR>\n";
                      }
 
                   print FUNCTION "    <TR><TD colspan=3><I><FONT color=black>);</FONT></I></TD></TR>\n",
@@ -209,44 +327,12 @@ sub parse_interface (NAME)
                }
             elsif ( /^\s*\/\*\s*$/ )
                {
-                  $headline = "";
-                  $detailed = "";
-
-                  $headline_mode = 1;
-
-                  while (<>)
-                     {
-                        chomp;
-                        last if /^\s*\*\/\s*$/;
-
-                        if ($headline_mode == 1)
-                           {
-                              if (/^\s*\*?\s*$/)
-                                 {
-                                    $headline_mode = 0;
-                                 }
-                              elsif (/^\s*\*?\s*(.+)$/)
-                                 {
-                                    $headline .= " $1";
-                                 }
-                           }
-                        else
-                           {
-                              if (/^\s*\*?\s*$/)
-                                 {
-                                    $detailed .= " </P><P>";
-                                 }
-                              elsif (/^\s*\*?\s*(.+)$/)
-                                 {
-                                    $detailed .= " $1";
-                                 }
-                           }
-                     }
+                  parse_comment( \$headline, \$detailed );
                }
          }
 
       print INTERFACE "    </TBODY>\n";
-      
+
       print INTERFACE "  </TABLE></CENTER>\n",
                       "</P>\n";
 
@@ -256,27 +342,29 @@ sub parse_interface (NAME)
 
 #
 # Reads stdin until the end of the enum is reached.
-# Writes formatted HTML to stdout.
+# Writes formatted HTML to "types.html".
 #
 sub parse_enum
    {
       %entries = ();
       @list = ();
 
+      $pre = "";
+
       while (<>)
          {
             chomp;
-                                                  
+
             $entry = "";
 
-            # complete one line entry with assignment
-            if ( /^\s*(\w+)\s*=\s*([\w\d]+)\s*,?\s*\/\*\s*(.+)\*\/\s*$/ )
+            # entry with assignment (complete comment)
+            if ( /^\s*(\w+)\s*=\s*([\w\d\(\)\,\!\s]+)\s*,?\s*\/\*\s*(.+)\s*\*\/\s*$/ )
                {
                   $entry = $1;
                   $entries{ $entry } = $3;
                }
-            # with comment opening
-            elsif ( /^\s*(\w+)\s*=\s*([\w\d]+)\s*,?\s*\/\*\s*(.+)\s*$/ )
+            # entry with assignment (opening comment)
+            elsif ( /^\s*(\w+)\s*=\s*([\w\d\(\)\,\!\s]+)\s*,?\s*\/\*\s*(.+)\s*$/ )
                {
                   $entry = $1;
 
@@ -286,7 +374,7 @@ sub parse_enum
                      {
                         chomp;
 
-                        if ( /^\s*(.+)\*\/\s*$/ )
+                        if ( /^\s*(.+)\s*\*\/\s*$/ )
                            {
                               $entries{ $entry } .= " $1";
                               last;
@@ -297,13 +385,19 @@ sub parse_enum
                            }
                   }
                }
-            # complete one line entry
-            elsif ( /^\s*(\w+)\s*,?\s*\/\*\s*(.+)\*\/\s*$/ )
+            # entry with assignment (none or preceding comment)
+            elsif ( /^\s*(\w+)\s*=\s*([\w\d\(\)\,\!\s]+)\s*,?\s*$/ )
+               {
+                  $entry = $1;
+                  $entries{ $entry } = $pre;
+               }
+            # entry without assignment (complete comment)
+            elsif ( /^\s*(\w+)\s*,?\s*\/\*\s*(.+)\s*\*\/\s*$/ )
                {
                   $entry = $1;
                   $entries{ $entry } = $2;
                }
-            # with comment opening
+            # entry without assignment (opening comment)
             elsif ( /^\s*(\w+)\s*,?\s*\/\*\s*(.+)\s*$/ )
                {
                   $entry = $1;
@@ -314,7 +408,7 @@ sub parse_enum
                      {
                         chomp;
 
-                        if ( /^\s*(.+)\*\/\s*$/ )
+                        if ( /^\s*(.+)\s*\*\/\s*$/ )
                            {
                               $entries{ $entry } .= " $1";
                               last;
@@ -325,14 +419,51 @@ sub parse_enum
                            }
                      }
                }
+            # entry without assignment (none or preceding comment)
+            elsif ( /^\s*(\w+)\s*,?\s*$/ )
+               {
+                  $entry = $1;
+                  $entries{ $entry } = $pre;
+               }
+            # preceding comment (complete)
+            elsif ( /^\s*\/\*\s*(.+)\s*\*\/\s*$/ )
+               {
+                  $pre = $1;
+               }
+            # preceding comment (opening)
+            elsif ( /^\s*\/\*\s*(.+)\s*$/ )
+               {
+                  $pre = $1;
+
+                  while (<>)
+                     {
+                        chomp;
+
+                        if ( /^\s*(.+)\s*\*\/\s*$/ )
+                           {
+                              $pre .= " $1";
+                              last;
+                           }
+                        elsif ( /^\s*(.+)\s*$/ )
+                           {
+                              $pre .= " $1";
+                           }
+                     }
+               }
+            # end of enum
             elsif ( /^\s*\}\s*(\w+)\s*\;\s*$/ )
                {
                   $enum = $1;
 
-                  $types{$enum} = 1;
+                  $types{$enum} = $comment;
 
                   last;
                }
+            # blank line?
+            else
+               {
+                  $pre = "";
+               }
 
             if ($entry ne "")
                {
@@ -342,26 +473,37 @@ sub parse_enum
 
       if (scalar @list > 0)
          {
-            print TYPES "<br><br>",
-                        "<a name=$enum>",
-                        "<font color=#D07070 size=+2>$enum</font>\n";
+            print TYPES "<p>\n",
+                        "  <a name=$enum>\n",
+                        "  <font color=#D07070 size=+1>$enum</font>\n";
 
-            print TYPES "$comment";
+            print TYPES "  <br>\n",
+                        "  <TABLE border=0 cellspacing=0 cellpadding=4 bgcolor=#F0F0F0>\n";
 
             foreach $key (@list)
                {
-                  print TYPES "<p style=\"margin-left:6mm;\" >",
-                              "  <font color=#40A040 size=+1>$key</font><br>",
-                              "  <font color=#404040>$entries{$key}</font>",
-                              "</p>\n";
+                  substitute_method_links (\$entries{$key});
+
+                  print TYPES "    <TR><TD width=32>&nbsp;</TD><TD valign=top>\n",
+                              "      <font color=#40A040>$key</font>\n",
+                              "    </TD><TD valign=top>\n",
+                              "      <font color=#404040>$entries{$key}</font>\n",
+                              "    </TD></TR>\n";
                }
+
+            substitute_method_links (\$comment);
+
+            print TYPES "  </TABLE>\n",
+                        "  $comment\n",
+                        "  <br>\n",
+                        "</p>\n";
          }
    }
 
 
 #
 # Reads stdin until the end of the enum is reached.
-# Writes formatted HTML to stdout.
+# Writes formatted HTML to "types.html".
 #
 sub parse_struct
    {
@@ -466,7 +608,7 @@ sub parse_struct
                {
                   $struct = $1;
 
-                  $types{$struct} = 1;
+                  $types{$struct} = $comment;
 
                   last;
                }
@@ -474,16 +616,18 @@ sub parse_struct
 
       if (scalar @entries > 0)
          {
-            print TYPES "<br><br>",
-                        "<a name=$struct>",
-                        "<font color=#70D070 size=+2>$struct</font>\n";
+            print TYPES "<p>",
+                        "  <a name=$struct>",
+                        "  <font color=#70D070 size=+1>$struct</font>\n";
 
-            print TYPES "<P>\n",
+            print TYPES "  <br>\n",
                         "  <TABLE border=0 cellspacing=0 cellpadding=4 bgcolor=#E0E0E0>\n";
 
             foreach $key (@entries)
                {
-                  print TYPES "    <TR><TD width=50>&nbsp;</TD><TD valign=top>\n",
+                  substitute_method_links (\$entries_params{$key});
+
+                  print TYPES "    <TR><TD width=32>&nbsp;</TD><TD valign=top>\n",
                               "      $entries_types{$key}\n",
                               "    </TD><TD valign=top>\n",
                               "      <FONT color=black><B>$key</B></FONT>\n",
@@ -492,14 +636,190 @@ sub parse_struct
                               "    </TD></TR>\n";
                }
 
+            substitute_method_links (\$comment);
+
             print TYPES "  </TABLE>\n",
-                        "</P>\n";
+                        "  $comment\n",
+                        "  <br>\n",
+                        "</p>\n";
 
-            print TYPES "$comment";
          }
    }
 
 
+sub parse_func (TYPE, NAME)
+   {
+      my $rtype = shift(@_);
+      my $name  = shift(@_);
+
+      @entries = ();
+      %entries_params = ();
+      %entries_types = ();
+
+      while (<>)
+         {
+            chomp;
+
+            # without comment
+            if ( /^\s*([\w\ ]+)\s+(\**[\w\d\+\[\]]+,?)\s*$/ )
+               {
+                  $type = $1;
+                  $entry = $2;
+
+                  $type =~ s/\ *$//;
+
+                  if ($types{$type})
+                     {
+                        $entries_types{$entry} = "<A href=\"types.html#$type\">$type</A>";
+                     }
+                  elsif ($interface_abstracts{$type})
+                     {
+                        $entries_types{$entry} = "<A href=\"$type.html\">$type</A>";
+                     }
+                  else
+                     {
+                        $entries_types{$entry} = "$type";
+                     }
+
+                  push (@entries, $entry);
+                  $entries_params{ $entry } = "";
+               }
+            # complete one line entry
+            elsif ( /^\s*([\w\ ]+)\s+(\**[\w\d\+\[\]]+,?)\s*\/\*\s*(.+)\*\/\s*$/ )
+               {
+                  $type = $1;
+                  $entry = $2;
+                  $text = $3;
+
+                  $type =~ s/\ *$//;
+
+                  if ($types{$type})
+                     {
+                        $entries_types{$entry} = "<A href=\"types.html#$type\">$type</A>";
+                     }
+                  elsif ($interface_abstracts{$type})
+                     {
+                        $entries_types{$entry} = "<A href=\"$type.html\">$type</A>";
+                     }
+                  else
+                     {
+                        $entries_types{$entry} = "$type";
+                     }
+
+                  push (@entries, $entry);
+                  $entries_params{ $entry } = $text;
+               }
+            # with comment opening
+            elsif ( /^\s*([\w\ ]+)\s+(\**[\w\d\+\[\]]+,?)\s*\/\*\s*(.+)\s*$/ )
+               {
+                  $type = $1;
+                  $entry = $2;
+                  $text = $3;
+
+                  $type =~ s/\ *$//;
+
+                  if ($types{$type})
+                     {
+                        $entries_types{$entry} = "<A href=\"types.html#$type\">$type</A>";
+                     }
+                  elsif ($interface_abstracts{$type})
+                     {
+                        $entries_types{$entry} = "<A href=\"$type.html\">$type</A>";
+                     }
+                  else
+                     {
+                        $entries_types{$entry} = "$type";
+                     }
+
+                  push (@entries, $entry);
+                  $entries_params{ $entry } = $text;
+
+                  while (<>)
+                     {
+                        chomp;
+
+                        if ( /^\s*(.+)\*\/\s*$/ )
+                           {
+                              $entries_params{ $entry } .= " $1";
+                              last;
+                           }
+                        elsif ( /^\s*(.+)\s*$/ )
+                           {
+                              $entries_params{ $entry } .= " $1";
+                           }
+                     }
+               }
+            elsif ( /^\s*\)\;\s*$/ )
+               {
+                  $types{$name} = $comment;
+
+                  last;
+               }
+         }
+
+      if ($types{$rtype})
+         {
+            $rtype = "<A href=\"types.html#$rtype\">$rtype</A>";
+         }
+
+      if (scalar @entries > 0)
+         {
+            print TYPES "<p>",
+                        "  <a name=$name>",
+                        "  <font color=#40A0F0 size=+1>$name</font>\n",
+                        "  <br>\n",
+                        "  <TABLE border=0 cellspacing=0 cellpadding=4 bgcolor=#E0E0E0>\n",
+                        "    <TR><TD colspan=4>\n",
+                        "         <I><FONT color=black>$rtype (*$name) (</FONT></I>\n",
+                        "    </TD></TR>\n";
+
+            foreach $key (@entries)
+               {
+                  print TYPES "    <TR><TD width=32>&nbsp;</TD><TD valign=top>\n",
+                              "      $entries_types{$key}\n",
+                              "    </TD><TD valign=top>\n",
+                              "      <FONT color=black><B>$key</B></FONT>\n",
+                              "    </TD><TD valign=top>\n",
+                              "      <font color=#404040>$entries_params{$key}</font>\n",
+                              "    </TD></TR>\n";
+               }
+
+            print TYPES "    <TR><TD colspan=3><I><FONT color=black>);</FONT></I></TD></TR>\n",
+                        "  </TABLE>\n",
+                        "  $comment\n",
+                        "</p>\n";
+
+         }
+   }
+
+#
+# Reads stdin until the end of the macro is reached.
+# Writes formatted HTML to "types.html".
+# Parameter is the macro name.
+#
+sub parse_macro (NAME, PARAMS)
+   {
+      my $macro  = shift(@_);
+      my $params = shift(@_);
+
+      while (<>)
+         {
+            chomp;
+
+            last unless /\\$/;
+         }
+
+      $definitions{"$macro $params"} = $comment;
+
+      print TYPES "<p>\n",
+                  "  <a name=\"$macro $params\">\n",
+                  "  <font color=#7070D0 size=+1>$macro $params</font>\n",
+                  "  <br>\n",
+                  "  $comment\n",
+                  "  <br>\n",
+                  "</p>\n";
+   }
+
 
 sub html_create (FILEHANDLE, FILENAME, TITLE, SUBTITLE)
    {
@@ -517,6 +837,9 @@ sub html_create (FILEHANDLE, FILENAME, TITLE, SUBTITLE)
                   "    A{textdecoration:none}\n",
                   "  -->\n",
                   "</STYLE>\n",
+                  "<STYLE type=\"text/css\">\n",
+                  "  A:link, A:visited, A:active { text-decoration: none; }\n",
+                  "</STYLE>\n",
                   "<HEAD>\n",
                   "  <TITLE>DirectFB Reference Manual</TITLE>\n",
                   "</HEAD>\n",
@@ -535,9 +858,9 @@ sub html_create (FILEHANDLE, FILENAME, TITLE, SUBTITLE)
       if ($subtitle)
          {
             print $FILE "    <TABLE border=0 cellspacing=0 cellpadding=0>\n",
-                        "      <TR><TD align=right width=50%>\n",
+                        "      <TR><TD nowrap align=right width=50%>\n",
                         "        <BIG>$title&nbsp;</BIG>\n",
-                        "      </TD><TD align=left width=50%>\n",
+                        "      </TD><TD nowrap align=left width=50%>\n",
                         "        <BIG><FONT color=orange>&nbsp;$subtitle</FONT></BIG>\n",
                         "      </TD></TR>\n",
                         "    </TABLE>\n";
@@ -557,16 +880,6 @@ sub html_close (FILEHANDLE)
       my $FILE = shift(@_);
 
       print $FILE "\n",
-                  "<TABLE width=100% bgcolor=black border=0 cellspacing=0 cellpadding=0>\n",
-                  "  <TR><TD valign=center>\n",
-                  "    <FONT size=-3>&nbsp;&nbsp;(C) Copyright by convergence GmbH</FONT>\n",
-                  "  </TD><TD align=right>\n",
-                  "    <A href=\"http://www.convergence.de\">\n",
-                  "      <IMG border=0 hspace=4 src=\"cimlogo.png\">\n",
-                  "    </A>\n",
-                  "  </TD></TR>\n",
-                  "</TABLE>\n",
-                  "\n",
                   "</BODY>\n",
                   "</HTML>\n";
 
