@@ -81,6 +81,8 @@ typedef struct {
 
 /**************************************************************************************************/
 
+#define MAX_KEYS 16
+
 typedef struct {
      CoreDFB                      *core;
 } WMData;
@@ -113,7 +115,7 @@ typedef struct {
           DFBInputDeviceKeyIdentifier  id;
           int                          code;
           CoreWindow                  *owner;
-     } keys[8];
+     } keys[MAX_KEYS];
 } StackData;
 
 typedef struct {
@@ -258,7 +260,7 @@ get_keyboard_window( CoreWindowStack     *stack,
           CoreWindow *window;
 
           /* Check active grabs. */
-          for (i=0; i<8; i++) {
+          for (i=0; i<MAX_KEYS; i++) {
                /* Key is grabbed, send to owner (NULL if destroyed). */
                if (data->keys[i].code == event->key_code)
                     return data->keys[i].owner;
@@ -292,7 +294,7 @@ get_keyboard_window( CoreWindowStack     *stack,
           int i;
 
           /* Lookup owner and ungrab the key. */
-          for (i=0; i<8; i++) {
+          for (i=0; i<MAX_KEYS; i++) {
                if (data->keys[i].code == event->key_code) {
                     data->keys[i].code = -1;
 
@@ -390,6 +392,7 @@ window_at_pointer( CoreWindowStack *stack,
                                              break;
                                         }
 
+
                                         /* fall through */
                                    }
 
@@ -405,7 +408,7 @@ window_at_pointer( CoreWindowStack *stack,
                          }
                          if (options & DWOP_COLORKEYING) {
                               int pixel = 0;
-
+                              __u8 *p;
                               switch (surface->format) {
                                    case DSPF_ARGB:
                                    case DSPF_AiRGB:
@@ -415,10 +418,13 @@ window_at_pointer( CoreWindowStack *stack,
                                                 & 0x00ffffff;
                                         break;
 
-                                   case DSPF_RGB24:   //endianess? boh...
-                                        pixel = (*(__u32*)(data +
-                                                           3 * wx + pitch * wy))
-                                                & 0x00ffffff;
+                                   case DSPF_RGB24:
+                                        p = (data + 3 * wx + pitch * wy);
+#ifdef WORDS_BIGENDIAN
+                                        pixel = (p[0] << 16) | (p[1] << 8) | p[2];
+#else
+                                        pixel = (p[2] << 16) | (p[1] << 8) | p[0];
+#endif
                                         break;
 
                                    case DSPF_RGB16:
@@ -623,6 +629,69 @@ draw_window( CoreWindow *window, CardState *state,
      if (window->surface->caps & DSCAPS_INTERLACED)
           flags |= DSBLIT_DEINTERLACE;
 
+     /* Different compositing methods depending on destination format. */
+     if (flags & DSBLIT_BLEND_ALPHACHANNEL) {
+          if (DFB_PIXELFORMAT_HAS_ALPHA( state->destination->format )) {
+               /*
+                * Always use compliant Porter/Duff SRC_OVER,
+                * if the destination has an alpha channel.
+                *
+                * Cd = destination color  (non-premultiplied)
+                * Ad = destination alpha
+                *
+                * Cs = source color       (non-premultiplied)
+                * As = source alpha
+                *
+                * Ac = color alpha
+                *
+                * cd = Cd * Ad            (premultiply destination)
+                * cs = Cs * As            (premultiply source)
+                *
+                * The full equation to calculate resulting color and alpha (premultiplied):
+                *
+                * cx = cd * (1-As*Ac) + cs * Ac
+                * ax = Ad * (1-As*Ac) + As * Ac
+                */
+               dfb_state_set_src_blend( state, DSBF_ONE );
+
+               /* Need to premultiply source with As*Ac or only with Ac? */
+               if (! (window->surface->caps & DSCAPS_PREMULTIPLIED))
+                    flags |= DSBLIT_SRC_PREMULTIPLY;
+               else if (flags & DSBLIT_BLEND_COLORALPHA)
+                    flags |= DSBLIT_SRC_PREMULTCOLOR;
+
+               /* Need to premultiply/demultiply destination? */
+//               if (! (state->destination->caps & DSCAPS_PREMULTIPLIED))
+//                    flags |= DSBLIT_DST_PREMULTIPLY | DSBLIT_DEMULTIPLY;
+          }
+          else {
+               /*
+                * We can avoid DSBLIT_SRC_PREMULTIPLY for destinations without an alpha channel
+                * by using another blending function, which is more likely that it's accelerated
+                * than premultiplication at this point in time.
+                *
+                * This way the resulting alpha (ax) doesn't comply with SRC_OVER,
+                * but as the destination doesn't have an alpha channel it's no problem.
+                *
+                * As the destination's alpha value is always 1.0 there's no need for
+                * premultiplication. The resulting alpha value will also be 1.0 without
+                * exceptions, therefore no need for demultiplication.
+                *
+                * cx = Cd * (1-As*Ac) + Cs*As * Ac  (still same effect as above)
+                * ax = Ad * (1-As*Ac) + As*As * Ac  (wrong, but discarded anyways)
+                */
+               if (window->surface->caps & DSCAPS_PREMULTIPLIED) {
+                    /* Need to premultiply source with Ac? */
+                    if (flags & DSBLIT_BLEND_COLORALPHA)
+                         flags |= DSBLIT_SRC_PREMULTCOLOR;
+
+                    dfb_state_set_src_blend( state, DSBF_ONE );
+               }
+               else
+                    dfb_state_set_src_blend( state, DSBF_SRCALPHA );
+          }
+     }
+
      /* Set blitting flags. */
      dfb_state_set_blitting_flags( state, flags );
 
@@ -660,7 +729,7 @@ draw_background( CoreWindowStack *stack, CardState *state, DFBRegion *region )
 
                /* Set the background color. */
                if (DFB_PIXELFORMAT_IS_INDEXED( dest->format ))
-                    dfb_state_set_color_index( state,
+                    dfb_state_set_color_index( state,  /* FIXME: don't search every time */
                                                dfb_palette_search( dest->palette, color->r,
                                                                    color->g, color->b, color->a ) );
                else
@@ -1204,7 +1273,7 @@ withdraw_window( CoreWindowStack *stack,
           data->pointer_window = NULL;
 
      /* Release all implicit key grabs. */
-     for (i=0; i<8; i++) {
+     for (i=0; i<MAX_KEYS; i++) {
           if (data->keys[i].code != -1 && data->keys[i].owner == window) {
                if (!DFB_WINDOW_DESTROYED( window )) {
                     DFBWindowEvent we;
@@ -1660,7 +1729,7 @@ grab_key( CoreWindow                 *window,
      direct_list_append( &data->grabbed_keys, &grab->link );
 
      /* Remove implicit grabs for this key. */
-     for (i=0; i<8; i++)
+     for (i=0; i<MAX_KEYS; i++)
           if (data->keys[i].code != -1 && data->keys[i].symbol == symbol)
                data->keys[i].code = -1;
 
@@ -2319,7 +2388,7 @@ wm_init_stack( CoreWindowStack *stack,
 
      fusion_vector_init( &data->windows, 64 );
 
-     for (i=0; i<8; i++)
+     for (i=0; i<MAX_KEYS; i++)
           data->keys[i].code = -1;
 
      D_MAGIC_SET( data, StackData );
@@ -2471,7 +2540,7 @@ wm_flush_keys( CoreWindowStack *stack,
 
      D_MAGIC_ASSERT( data, StackData );
 
-     for (i=0; i<8; i++) {
+     for (i=0; i<MAX_KEYS; i++) {
           if (data->keys[i].code != -1) {
                DFBWindowEvent we;
 
